/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.ejb;

import com.livis.flabes.util.*;
import com.livis.livingtalks.model.*;
import java.rmi.*;
import java.util.*;

public class Convenience {

    /* @return All type keys whose "editorsInDutyList" contains user */
    public Collection getPermittedTypeKeys (ManageEJB mgr, String user) {
	Collection models = mgr.getTypeShortmodelsAll ();
	Collection keys   = new ArrayList (models.size ());
	for (Iterator i = models.iterator (); i.hasNext (); ) {
            TypeModel model = (TypeModel) i.next ();
 	    if (model.isEditorInDuty (user)) keys.add (model.getKey ());
        }
	return keys;
    }

    /* @return The author keys where no talk lists the authors (ie, without any Authorship association) */
    public Collection getDanglingAuthorKeys (ManageEJB mgr) {

	// put all authors into the set, keyed by their primary key
        SortedSet result = new TreeSet (mgr.getAuthorKeysAll ());

	// remove all authors which are found in authorship mappings
	for (Iterator i = mgr.getAuthorshipShortmodelsAll ().iterator (); i.hasNext (); )
	    result.remove (((AuthorshipModel) i.next ()).getAuthorKey ());

	// WORKAROUND: just returning result may lead to
	//    java.lang.ClassCastException: java.util.HashMap$2
	// on client side (as of JDK-1.2.2, J2EE-1.2.1) 8(
	return new ArrayList (result);
    }

    /* @return The talk keys of talks which are not yet in any block */
    public Collection getDanglingTalkKeys (ManageEJB mgr) {
        SortedSet result = new TreeSet (mgr.getTalkKeysAll ());
	for (Iterator i = mgr.getBlockShortmodelsAll ().iterator (); i.hasNext (); ) {
	    BlockModel block = (BlockModel) i.next ();
	    for (Iterator t = block.getTalks ().iterator (); t.hasNext (); )
		result.remove (t.next ());
        }
	return new ArrayList (result);
    }

    /* @return The block keys of blocks which are not yet scheduled, ie. within no pitem */
    public Collection getDanglingBlockKeys (ManageEJB mgr) {
        SortedSet result = new TreeSet (mgr.getBlockKeysAll ());
	for (Iterator i = mgr.getPitemShortmodelsAll ().iterator (); i.hasNext (); )
	    result.remove (((PitemModel) i.next ()).getBlockKey ());
	return new ArrayList (result);
    }
}
