/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.ejb;

import com.livis.livingtalks.model.*;
import com.livis.flabes.util.*;
import java.sql.Date;
import java.util.Collection;
import java.rmi.*;
import javax.ejb.*;


public class ManageEJB implements ManageIF, SessionBean {

    // Coded bean names here must not be needed elsewhere (this enables caching)
    private CachingAuthorHandler     authors      = new CachingAuthorHandler     ( "ejb/author"     );
    private CachingAuthorshipHandler authorships  = new CachingAuthorshipHandler ( "ejb/authorship" );
    private CachingBlockHandler      blocks       = new CachingBlockHandler      ( "ejb/block"      );
    private CachingPitemHandler      pitems       = new CachingPitemHandler      ( "ejb/pitem"      );
    private CachingTalkHandler       talks        = new CachingTalkHandler       ( "ejb/talk"       );
    private CachingTrackHandler      tracks       = new CachingTrackHandler      ( "ejb/track"      );
    private CachingTypeHandler       types        = new CachingTypeHandler       ( "ejb/type"       );
    private Convenience              conveniences = new Convenience              ();

    // Implements interface ManageIF:

    // author
    public AuthorModel     getAuthorModel                (String key)                                    { return (AuthorModel) authors.getModel (key); }
    public AuthorModel     getAuthorShortmodel           (String key)                                    { return (AuthorModel) authors.getShortmodel (key); }
    public void            setAuthorModel                (AuthorModel model)      throws CreateException { authors.setModel (model); }
    public void            removeAuthor                  (String key)             throws RemoveException { authors.remove (key); }
    public String          createAuthor                  (AuthorModel model)      throws CreateException { return authors.create (model); }
    public Collection      getAuthorShortmodelsAll       ()                                              { return authors.getShortmodelsAll (); }
    public Collection      getAuthorKeysAll              ()                                              { return authors.getKeysAll (); }

    // authorship
    public AuthorshipModel getAuthorshipModel            (String key)                                    { return (AuthorshipModel) authorships.getModel (key); }
    public AuthorshipModel getAuthorshipShortmodel       (String key)                                    { return (AuthorshipModel) authorships.getShortmodel (key); }
    public void            setAuthorshipModel            (AuthorshipModel model)  throws CreateException { authorships.setModel (model); }
    public void            removeAuthorship              (String key)             throws RemoveException { authorships.remove (key); }
    public String          createAuthorship              (AuthorshipModel model)  throws CreateException { return authorships.create (model); }
    public Collection      getAuthorshipShortmodelsAll   ()                                              { return authorships.getShortmodelsAll (); }
    public Collection      getAuthorshipKeysAll          ()                                              { return authorships.getKeysAll (); }
    public Collection      getAuthorshipKeysByTalkKey    (String talkKey)                                { return authorships.getKeys1 (talkKey); }
    public Collection      getAuthorshipKeysByAuthorKey  (String authorKey)                              { return authorships.getKeys1a (authorKey); }

    // block
    public BlockModel      getBlockModel                 (String key)                                    { return (BlockModel) blocks.getModel (key); }
    public BlockModel      getBlockShortmodel            (String key)                                    { return (BlockModel) blocks.getShortmodel (key); }
    public void            setBlockModel                 (BlockModel model)       throws CreateException { blocks.setModel (model); }
    public void            removeBlock                   (String key)             throws RemoveException { blocks.remove (key); }
    public String          createBlock                   (BlockModel model)       throws CreateException { return blocks.create (model); }
    public Collection      getBlockShortmodelsAll        ()                                              { return blocks.getShortmodelsAll (); }
    public Collection      getBlockKeysAll               ()                                              { return blocks.getKeysAll (); }
    public Collection      getBlockKeysByTalkList        (String talkList)                               { return blocks.getKeys1 (talkList); }

    // pitem
    public PitemModel      getPitemModel                 (String key)                                    { return (PitemModel) pitems.getModel (key); }
    public PitemModel      getPitemShortmodel            (String key)                                    { return (PitemModel) pitems.getShortmodel (key); }
    public void            setPitemModel                 (PitemModel model)       throws CreateException { pitems.setModel (model); }
    public void            removePitem                   (String key)             throws RemoveException { pitems.remove (key); }
    public String          createPitem                   (PitemModel model)       throws CreateException { return pitems.create (model); }
    public Collection      getPitemShortmodelsAll        ()                                              { return pitems.getShortmodelsAll (); }
    public Collection      getPitemKeysAll               ()                                              { return pitems.getKeysAll (); }
    public Collection      getPitemKeysByBlockKey        (String blockKey)                               { return pitems.getKeys1 (blockKey); }
    public Collection      getPitemKeysByTrackKey        (String trackKey)                               { return pitems.getKeys1a (trackKey); }

    // talk
    public TalkModel       getTalkModel                  (String key)                                    { return (TalkModel) talks.getModel (key); }
    public TalkModel       getTalkShortmodel             (String key)                                    { return (TalkModel) talks.getShortmodel (key); }
    public void            setTalkModel                  (TalkModel model)        throws CreateException { try { talks.setModel (model); }
                                                                                                           finally { if (context.getRollbackOnly ()) authorships.invalidateCache (); } }
    public void            removeTalk                    (String key)             throws RemoveException { try { talks.remove (key); }
                                                                                                           finally { if (context.getRollbackOnly ()) authorships.invalidateCache (); } }
    public String          createTalk                    (TalkModel model)        throws CreateException { try { return talks.create (model); }
                                                                                                           finally { if (context.getRollbackOnly ()) authorships.invalidateCache (); } }
    public Collection      getTalkShortmodelsAll         ()                                              { return talks.getShortmodelsAll (); }
    public Collection      getTalkKeysAll                ()                                              { return talks.getKeysAll (); }
    public Collection      getTalkKeysByTypeKey          (String typeKey)                                { return talks.getKeys1 (typeKey); }

    // track
    public TrackModel      getTrackModel                 (String key)                                    { return (TrackModel) tracks.getModel (key); }
    public TrackModel      getTrackShortmodel            (String key)                                    { return (TrackModel) tracks.getShortmodel (key); }
    public void            setTrackModel                 (TrackModel model)       throws CreateException { tracks.setModel (model); }
    public void            removeTrack                   (String key)             throws RemoveException { tracks.remove (key); }
    public String          createTrack                   (TrackModel model)       throws CreateException { return tracks.create (model); }
    public Collection      getTrackShortmodelsAll        ()                                              { return tracks.getShortmodelsAll (); }
    public Collection      getTrackKeysAll               ()                                              { return tracks.getKeysAll (); }
    public Collection      getTrackKeysByPage       	 (int page)  	                                 { return tracks.getKeys1 (new Integer (page)); }
    public Collection      getTrackKeysByPageAndTrackCol (int page, int trackCol)                        { return tracks.getKeys2 (new Integer (page), new Integer (trackCol)); }
    public Collection      getTrackKeysByDate            (Date date)                                     { return tracks.getKeys1a (date); }

    // type
    public TypeModel       getTypeModel                  (String key)                                    { return (TypeModel) types.getModel (key); }
    public TypeModel       getTypeShortmodel             (String key)                                    { return (TypeModel) types.getShortmodel (key); }
    public void            setTypeModel                  (TypeModel model)        throws CreateException { types.setModel (model); }
    public void            removeType                    (String key)             throws RemoveException { types.remove (key); }
    public String          createType                    (TypeModel model)        throws CreateException { return types.create (model); }
    public Collection      getTypeShortmodelsAll         ()                                              { return types.getShortmodelsAll (); }
    public Collection      getTypeKeysAll                ()                                              { return types.getKeysAll (); }

    // conveniences
    public Collection      getPermittedTypeKeys          ()                                              { return conveniences.getPermittedTypeKeys  (this, context.getCallerPrincipal ().getName ()); }
    public Collection      getDanglingAuthorKeys         ()                                              { return conveniences.getDanglingAuthorKeys (this); }
    public Collection      getDanglingTalkKeys           ()                                              { return conveniences.getDanglingTalkKeys   (this); }
    public Collection      getDanglingBlockKeys          ()                                              { return conveniences.getDanglingBlockKeys  (this); }
    public void            invalidate                    ()                                              {
      authors.invalidateCache ();
      authorships.invalidateCache ();
      blocks.invalidateCache ();
      pitems.invalidateCache ();
      talks.invalidateCache ();
      tracks.invalidateCache ();
      types.invalidateCache ();
    }

    // Implements interface SessionBean:
    public void ejbCreate    () throws CreateException  { }
    public void ejbActivate  ()                         { }
    public void ejbPassivate ()                         { }
    public void ejbRemove    ()                         { }

    private SessionContext context;
    public void            setSessionContext (SessionContext context) { this.context = context; }
    public SessionContext  getSessionContext ()                       { return context; }
}
