/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.ejb;

import com.livis.livingtalks.model.*;
import java.sql.Date;
import java.util.Collection;
import java.rmi.*;
import javax.ejb.*;

public interface ManageIF {

    // author
    public AuthorModel     getAuthorModel                (String key)             throws RemoteException;
    public AuthorModel     getAuthorShortmodel           (String key)             throws RemoteException;
    public void            setAuthorModel                (AuthorModel model)      throws CreateException, RemoteException;
    public void            removeAuthor                  (String key)             throws RemoveException, RemoteException;
    public String          createAuthor                  (AuthorModel model)      throws CreateException, RemoteException;
    public Collection      getAuthorShortmodelsAll       ()                       throws RemoteException;
    public Collection      getAuthorKeysAll              ()                       throws RemoteException;

    // authorship
    public AuthorshipModel getAuthorshipModel            (String key)             throws RemoteException;
    public AuthorshipModel getAuthorshipShortmodel       (String key)             throws RemoteException;
    public void            setAuthorshipModel            (AuthorshipModel model)  throws CreateException, RemoteException;
    public void            removeAuthorship              (String key)             throws RemoveException, RemoteException;
    public String          createAuthorship              (AuthorshipModel model)  throws CreateException, RemoteException;
    public Collection      getAuthorshipShortmodelsAll   ()                       throws RemoteException;
    public Collection      getAuthorshipKeysAll          ()                       throws RemoteException;
    public Collection      getAuthorshipKeysByTalkKey    (String talkKey)         throws RemoteException;
    public Collection      getAuthorshipKeysByAuthorKey  (String authorKey)       throws RemoteException;

    // block
    public BlockModel      getBlockModel                 (String key)             throws RemoteException;
    public BlockModel      getBlockShortmodel            (String key)             throws RemoteException;
    public void            setBlockModel                 (BlockModel model)       throws CreateException, RemoteException;
    public void            removeBlock                   (String key)             throws RemoveException, RemoteException;
    public String          createBlock                   (BlockModel model)       throws CreateException, RemoteException;
    public Collection      getBlockShortmodelsAll        ()                       throws RemoteException;
    public Collection      getBlockKeysAll               ()                       throws RemoteException;
    public Collection      getBlockKeysByTalkList        (String talkList)        throws RemoteException;

    // pitem
    public PitemModel      getPitemModel                 (String key)             throws RemoteException;
    public PitemModel      getPitemShortmodel            (String key)             throws RemoteException;
    public void            setPitemModel                 (PitemModel model)       throws CreateException, RemoteException;
    public void            removePitem                   (String key)             throws RemoveException, RemoteException;
    public String          createPitem                   (PitemModel model)       throws CreateException, RemoteException;
    public Collection      getPitemShortmodelsAll        ()                       throws RemoteException;
    public Collection      getPitemKeysAll               ()                       throws RemoteException;
    public Collection      getPitemKeysByBlockKey        (String blockKey)        throws RemoteException;
    public Collection      getPitemKeysByTrackKey        (String trackKey)        throws RemoteException;

    // talk
    public TalkModel       getTalkModel                  (String key)             throws RemoteException;
    public TalkModel       getTalkShortmodel             (String key)             throws RemoteException;
    public void            setTalkModel                  (TalkModel model)        throws CreateException, RemoteException;
    public void            removeTalk                    (String key)             throws RemoveException, RemoteException;
    public String          createTalk                    (TalkModel model)        throws CreateException, RemoteException;
    public Collection      getTalkShortmodelsAll         ()                       throws RemoteException;
    public Collection      getTalkKeysAll                ()                       throws RemoteException;
    public Collection      getTalkKeysByTypeKey          (String typeKey)         throws RemoteException;

    // track
    public TrackModel      getTrackModel                 (String key)             throws RemoteException;
    public TrackModel      getTrackShortmodel            (String key)             throws RemoteException;
    public void            setTrackModel                 (TrackModel model)       throws CreateException, RemoteException;
    public void            removeTrack                   (String key)             throws RemoveException, RemoteException;
    public String          createTrack                   (TrackModel model)       throws CreateException, RemoteException;
    public Collection      getTrackShortmodelsAll        ()                       throws RemoteException;
    public Collection      getTrackKeysAll               ()                       throws RemoteException;
    public Collection      getTrackKeysByPage       	 (int page)  	          throws RemoteException;
    public Collection      getTrackKeysByPageAndTrackCol (int page, int trackCol) throws RemoteException;
    public Collection      getTrackKeysByDate            (Date date)              throws RemoteException;

    // type
    public TypeModel       getTypeModel                  (String key)             throws RemoteException;
    public TypeModel       getTypeShortmodel             (String key)             throws RemoteException;
    public void            setTypeModel                  (TypeModel model)        throws CreateException, RemoteException;
    public void            removeType                    (String key)             throws RemoveException, RemoteException;
    public String          createType                    (TypeModel model)        throws CreateException, RemoteException;
    public Collection      getTypeShortmodelsAll         ()                       throws RemoteException;
    public Collection      getTypeKeysAll                ()                       throws RemoteException;

    // conveniences
    public Collection      getPermittedTypeKeys          ()                       throws RemoteException;
    public Collection      getDanglingAuthorKeys         ()                       throws RemoteException;
    public Collection      getDanglingTalkKeys           ()                       throws RemoteException;
    public Collection      getDanglingBlockKeys          ()                       throws RemoteException;
    public void            invalidate                    ()                       throws RemoteException;
}