/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.ejb;

import com.livis.livingtalks.model.*;
import com.livis.livingtalks.xml.*;
import javax.ejb.*;
import java.rmi.RemoteException;

public class PublishEJB implements PublishIF, SessionBean, SessionSynchronization {

    /** USERMAXOPS is the number of records a user other than the master may import at once */
    public static final int USERMAXOPS = 70;

    // Implements interface PublishIF:

    // can't name it import since that's a keyword
    public ImportResult importXML (String inSpec, String conference, int mode) {
        return doImportXML (inSpec, conference, mode, false);
    }
    public ImportResult importXMLinOneTx (String inSpec, String conference, int mode) {
        return doImportXML (inSpec, conference, mode, true);
    }
    public void exportXML (String outSpec) {
        new Export   ().write (outSpec);
    }
    public void authors   (String outSpec,                        String fromSpec, String[] contextSpecs) {
        new Authors  (               fromSpec, contextSpecs).write (outSpec);
    }
    public void talks     (String outSpec,                        String fromSpec, String[] contextSpecs) {
        new Talks    (               fromSpec, contextSpecs).write (outSpec);
    }
    public void agenda    (String outSpec, boolean printAbstract, String fromSpec, String[] contextSpecs) {
        new Agenda   (printAbstract, fromSpec, contextSpecs).write (outSpec);
    }
    public void overview  (String outSpec,                        String fromSpec, String[] contextSpecs) {
        new Overview (               fromSpec, contextSpecs).write (outSpec);
    }

    private ImportResult doImportXML (String inSpec, String conference, int mode, boolean oneTx) {
        try {
            int maxops = Integer.MAX_VALUE;
            if (!context.isCallerInRole ("master")) maxops = USERMAXOPS;
            ImportResult result = new Import (conference, mode, !oneTx, maxops).read (inSpec);
            LTUtils.createManage ().invalidate ();
            return result;
        } catch (Exception ex) {
            try { LTUtils.createManage ().invalidate (); } catch (RemoteException ignore) {}
            if (oneTx) context.setRollbackOnly ();
            LTUtils.handleException(ex, this, null, null); return null;
        }
    }

    // Implements interface SessionBean:
    public void ejbCreate    () throws CreateException  { }
    public void ejbActivate  () { }
    public void ejbPassivate () { }
    public void ejbRemove    () { }

    private SessionContext context;
    public void           setSessionContext (SessionContext context) { this.context = context; }
    public SessionContext getSessionContext ()                       { return context; }

    // Implements interface SessionSynchronization:
    public void afterBegin       ()                throws EJBException                  {}
    public void beforeCompletion ()                throws EJBException                  {}
    /** only called back from transactions, only importXML () should run in a transaction */
    public void afterCompletion  (boolean success) throws EJBException, RemoteException { if (!success) LTUtils.createManage ().invalidate (); }
}
