/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Oliver Imbusch<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.model;

import com.livis.flabes.util.*;
import java.util.*;

public class AuthorModel
    extends AbstractModel {

    //----------------------------------------------------------------------
    // constructors/assignment
    //----------------------------------------------------------------------
    private AuthorModel copy (AuthorModel model) {
	return assignAuthorModel (model.key, model.editor, model.timestamp,
				  model.lastName, model.firstName, model.title, model.cvHtml, model.affiliation, model.address, model.email, model.homePageUrl, model.photoUrl);
    }

    protected AuthorModel assignAuthorModel (String key, String editor, String timestamp,
					     String lastName, String firstName, String title, String cvHtml, String affiliation, String address, String email, String homePageUrl, String photoUrl) {

	super.assignAbstractModel (key, editor, timestamp);

	this.lastName    = lastName;
	this.firstName   = firstName;
	this.title       = title;
	this.cvHtml      = cvHtml;
	this.affiliation = affiliation;
	this.address     = address;
	this.email       = email;
	this.homePageUrl = homePageUrl;
	this.photoUrl    = photoUrl;

	return this;
    }

    public AuthorModel getModel ()                  { return (new AuthorModel ()).copy (this); }
    public void        setModel (AuthorModel model) { copy (model); }

    public void        init     ()                  { assignAuthorModel ("", "", "",
									 "", "", "", "", "", "", "", "", ""); }
    public void        clear    ()                  { assignAuthorModel ("", "", "",
									 "", "", "", "", "", "", "", "", ""); super.clear (); }

    //----------------------------------------------------------------------
    // CMP fields/properties
    //----------------------------------------------------------------------
    public String lastName;
    public String getLastName    ()                   { return lastName; }
    public void   setLastName    (String lastName)    { this.lastName = lastName;        }
    public String firstName;
    public String getFirstName   ()                   { return firstName; }
    public void   setFirstName   (String firstName)   { this.firstName = firstName; }
    public String title;
    public String getTitle       ()                   { return title; }
    public void   setTitle       (String title)       { this.title = title; }
    public String cvHtml;
    public String getCvHtml      ()                   { return cvHtml; }
    public void   setCvHtml      (String cvHtml)      { this.cvHtml = cvHtml; }
    public String affiliation;
    public String getAffiliation ()                   { return affiliation; }
    public void   setAffiliation (String affiliation) { this.affiliation = affiliation; }
    public String address;
    public String getAddress     ()                   { return address; }
    public void   setAddress     (String address)     { this.address = address; }
    public String email;
    public String getEmail       ()                   { return email; }
    public void   setEmail       (String email)       { this.email = email; }
    public String homePageUrl;
    public String getHomePageUrl ()                   { return homePageUrl; }
    public void   setHomePageUrl (String homePageUrl) { this.homePageUrl = homePageUrl; }
    public String photoUrl;
    public String getPhotoUrl    ()                   { return photoUrl; }
    public void   setPhotoUrl    (String photoUrl)    { this.photoUrl = photoUrl; }

    //----------------------------------------------------------------------
    // short model
    //----------------------------------------------------------------------
    public void makeShort () {
      setCvHtml  (null);
      setAddress (null);
    }

    //----------------------------------------------------------------------
    // key creation/check
    //----------------------------------------------------------------------
    public    boolean isKeyCreatable 	  () { return true; }
    protected String  createKey      	  () { setKey (getLastName () + ", " + getFirstName ().charAt (0)); return getKey (); }
    public    void    checkForKeyCreation () {
	if (StringUtils.isNullOrEmpty (getFirstName ())) {
	    throw new LivingTalksException ("The first name is mandatory!", this);
	}
	if (StringUtils.isNullOrEmpty (getLastName ())) {
	    throw new LivingTalksException ("The last name is mandatory!", this);
	}
    }

    //----------------------------------------------------------------------
    // XML
    //----------------------------------------------------------------------
    private static List xmlTagProperties = new ArrayList ();
    protected List xmlTagPropertiesStore () { return xmlTagProperties; }
}
