/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Oliver Imbusch<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.model;

import com.livis.flabes.util.*;
import java.util.*;

public class AuthorshipModel
    extends AbstractModel {

    //----------------------------------------------------------------------
    // constructors/assignment
    //----------------------------------------------------------------------
    public AuthorshipModel ()                                     { super (); setAutoKey ("<will be automatically generated - leave as is>"); }
    public AuthorshipModel (String talkKey, String authorKey)     { this (); assignAuthorshipModel (getAutoKey (), "", "", talkKey, authorKey); }

    private AuthorshipModel copy (AuthorshipModel model) {
	return assignAuthorshipModel (model.key, model.editor, model.timestamp,
				      model.talkKey, model.authorKey);
    }
    protected AuthorshipModel assignAuthorshipModel (String key, String editor, String timestamp,
						     String talkKey, String authorKey) {

	super.assignAbstractModel (key, editor, timestamp);

	this.talkKey   = talkKey;
	this.authorKey = authorKey;

	return this;
    }

    public AuthorshipModel getModel ()                      { return (new AuthorshipModel ()).copy (this); }
    public void            setModel (AuthorshipModel model) { copy (model); }

    public void            init     ()                      { assignAuthorshipModel ("", "", "",
										     "", ""); }
    public void            clear    ()                      { assignAuthorshipModel ("", "", "",
										     "", ""); super.clear (); }

    //----------------------------------------------------------------------
    // CMP fields/properties
    //----------------------------------------------------------------------
    public String talkKey;
    public String getTalkKey   ()                 { return talkKey; }
    public void   setTalkKey   (String talkKey)   { this.talkKey = talkKey; }
    public String authorKey;
    public String getAuthorKey ()                 { return authorKey; }
    public void   setAuthorKey (String authorKey) { this.authorKey = authorKey; }

    //----------------------------------------------------------------------
    // key creation/check
    //----------------------------------------------------------------------
    public    boolean isKeyCreatable 	  () { return true; }
    protected String  createKey      	  () { setKey (getTalkKey () + "#" + getAuthorKey ()); return getKey (); }
    public    void    checkForKeyCreation () {
	if (StringUtils.isNullOrEmpty (getTalkKey ())) {
	    throw new LivingTalksException ("The talk Key is mandatory!", this);
	}
	if (StringUtils.isNullOrEmpty (getAuthorKey ())) {
	    throw new LivingTalksException ("The author Key is mandatory!", this);
	}
    }

    //----------------------------------------------------------------------
    // XML
    //----------------------------------------------------------------------
    private static List xmlTagProperties = new ArrayList ();
    protected List xmlTagPropertiesStore () { return xmlTagProperties; }
}
