/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Oliver Imbusch<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.model;

import com.livis.flabes.util.*;
import java.util.*;

public class BlockModel
    extends AbstractModel {

    //----------------------------------------------------------------------
    // constructors/assignment
    //----------------------------------------------------------------------
    private BlockModel copy (BlockModel model) {
	return assignBlockModel (model.key, model.editor, model.timestamp,
				 model.duration, model.blockType, model.authorKey, model.title, model.talkList);
    }

    protected BlockModel assignBlockModel (String key, String editor, String timestamp,
					   int duration, String blockType, String authorKey, String title, String talkList) {

	super.assignAbstractModel (key, editor, timestamp);

	this.duration  = duration;
	this.blockType = blockType;
	this.authorKey = authorKey;
	this.title     = title;
	this.talkList  = talkList;

	return this;
    }

    public BlockModel getModel ()                 { return (new BlockModel ()).copy (this); }
    public void       setModel (BlockModel model) { copy (model); }

    public void       init     ()                 { assignBlockModel ("", "", "",
								      0, "", "", "", ""); }
    public void       clear    ()                 { assignBlockModel ("", "", "",
								      0, "", "", "", ""); super.clear (); }

    //----------------------------------------------------------------------
    // CMP fields/properties
    //----------------------------------------------------------------------
    public int    duration;    // (0 for talk blocks)
    public int    getDuration  ()                 { return duration; }
    public void   setDuration  (int    minutes)   { this.duration = minutes; }
    public String blockType;   // eg. "talk", "session", "keynote", "other", "event", "break",
    public String getBlockType ()                 { return blockType; }
    public void   setBlockType (String blockType) { this.blockType = blockType; }
    public String authorKey;   // eg. Leader of session, Speaker of greeting
    public String getAuthorKey ()                 { return authorKey; }
    public void   setAuthorKey (String authorKey) { this.authorKey = authorKey; }
    public String title;       // eg. "Kaffeepause" (null for talk blocks)
    public String getTitle     ()                 { return title; }
    public void   setTitle     (String title)     { this.title = title; }
    public String talkList =""; // eg. "tk003|tk3463" (null for break etc.)
    public String getTalkList  ()                 { return talkList; }
    public void   setTalkList  (String talkList)  { this.talkList = talkList; } // order matters!

    //----------------------------------------------------------------------
    // transient convenience wrappers
    //----------------------------------------------------------------------
    public Collection getTalks          ()                 	    { return Arrays.asList (StringUtils.split (getTalkList (), '|')); }
    public void       setTalks          (Collection talks) 	    { setTalkList (StringUtils.concat (talks, "|")); }
    public String     getDurationString ()                 	    { return "" + getDuration (); }
    public void       setDurationString (String     durationString) { setDuration (LTUtils.parseInt (durationString)); }

    //----------------------------------------------------------------------
    // key creation/check
    //----------------------------------------------------------------------
    public    boolean isKeyCreatable () { return true; }
    protected String  createKey      () {
	setKey (getBlockType () + ":"
		+ ("talk".equals (getBlockType ()) ? getTalkList () : getTitle ()));
	return getKey ();
    }
    public    void    checkForKeyCreation () {
	if (StringUtils.isNullOrEmpty (getBlockType ())) {
	    throw new LivingTalksException ("The block type is mandatory!", this);
	}
	if ("talk".equals (getBlockType ())) {
	    if ((null == getTalks ()) || (1 != getTalks ().size ())) {
		throw new LivingTalksException ("For blocks of kind \"talk\" exactly one talk must be provided!", this);
	    }
	} else if (StringUtils.isNullOrEmpty (getTitle ())) {
	    throw new LivingTalksException ("For non-\"talk\" blocks the title is mandatory!", this);
	}
    }
    public    void    check () {
	if ((0 == getDuration ())
	    && (!("talk".equals (getBlockType ()) || "session".equals (getBlockType ())))) {
	    throw new LivingTalksException ("The duration is mandatory for blocks of kind \"" + getBlockType () + "\"!", this);
	}
	super.check ();
    }

    //----------------------------------------------------------------------
    // XML
    //----------------------------------------------------------------------
    private static List xmlTagProperties = new ArrayList ();
    protected List xmlTagPropertiesStore () { return xmlTagProperties; }
}
