/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Oliver Imbusch<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.model;

import com.livis.flabes.util.*;
import java.sql.*;
import java.util.*;

public class PitemModel
    extends AbstractModel {

    //----------------------------------------------------------------------
    // constructors/assignment
    //----------------------------------------------------------------------
    private PitemModel copy (PitemModel model) {
	return assignPitemModel (model.key, model.editor, model.timestamp,
				 model.start, model.blockKey, model.trackKey);
    }

    protected PitemModel assignPitemModel (String key, String editor, String timestamp,
					   Time start, String blockKey, String trackKey) {

	super.assignAbstractModel (key, editor, timestamp);

	this.start    = start;
	this.blockKey = blockKey;
	this.trackKey = trackKey;

	return this;
    }

    public PitemModel getModel ()                 { return (new PitemModel ()).copy (this); }
    public void       setModel (PitemModel model) { copy (model); }

    public void       init     ()                 { assignPitemModel ("", "", "",
								      new Time (0), "", ""); }
    public void       clear    ()                 { assignPitemModel ("", "", "",
								      new Time (12, 0, 0), "", ""); super.clear (); }

    //----------------------------------------------------------------------
    // CMP fields/properties
    //----------------------------------------------------------------------
    public Time   start = new Time (12, 0, 0);
    public Time   getStart    ()                { return start; }
    public void   setStart    (Time   start)    { this.start = start; }
    public String blockKey;
    public String getBlockKey ()                { return blockKey; }
    public void   setBlockKey (String blockKey) { this.blockKey = blockKey; }
    public String trackKey;
    public String getTrackKey ()                { return trackKey; }
    public void   setTrackKey (String trackKey) { this.trackKey = trackKey; }

    //----------------------------------------------------------------------
    // transient convenience wrappers
    //----------------------------------------------------------------------
    public String getStartHourString   ()                         { return "" + getStart ().getHours (); }
    public void   setStartHourString   (String startHourString)   { start.setHours (LTUtils.parseInt (startHourString)); }
    public String getStartMinuteString ()                         { return StringUtils.flushRight ("" + getStart ().getMinutes (), 2, '0'); }
    public void   setStartMinuteString (String startMinuteString) { start.setMinutes (LTUtils.parseInt (startMinuteString)); }

    //----------------------------------------------------------------------
    // key creation/check
    //----------------------------------------------------------------------
    public boolean    isKeyCreatable () { return true; }
    // FIXME: better key creation?
    protected String  createKey      () { setKey ("" + (getStartHourString () + ":" + getStartMinuteString () + "-" + getBlockKey ()).hashCode ()); return getKey (); }
    public    void    check          () {
	if (StringUtils.isNullOrEmpty (getStartHourString ())) {
	    throw new LivingTalksException ("The start hour is mandatory!", this);
	}
	if (StringUtils.isNullOrEmpty (getStartMinuteString ())) {
	    throw new LivingTalksException ("The start minute is mandatory!", this);
	}
	if (StringUtils.isNullOrEmpty (getBlockKey ())) {
	    throw new LivingTalksException ("The block key is mandatory!", this);
	}
	super.check ();
    }

    //----------------------------------------------------------------------
    // XML
    //----------------------------------------------------------------------
    private static List xmlTagProperties = new ArrayList ();
    protected List xmlTagPropertiesStore () { return xmlTagProperties; }
}
