/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Oliver Imbusch<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.model;

import com.livis.flabes.util.*;
import java.sql.*;
import java.util.*;

public class TalkModel
    extends AbstractModel {

    //----------------------------------------------------------------------
    // constructors/assignment
    //----------------------------------------------------------------------
    private TalkModel copy (TalkModel model) {
	return assignTalkModel (model.key, model.editor, model.timestamp,
				model.duration, model.state, model.title, model.typeKey, model.abstractHtml, model.homePageUrl, model.slidesUrl, model.slidesSize, model.paperUrl, model.draftUrl, model.speaker, model.authors);
    }

    protected TalkModel assignTalkModel (String key, String editor, String timestamp,
					 int duration, String state, String title, String typeKey, String abstractHtml, String homePageUrl, String slidesUrl, String slidesSize, String paperUrl, String draftUrl, String speaker, Collection authors) {

	super.assignAbstractModel (key, editor, timestamp);

	this.duration     = duration;
	this.state        = state;
	this.title        = title;
	this.typeKey      = typeKey;
	this.abstractHtml = abstractHtml;
	this.homePageUrl  = homePageUrl;
	this.slidesUrl    = slidesUrl;
	this.slidesSize   = slidesSize;
	this.paperUrl     = paperUrl;
	this.draftUrl     = draftUrl;
	this.speaker      = speaker;

	this.authors      = authors;

	return this;
    }

    public TalkModel getModel ()                { return (new TalkModel ()).copy (this); }
    public void      setModel (TalkModel model) { copy (model); }

    public void      init     ()                { assignTalkModel ("", "", "",
								   0, "", "", "", "", "", "", "", "", "", "", new ArrayList (0)); }
    public void      clear    ()                { assignTalkModel ("", "", "",
								   0, "released", "", "", "", "", "", "", "", "", "", new ArrayList (0)); super.clear (); }

    //----------------------------------------------------------------------
    // CMP fields/properties
    //----------------------------------------------------------------------
    public int    duration;
    public int    getDuration     ()                    { return duration; }
    public void   setDuration     (int    minutes)      { this.duration = minutes; }
    public String state;
    public String getState 	  ()                    { return state; }
    public void   setState 	  (String state)        { this.state = state; }
    public String title;
    public String getTitle        ()                    { return title; }
    public void   setTitle        (String title)        { this.title = title; }
    public String typeKey;
    public String getTypeKey      ()                    { return typeKey; }
    public void   setTypeKey      (String typeKey)      { this.typeKey = typeKey; }
    public String abstractHtml;
    public String getAbstractHtml ()                    { return abstractHtml; }
    public void   setAbstractHtml (String abstractHtml) { this.abstractHtml = abstractHtml; }
    public String homePageUrl;
    public String getHomePageUrl  ()                    { return homePageUrl; }
    public void   setHomePageUrl  (String homePageUrl)  { this.homePageUrl = homePageUrl; }
    public String slidesUrl;      // eg. "pdf/00/slides/smith.pdf"
    public String getSlidesUrl    ()                    { return slidesUrl; }
    public void   setSlidesUrl    (String slidesUrl)    { this.slidesUrl = slidesUrl; }
    public String slidesSize;     // eg. "37 kB"
    public String getSlidesSize   ()                    { return slidesSize; }
    public void   setSlidesSize   (String slidesSize)   { this.slidesSize = slidesSize; }
    public String paperUrl;
    public String getPaperUrl     ()                    { return paperUrl; }
    public void   setPaperUrl     (String paperUrl)     { this.paperUrl = paperUrl; }
    public String draftUrl;
    public String getDraftUrl     ()                    { return draftUrl; }
    public void   setDraftUrl     (String draftUrl)     { this.draftUrl = draftUrl; }
    public String speaker;
    public String getSpeaker      ()                    { return speaker; }
    public void   setSpeaker      (String speaker)      { this.speaker = speaker; }

    //----------------------------------------------------------------------
    // transient convenience wrappers
    //----------------------------------------------------------------------
    public  String     getAuthorList  	 ()                   	    { return StringUtils.concat (authors, "|"); }
    public  void       setAuthorList  	 (String authorList)  	    { setAuthors (Arrays.asList (StringUtils.split (authorList, '|'))); }
    private Collection authors = new ArrayList (0);
    public  Collection getAuthors     	 ()                   	    { return authors; }
    public  void       setAuthors        (Collection authors)       { this.authors = authors; } // order matters !
    public  String     getDurationString ()                         { return "" + getDuration (); }
    public  void       setDurationString (String     minutesString) { setDuration (LTUtils.parseInt (minutesString)); }

    //----------------------------------------------------------------------
    // short model
    //----------------------------------------------------------------------
    public void makeShort () {
      setAbstractHtml (null);
    }

    //----------------------------------------------------------------------
    // key creation/check
    //----------------------------------------------------------------------
    public    boolean isKeyCreatable      () { return true; }
    protected String  createKey           (int minWordLength, int maxKeyLength) {
	setKey (LTUtils.createKey (getTitle (), minWordLength, maxKeyLength));
	return getKey ();
    }
    protected String  createKey           () { return createKey (5, 20); }
    public    void    checkForKeyCreation () {
	if (StringUtils.isNullOrEmpty (getTitle ())) {
	    throw new LivingTalksException ("The title is mandatory!", this);
	}
    }
    public   void     check               () {
	if (getDuration () <= 0) {
	    throw new LivingTalksException ("The duration is mandatory!", this);
	}
	if (StringUtils.isNullOrEmpty (getTypeKey ())) {
	    throw new LivingTalksException ("The category is mandatory!", this);
	}
	if (StringUtils.isNullOrEmpty (getState ())) {
	    throw new LivingTalksException ("The state is mandatory!", this);
	}
	super.check ();
    }

    //----------------------------------------------------------------------
    // XML
    //----------------------------------------------------------------------
    private static List xmlTagProperties = new ArrayList ();
    protected List xmlTagPropertiesStore () { return xmlTagProperties; }
}
