/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Oliver Imbusch<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.model;

import com.livis.flabes.util.*;
import java.sql.Date;  // must avoid name clash with java.util.Date
import java.util.List;
import java.util.ArrayList;

public class TrackModel
    extends AbstractModel {

    //----------------------------------------------------------------------
    // constructors/assignment
    //----------------------------------------------------------------------
    private TrackModel copy (TrackModel model) {
	return assignTrackModel (model.key, model.editor, model.timestamp,
				 model.title, model.date, model.room, model.trackCol, model.extraTrackCols, model.page);
    }

    protected TrackModel assignTrackModel (String key, String editor, String timestamp,
					   String title, Date date, String room, int trackCol, int extraTrackCols, int page) {

	super.assignAbstractModel (key, editor, timestamp);

	this.title          = title;
	this.date           = date;
	this.room           = room;
	this.trackCol       = trackCol;
	this.extraTrackCols = extraTrackCols;
	this.page           = page;

	return this;
    }

    public TrackModel getModel ()                 { return (new TrackModel ()).copy (this); }
    public void       setModel (TrackModel model) { copy (model); }

    public void       init     ()                 { assignTrackModel ("", "", "",
								      "", new Date (0), "", 0, 0, 0); }
    public void       clear    ()                 { assignTrackModel ("", "", "",
								      "", new Date (System.currentTimeMillis ()), "", 0, 0, 0); super.clear (); }

    //----------------------------------------------------------------------
    // CMP fields/properties
    //----------------------------------------------------------------------
    public String 	 title;            // eg. "Track 2"
    public String 	 getTitle 	   ()                      { return title; }
    public void   	 setTitle 	   (String title)          { this.title = title; }
    public Date          date              = new Date (System.currentTimeMillis ());
    public Date          getDate 	   ()                      { return date; }
    public void   	 setDate 	   (Date date)             { this.date = date; }
    public String 	 room;
    public String 	 getRoom 	   ()                      { return room; }
    public void   	 setRoom 	   (String room)           { this.room = room; }
    public int    	 trackCol;         // logical number in concurrent flow of tracks, from left (=1) to right in printing on a page.
    public int    	 getTrackCol       ()                      { return trackCol; }
    public void   	 setTrackCol       (int    trackCol)       { this.trackCol = trackCol; }
    public int    	 extraTrackCols;   // normally 0, 3 for something spanning tracks 1 to 4.
    public int    	 getExtraTrackCols ()                      { return extraTrackCols; }
    public void   	 setExtraTrackCols (int    extraTrackCols) { this.extraTrackCols = extraTrackCols; }
    public int    	 page;             // something like 1 or 2 pages par day, starting at 1
    public int    	 getPage           ()                      { return page; }
    public void   	 setPage           (int    page)           { this.page = page; }

    //----------------------------------------------------------------------
    // transient convenience wrappers
    //----------------------------------------------------------------------
    public String getDateDayString        ()                            { return StringUtils.flushRight ("" + getDate ().getDate (), 2, '0'); }
    public void   setDateDayString        (String dayString)            { date.setDate (LTUtils.parseInt (dayString)); }
    public String getDateMonthString      ()                            { return StringUtils.flushRight ("" + (getDate ().getMonth () + 1), 2, '0'); }
    public void   setDateMonthString      (String monthString)          { date.setMonth (LTUtils.parseInt (monthString) - 1); }
    public String getDateYearString       ()                            { return "" + (getDate ().getYear () + 1900); }
    public void   setDateYearString       (String yearString)           { date.setYear (LTUtils.parseInt (yearString) - 1900); }
    public String getTrackColString       ()                            { return "" + getTrackCol (); }
    public void   setTrackColString       (String trackColString)       { setTrackCol (LTUtils.parseInt (trackColString)); }
    public String getExtraTrackColsString ()                            { return "" + getExtraTrackCols (); }
    public void   setExtraTrackColsString (String extraTrackColsString) { setExtraTrackCols (LTUtils.parseInt (extraTrackColsString)); }
    public String getPageString           ()                            { return "" + getPage (); }
    public void   setPageString           (String pageString)           { setPage (LTUtils.parseInt (pageString)); }

    //----------------------------------------------------------------------
    // XML
    //----------------------------------------------------------------------
    private static List xmlTagProperties = new ArrayList ();
    protected List xmlTagPropertiesStore () { return xmlTagProperties; }
}
