/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.1
 */
package com.livis.livingtalks.web;

import com.livis.livingtalks.xml.Export;
import com.livis.livingtalks.model.*;
import com.livis.livingtalks.ejb.*;
import java.io.*;


public class ExportWebImpl {

  private static PublishHome  home    = LTUtils.getPublishHome ();
  private Publish             publish = null;
  private String              user    = "unknown";

  public void init () {
    user = "unknown";
  }

  public void processRequest (String request, Writer out, String user) {
    this.user = user;
    if (request == null) request = "";
    if (request.equals ("clear"))  { }
    if (request.equals ("backup")) { exportToFile (new File (getStorePath () + getExportName () + ".xml"), out); }
  }

  public void write (String out) {
    try {
      if (publish == null) publish = home.create ();
      publish.exportXML (out);
    } catch (Exception ex) { LTUtils.handleException (ex, this, null, null);
    }
  }

  public void write (Writer out) { // we cannot pass a Writer to an EJB - not serializable => do it locally
    Export export = new Export ();
    export.setPrintHeader (false);
    export.write (out);
  }

  public void exportToFile (File file, Writer out) {
    PrintWriter pout = null;
    try {
      try {
        if (out != null) {
          pout = (out instanceof PrintWriter) ? (PrintWriter) out : new PrintWriter (out);
          pout.println ("<p class=\"psmallNormal\"><b>[making backup ...</b><br>");
          pout.println ("to <b>" + file.getName () + "</b><br>");
          pout.println ("please, stand by</p>");
          pout.flush ();
        }
        write (file.getAbsolutePath ());
        if (pout != null) {
          pout.println ("<p class=\"psmallNormal\"><b>... backup complete]</b></p>");
          pout.flush ();
        }
      } catch (Exception ex) { LTUtils.handleException (ex, this, null, null);
      }
    } catch (LivingTalksException le) {
      if (pout != null) {
        pout.println ("<hr><hr><hr><h1 style=\"color: #770000\">ERROR: Backup failed!</h1>");
        pout.println ("<hr><hr><hr><h2 style=\"color: #770000\">" + le.getTitle() +"</h2><p class=\"psmallNormal\">Details follow:</p><pre>");
        pout.println (le.getDetails ());
        pout.println ("</pre>");
      }
      else throw le;
    }
  }

  /** @return a webmenu of filenames (listed with no suffix) found under storePath.
   *  @see    #getStorePath()
   */
  public WebMenu getSources () {
    File dir = new File (getStorePath ().substring (0, getStorePath ().length () - 1));
    if (!dir.isDirectory ()) throw new LivingTalksException ("Given path (" + dir.getAbsolutePath () + ") is no directory!", this);
    File [] files = dir.listFiles ();
    WebMenu sources = new WebMenu ();
    for (int i = 0; i < files.length; ++i) {
      String file = files [i].getName ();
      String name = PublishWebImpl.cutAfterLast (file, '.'); name = name.substring (0, name.length () - 1);
      sources.add (file, name);
    }
    return sources;
  }

  /** @return a suitable name for a freshly created backup target
   */
  public String mkExportName () {
    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat ("yyyy-MM-dd__HHmmss");
    return "LTFullBackup__" + formatter.format (new java.util.Date ()) + "__" + user;
  }

  //
  // bean properties settable by form
  //

  // recognized export properties
  private String exportName     = "";
  public  String getExportName  ()                    { return (exportName == null || exportName.equals ("")) ? mkExportName () : exportName; }
  public  void   setExportName  (String exportName)   { this.exportName = exportName; }

  // recognized context properties

  /** StorePath is the path prefix of all names of files to be stored or read during im- and export.
   *  This path should be an absolute file path prefix (not a file URL!) if the server's working directory is unknown.
   *  (Anything past the last "/", or "\\" resp., is ignored.)
   */
  public  void   setStorePath   (String storePath)   { this.storePath = PublishWebImpl.cutAfterLast (storePath, java.io.File.separatorChar); }
  public  String getStorePath   ()                   { return storePath; }
  private String storePath      = "." + java.io.File.separator;

  /** RequestPath is the URL base relative which all names of files read during publishing are interpreted to.
   *  This path must be an absolute URL. It works like the HTML &lt;<code>base</code>&gt;-tag.
   *  (Anything past the last "/" is ignored.)
   *  RequestPath and StorePath must point to the same thing in browser, and server notations, resp.
   */
  public  void   setRequestPath (String requestPath) { this.requestPath = PublishWebImpl.cutAfterLast (requestPath, '/'); }
  public  String getRequestPath ()                   { return requestPath; }
  private String requestPath    = "";
}
