/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.web;

import com.livis.livingtalks.model.*;
import com.livis.livingtalks.xml.*;
import com.livis.livingtalks.ejb.*;
import com.livis.flabes.util.*;
import java.io.*;
import java.net.*;
import java.util.*;


public class ImportWebImpl {

  private static PublishHome  home    = LTUtils.getPublishHome ();
  private Publish             publish = null;
  private Manage              manage  = null;

  public void init () {
    setMode      ("IMPORT");
    setUrlSource ("no");
    setInOneTx   ("no");
  }

  public void processRequest (String request, Writer out) {
    if (request == null) request = "";
    if (request.equals ("clear"))  { }
    if (request.equals ("import")) { importIt (out, getUrlSource ().equals ("yes") ? getImportUrl () : getImportXml ()); }
  }

  public void importIt (Writer out, String source) {
    try {
      URL url = new URL (new URL (getRequestPath ()), source);
      importFromUrl (url, out);
    } catch (MalformedURLException ue) { LTUtils.handleException (ue, this, "Import from where?<br>", getRequestPath () + source);
    }
  }

  public void importFromUrl (URL url, Writer out) {
    PrintWriter pout = null;
    try {
      try {
        if (out != null) {
          pout = (out instanceof PrintWriter) ? (PrintWriter) out : new PrintWriter (out);
          pout.println ("<p class=\"psmallNormal\"><b>[... Importing</b><br>");
          pout.println ("from <b>" + url.toExternalForm () + "</b> into <b>" + getConference () + "</b><br>");
          pout.println ("Options are: " + getMode () + ", <i>from URL</i> = " + getUrlSource () + " and <i>must succeed</i> = " + getInOneTx () + "<br>");
          pout.println ("please, stand by</p>");
          if (getMode ().equals ("IMPORT") && getInOneTx ().equals ("yes"))
            pout.println ("<hr><h2 style=\"color: #770000\">Warning:<br>This may take several hours!!<br>You may see a browser time-out while the server continues its job!<br>This may be ignored. But You will not know when the job is done.</h2>");
          pout.flush ();
        }
        if (publish == null) publish = home.create ();
        ImportResult result = null;
        if (getInOneTx ().equals ("yes")) {
            // give ManageBean a chance to warm up out of transaction... touch 'em all
            if (manage == null) manage = LTUtils.createManage ();
            manage.getAuthorKeysAll ();
            manage.getAuthorshipKeysAll ();
            manage.getBlockKeysAll ();
            manage.getPitemKeysAll ();
            manage.getTrackKeysAll ();
            manage.getTalkKeysAll ();
            manage.getTrackKeysAll ();

            result = publish.importXMLinOneTx (url.toExternalForm (), getConference (), ((Integer) modes.get (getMode ())).intValue ());
        }
        else {
            result = publish.importXML        (url.toExternalForm (), getConference (), ((Integer) modes.get (getMode ())).intValue ());
        }
        if (pout != null) {
          pout.println ("<p class=\"psmallNormal\"><b>[Import complete]</b> The following details are available:</p>");
          if (result != null) {
            pout.println ("<p class=\"psmall\">Number of old records deleted:  " + result.deleted + "</p>");
            pout.println ("<p class=\"psmall\">Number of old records changed: "  + result.changed + "</p>");
            pout.println ("<p class=\"psmall\">Number of new records added: "    + result.created + "</p>");
            if (result.notchanged + result.notcreated + result.notdeleted + result.exceptions.size () != 0) {
              pout.println ("<hr><h2 style=\"color: #770000\">There have been errors or warnings:</h2><p class=\"psmallNormal\">Details follow:</p>");
              pout.println ("<p class=\"psmall\">Number of old records <b>not</b> deleted:  " + result.notdeleted + "</p>");
              pout.println ("<p class=\"psmall\">Number of old records <b>not</b> changed: "  + result.notchanged + "</p>");
              pout.println ("<p class=\"psmall\">Number of new records <b>not</b> added: "    + result.notcreated + "</p>");
              if (result.exceptions.size () != 0) {
                pout.println ("<table width=\"100%\" border=\"1\">");
                pout.println ("<tr><th><b>Exception</b></th><th><b>Reason:</b></th></tr>");
                for (Iterator i = result.exceptions.iterator (); i.hasNext (); ) {
                  LivingTalksException le = (LivingTalksException) i.next ();
                  Throwable            ex = le.getNested ();
                  LivingTalksException ne = (ex instanceof LivingTalksException) ? ((LivingTalksException) ex) : le;
                  pout.println ("<tr><td valign=\"top\"><b>" + le.getTitle() + "</b></td>");
                  pout.println ("<td class=\"psmallNormal\">" + ne.getTitle () + "<br><pre class=\"psmallNormal\">" + ne.getDetails () + "</pre></td></tr>");
                }
                pout.println ("</table>");
              }
            }
            else if (getMode ().equals ("IMPORT") && result.deleted < result.created) {
              pout.println ("<p class=\"psmallNormal\"><b>Note:</b> It is normal that the number of old records deleted is smaller than the number"
                          + " of new records created even with identical content (authorship deletions are not counted).</p>");
            }
          }
          pout.flush ();
        }
      } catch (Exception ex) { LTUtils.handleException (ex, this, null, null);
      }
    } catch (LivingTalksException le) {
      if (pout != null) {
        pout.println ("<hr><hr><hr><h1 style=\"color: #770000\">ERROR: Importing failed!</h1>");
        if (getInOneTx ().equals ("yes"))
          pout.println ("<h1 style=\"color: #770000\">We have tried to restore the old content.</h1>");
        pout.println ("<hr><hr><hr><h2 style=\"color: #770000\">" + le.getTitle() +"</h2><p class=\"psmallNormal\">Details follow:</p><pre>");
        pout.println (le.getDetails ());
        // le.printStackTrace (pout);
        pout.println ("</pre>");
      }
      else throw le;
    }
  }

  /** @return a webmenu of filenames (listed with no suffix) found under storePath.
   *  @see    #getStorePath()
   */
  public WebMenu getSources () {
    File dir = new File (getStorePath ().substring (0, getStorePath ().length () - 1));
    if (!dir.isDirectory ()) throw new LivingTalksException ("Given path (" + dir.getAbsolutePath () + ") is no directory!", this);
    File [] files = dir.listFiles ();
    WebMenu sources = new WebMenu ();
    for (int i = 0; i < files.length; ++i) {
      String file = files [i].getName ();
      String name = PublishWebImpl.cutAfterLast (file, '.'); name = name.substring (0, name.length () - 1);
      sources.add (file, name);
    }
    return sources;
  }

  //
  // bean properties settable by form
  //

  // recognized parameter settings
  private static Hashtable modes = new Hashtable (3);
  static {
    modes.put ("IMPORT", new Integer (DOMContainer.IMPORT));
    modes.put ("ADD",    new Integer (DOMContainer.ADD));
    modes.put ("CHANGE", new Integer (DOMContainer.CHANGE));
  }
  private String mode           = "IMPORT";
  public  String getMode        ()                   { return mode; }
  public  void   setMode        (String mode)        { this.mode = mode; }

  private String urlSource      = "no";
  public  String getUrlSource   ()                   { return urlSource; }
  public  void   setUrlSource   (String urlSource)   { this.urlSource = urlSource; }

  private String inOneTx        = "no";
  public  String getInOneTx     ()                   { return inOneTx; }
  public  void   setInOneTx     (String inOneTx)     { this.inOneTx = inOneTx; }

  // recognized import properties
  private String importXml      = ".xml";
  public  String getImportXml   ()                   { return importXml; }
  public  void   setImportXml   (String importXml)   { this.importXml = importXml; }

  private String importUrl      = "http://";
  public  String getImportUrl   ()                   { return importUrl; }
  public  void   setImportUrl   (String importUrl)   { this.importUrl = importUrl; }

  // recognized context properties

  /** Conference is the name of the conference to be imported.
   *  It must match the corresponding item in the import document.
   */
  public  void   setConference  (String conference)  { this.conference = conference; }
  public  String getConference  ()                   { return conference; }
  private String conference     = "Net.ObjectDays 2000";

  /** StorePath is the path prefix of all names of files to be stored or read during ex- and import.
   *  This path should be an absolute file path prefix (not a file URL!) if the server's working directory is unknown.
   *  (Anything past the last "/", or "\\" resp., is ignored.)
   */
  public  void   setStorePath   (String storePath)   { this.storePath = PublishWebImpl.cutAfterLast (storePath, java.io.File.separatorChar); }
  public  String getStorePath   ()                   { return storePath; }
  private String storePath      = "." + java.io.File.separator;

  /** RequestPath is the URL base relative which all names of files read during publishing are interpreted to.
   *  This path must be an absolute URL. It works like the HTML &lt;<code>base</code>&gt;-tag.
   *  (Anything past the last "/" is ignored.)
   *  RequestPath and StorePath must point to the same thing in browser, and server notations, resp.
   */
  public  void   setRequestPath (String requestPath) { this.requestPath = PublishWebImpl.cutAfterLast (requestPath, '/'); }
  public  String getRequestPath ()                   { return requestPath; }
  private String requestPath    = "";
}
