/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.web;

import com.livis.flabes.util.*;
import com.livis.livingtalks.ejb.*;
import com.livis.livingtalks.model.*;
import java.io.*;
import java.net.*;
import java.util.*;


public class PublishWebImpl {

  public static final String MKPREFIX   = "mk-";
  public static final String PREVPREFIX = "preview-";
  public static final String RESPREFIX  = "resources/";

  private static PublishHome  home      = LTUtils.getPublishHome ();
  private Publish             publish   = null;
  private String              query     = "";

  public void init () {
    setAuthorsChecked         (false);
    setAgendaChecked          (false);
    setTalksChecked           (false);
    setOverviewChecked        (false);
    setNsviewChecked          (false);
    setArchiveChecked         (false);
    setPreviewChecked         (false);
    setStyleDraftsChecked     (false);
    setStyleSessautChecked    (false);
    setStyleDoublepageChecked (false);
    setPrintChecked           (false);
    setXprintChecked          (false);
  }

  public void processRequest (String request, Writer out, String query) {
    if (query != null) this.query = "?" + query;
    else               this.query = "";
    if (request == null) request = "";
    if (request.equals ("clear"))   { }
    if (request.equals ("update"))  { updateAll (out); }
    if (request.equals ("archive")) { updateArchive (null); }
  }

  public void updateAll (Writer out) {
    PrintWriter pout = null;
    try {
      try {
        if (publish == null) publish = home.create ();
        String [] websitePaths = StringUtils.split (getWebsitePath (), ';');
        for (int i = 0; i < websitePaths.length; ++i)
          websitePaths [i] = cutAfterLast (websitePaths [i], '/');
        String assumedBaseUrl  = websitePaths [0] + (isPreviewChecked () ? getPreviewInstallPath () : getInstallPath ());
        if (out != null) {
          pout = (out instanceof PrintWriter) ? (PrintWriter) out : new PrintWriter (out);
          pout.println ("<p class=\"psmallNormal\"><b>[... Processing requests</b><br>please, stand by</p><ol class=\"psmallNormal\">");
        }
        if (isXprintChecked ())   updateXPrint   (pout, assumedBaseUrl, websitePaths);
        if (isPrintChecked ())    updatePrint    (pout, assumedBaseUrl, websitePaths);
        if (isOverviewChecked ()) updateOverview (pout, assumedBaseUrl, websitePaths);
        if (isNsviewChecked ())   updateNsview   (pout, assumedBaseUrl, websitePaths);
        if (isAgendaChecked ())   updateAgenda   (pout, assumedBaseUrl, websitePaths);
        if (isTalksChecked ())    updateTalks    (pout, assumedBaseUrl, websitePaths);
        if (isAuthorsChecked ())  updateAuthors  (pout, assumedBaseUrl, websitePaths);
        if (isArchiveChecked ())  updateArchive  (pout);
        if (pout != null) { pout.println ("</ol><p class=\"psmallNormal\"><b>requests complete ...]</b></p>"); }
      } catch (Exception ex) { LTUtils.handleException (ex, this, null, null);
    }
    } catch (LivingTalksException le) {
      if (pout != null) {
        pout.println ("</li></ol><hr><hr><hr><h1 style=\"color: #770000\">ERROR: Publishing failed!</h1>");
        pout.println ("</li></ol><hr><hr><hr><h2 style=\"color: #770000\">" + le.getTitle() +"</h2><p class=\"psmallNormal\">Details follow:</p><pre>");
        pout.println (le.getDetails ());
        // le.printStackTrace (pout);
        pout.println ("</pre>");
      }
      else throw le;
    }
  }

  public void updateAuthors (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating XML for authors [.."); pout.flush (); }
    publish.authors (getStorePath () + getAuthorsXml (), assumedBaseUrl, websitePaths);
    if (pout != null) { pout.println (".]</li><li>creating HTML for authors [.."); pout.flush (); }
    String readas = getAuthorsJsp (); if (isPreviewChecked ()) readas = StringUtils.replace (readas, MKPREFIX, PREVPREFIX);
    saveURLAs (getStorePath () + getAuthorsHtml (), getRequestPath () + readas + query);
    if (pout != null) { pout.println (".]</li>"); }
  }

  public void updateAgenda (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating XML for agenda [.."); pout.flush (); }
    publish.agenda (getStorePath () + getAgendaXml (), agendaAbstract, assumedBaseUrl, websitePaths);
    if (pout != null) { pout.println (".]</li><li>creating HTML for agenda [.."); pout.flush (); }
    String readas = getAgendaJsp (); if (isPreviewChecked ()) readas = StringUtils.replace (readas, MKPREFIX, PREVPREFIX);
    saveURLAs (getStorePath () + getAgendaHtml (), getRequestPath () + readas + query);
    if (pout != null) { pout.println (".]</li>"); }
  }

  public void updateTalks (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating XML for talks [.."); pout.flush (); }
    publish.talks (getStorePath () + getTalksXml (), assumedBaseUrl, websitePaths);
    if (pout != null) { pout.println (".]</li><li>creating HTML for talks [.."); pout.flush (); }
    String readas = getTalksJsp (); if (isPreviewChecked ()) readas = StringUtils.replace (readas, MKPREFIX, PREVPREFIX);
    saveURLAs (getStorePath () + getTalksHtml (), getRequestPath () + readas + query);
    if (pout != null) { pout.println (".]</li>"); }
  }

  public void updateXPrint (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating XML for print and overview [.."); pout.flush (); }
    publish.overview (getStorePath () + getPrintXml (), assumedBaseUrl, websitePaths);
    if (pout != null) { pout.println (".]</li>"); pout.flush (); }
  }

  public void updatePrint (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating HTML for print [.."); pout.flush (); }
    String readas = getPrintJsp (); if (isPreviewChecked ()) readas = StringUtils.replace (readas, MKPREFIX, PREVPREFIX);
    saveURLAs (getStorePath () + getPrintHtml (), getRequestPath () + readas + query);
    if (pout != null) { pout.println (".]</li>"); }
  }

  public void updateOverview (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating HTML for overview [.."); pout.flush (); }
    String readas = getOverviewJsp (); if (isPreviewChecked ()) readas = StringUtils.replace (readas, MKPREFIX, PREVPREFIX);
    saveURLAs (getStorePath () + getOverviewHtml (), getRequestPath () + readas + query);
    if (pout != null) { pout.println (".]</li>"); }
  }

  public void updateNsview (PrintWriter pout, String assumedBaseUrl, String [] websitePaths) throws java.rmi.RemoteException, IOException {
    if (pout != null) { pout.println ("<li>creating HTML for Netscape overview [.."); pout.flush (); }
    String readas = getNsviewJsp (); if (isPreviewChecked ()) readas = StringUtils.replace (readas, MKPREFIX, PREVPREFIX);
    saveURLAs (getStorePath () + getNsviewHtml (), getRequestPath () + readas + query);
    if (pout != null) { pout.println (".]</li>"); }
  }

  public void updateArchive (PrintWriter pout) {
    try {
      if (pout != null) { pout.println ("<li>creating archive [.."); pout.flush (); }
      createJar (getStorePath () + getArchiveJar (), getStorePath (), getInstallPath(), new
          String [] {
            getPrintHtml    (), getPrintXml    (),
            getOverviewHtml (),
            getNsviewHtml   (),
            getAgendaHtml   (), getAgendaXml   (),
            getTalksHtml    (), getTalksXml    (),
            getAuthorsHtml  (), getAuthorsXml  ()
          }
        );
      if (pout != null) { pout.println (".]</li>"); }
    } catch (IOException ie) { LTUtils.handleException (ie, this, "The archive file could not be produced!", getStorePath () + getArchiveJar ());
    }
  }

  private void createJar (String jarSpec, String readFromPrefix, String writeAsPrefix, String[] entryNames) throws IOException {
    JarUtils.NamedInputStream[] ins = new JarUtils.NamedInputStream [entryNames.length];
    for (int i = 0; i < ins.length; ++i) {
      String fileName = readFromPrefix + entryNames [i];
      ins [i] = new JarUtils.NamedInputStream (writeAsPrefix + entryNames [i], new FileInputStream (fileName));
    }
    OutputStream os = new FileOutputStream (new File (jarSpec).getAbsolutePath ());
    JarUtils.createJar (os, ins);
    os.close ();
  }

  private void saveURLAs (String outName, String inSpec) throws MalformedURLException, IOException {
    InputStream  in  = new URL (inSpec).openConnection ().getInputStream ();
    OutputStream out = new FileOutputStream (new File (outName).getAbsolutePath ());
    IOUtils.copy (out, in);
    in.close ();
    out.close ();
  }

  public static String cutAfterLast (String s, char last) {
    int i; return (s != null && (i = s.lastIndexOf (last)) != -1) ? s.substring (0, i + 1) : s;
  }

  //
  // bean properties settable by form
  //

  // recognized authors properties
  private boolean authorsChecked          = false;                       // JSP-automatism will fail when default is chenged to true
  public void    setAuthorsChecked        (boolean authorsChecked)       { this.authorsChecked = authorsChecked; }
  public boolean isAuthorsChecked         ()                             { return authorsChecked; }
  public void    setAuthorsCheckBoxState  (String authorsCheckBoxState)  { setAuthorsChecked ("checked".equals (authorsCheckBoxState)); }
  public String  getAuthorsCheckBoxState  ()                             { return isAuthorsChecked () ? "checked" : ""; }

  private String authorsXml               = "authors.xml";
  public  String getAuthorsXml            ()                             { return authorsXml; }
  public  void   setAuthorsXml            (String authorsXml)            { this.authorsXml = authorsXml; }

  private String authorsHtml              = "authors.html";
  public  String getAuthorsHtml           ()                             { return authorsHtml; }
  public  void   setAuthorsHtml           (String authorsHtml)           { this.authorsHtml = authorsHtml; }

  private String authorsJsp               = MKPREFIX + "authors.jsp";
  public  String getAuthorsJsp            ()                             { return authorsJsp; }
  public  void   setAuthorsJsp            (String authorsJsp)            { this.authorsJsp = authorsJsp; }

  // recognized agenda properties
  private boolean agendaChecked           = false;
  public void    setAgendaChecked         (boolean agendaChecked)        { this.agendaChecked = agendaChecked; }
  public boolean isAgendaChecked          ()                             { return agendaChecked; }
  public void    setAgendaCheckBoxState   (String agendaCheckBoxState)   { setAgendaChecked ("checked".equals (agendaCheckBoxState)); }
  public String  getAgendaCheckBoxState   ()                             { return isAgendaChecked () ? "checked" : ""; }

  private String agendaXml                = "agenda.xml";
  public  String getAgendaXml             ()                             { return agendaXml; }
  public  void   setAgendaXml             (String agendaXml)             { this.agendaXml = agendaXml; }

  private String agendaHtml               = "agenda.html";
  public  String getAgendaHtml            ()                             { return agendaHtml; }
  public  void   setAgendaHtml            (String agendaHtml)            { this.agendaHtml = agendaHtml; }

  private String agendaJsp                = MKPREFIX + "agenda.jsp";
  public  String getAgendaJsp             ()                             { return agendaJsp; }
  public  void   setAgendaJsp             (String agendaJsp)             { this.agendaJsp = agendaJsp; }

  private boolean agendaAbstract          = false;  // printing of abstract in agenda is turned off.

  // recognized talks properties
  private boolean talksChecked            = false;
  public void    setTalksChecked          (boolean talksChecked)         { this.talksChecked = talksChecked; }
  public boolean isTalksChecked           ()                             { return talksChecked; }
  public void    setTalksCheckBoxState    (String talksCheckBoxState)    { setTalksChecked ("checked".equals (talksCheckBoxState)); }
  public String  getTalksCheckBoxState    ()                             { return isTalksChecked () ? "checked" : ""; }

  private String talksXml                 = "talks.xml";
  public  String getTalksXml              ()                             { return talksXml; }
  public  void   setTalksXml              (String talksXml)              { this.talksXml = talksXml; }

  private String talksHtml                = "talks.html";
  public  String getTalksHtml             ()                             { return talksHtml; }
  public  void   setTalksHtml             (String talksHtml)             { this.talksHtml = talksHtml; }

  private String talksJsp                 = MKPREFIX + "talks.jsp";
  public  String getTalksJsp              ()                             { return talksJsp; }
  public  void   setTalksJsp              (String talksJsp)              { this.talksJsp = talksJsp; }

  // recognized overview properties
  private boolean overviewChecked         = false;
  public void    setOverviewChecked       (boolean overviewChecked)      { this.overviewChecked = overviewChecked; }
  public boolean isOverviewChecked        ()                             { return overviewChecked; }
  public void    setOverviewCheckBoxState (String overviewCheckBoxState) { setOverviewChecked ("checked".equals (overviewCheckBoxState)); }
  public String  getOverviewCheckBoxState ()                             { return isOverviewChecked () ? "checked" : ""; }

  private String overviewHtml             = "overview.html";
  public  String getOverviewHtml          ()                             { return overviewHtml; }
  public  void   setOverviewHtml          (String overviewHtml)          { this.overviewHtml = overviewHtml; }

  private String overviewJsp              = MKPREFIX + "overview.jsp";
  public  String getOverviewJsp           ()                             { return overviewJsp; }
  public  void   setOverviewJsp           (String overviewJsp)           { this.overviewJsp = overviewJsp; }

  // recognized nsview properties
  private boolean nsviewChecked           = false;
  public void    setNsviewChecked         (boolean nsviewChecked)        { this.nsviewChecked = nsviewChecked; }
  public boolean isNsviewChecked          ()                             { return nsviewChecked; }
  public void    setNsviewCheckBoxState   (String nsviewCheckBoxState)   { setNsviewChecked ("checked".equals (nsviewCheckBoxState)); }
  public String  getNsviewCheckBoxState   ()                             { return isNsviewChecked () ? "checked" : ""; }

  private String nsviewHtml               = "nsoverview.html";
  public  String getNsviewHtml            ()                             { return nsviewHtml; }
  public  void   setNsviewHtml            (String nsviewHtml)            { this.nsviewHtml = nsviewHtml; }

  private String nsviewJsp                = MKPREFIX + "nsoverview.jsp";
  public  String getNsviewJsp             ()                             { return nsviewJsp; }
  public  void   setNsviewJsp             (String nsviewJsp)             { this.nsviewJsp = nsviewJsp; }

  // recognized print properties
  private boolean xprintChecked           = false;
  public void    setXprintChecked         (boolean xprintChecked)        { this.xprintChecked = xprintChecked; }
  public boolean isXprintChecked          ()                             { return xprintChecked; }
  public void    setXprintCheckBoxState   (String xprintCheckBoxState)   { setXprintChecked ("checked".equals (xprintCheckBoxState)); }
  public String  getXprintCheckBoxState   ()                             { return isXprintChecked () ? "checked" : ""; }

  private boolean printChecked            = false;
  public void    setPrintChecked          (boolean printChecked)         { this.printChecked = printChecked; }
  public boolean isPrintChecked           ()                             { return printChecked; }
  public void    setPrintCheckBoxState    (String printCheckBoxState)    { setPrintChecked ("checked".equals (printCheckBoxState)); }
  public String  getPrintCheckBoxState    ()                             { return isPrintChecked () ? "checked" : ""; }

  private String printXml                 = "print.xml";
  public  String getPrintXml              ()                             { return printXml; }
  public  void   setPrintXml              (String printXml)              { this.printXml = printXml; }

  private String printHtml                = "print.html";
  public  String getPrintHtml             ()                             { return printHtml; }
  public  void   setPrintHtml             (String printHtml)             { this.printHtml = printHtml; }

  private String printJsp                 = MKPREFIX + "print.jsp";
  public  String getPrintJsp              ()                             { return printJsp; }
  public  void   setPrintJsp              (String printJsp)              { this.printJsp = printJsp; }

  // recognized archive properties
  private boolean archiveChecked          = false;
  public void    setArchiveChecked        (boolean archiveChecked)       { this.archiveChecked = archiveChecked; }
  public boolean isArchiveChecked         ()                             { return archiveChecked; }
  public void    setArchiveCheckBoxState  (String archiveCheckBoxState)  { setArchiveChecked ("checked".equals (archiveCheckBoxState)); }
  public String  getArchiveCheckBoxState  ()                             { return isArchiveChecked () ? "checked" : ""; }

  private String archiveJar               = "published-program.zip";
  public  String getArchiveJar            ()                             { return archiveJar; }
  public  void   setArchiveJar            (String archiveJar)            { this.archiveJar = archiveJar; }

  // recognized context properties
  private boolean previewChecked          = false;
  public void    setPreviewChecked        (boolean previewChecked)       { this.previewChecked = previewChecked; }
  public boolean isPreviewChecked         ()                             { return previewChecked; }
  public void    setPreviewCheckBoxState  (String previewCheckBoxState)  { setPreviewChecked ("checked".equals (previewCheckBoxState)); }
  public String  getPreviewCheckBoxState  ()                             { return isPreviewChecked () ? "checked" : ""; }

  /** WebsitePath is the path (at least one absolute URL) to the homepage of the publishing website.
   *  (Anything past the last "/" is ignored.)
   *  This URL is defined to act as root for any relative URL entered into the database.
   *  Multiple such URLs to equivalent homepages (mirrors) may be specified by separating them by the ';'-character and with no blanks.
   *  (We do not use the Unix path separator character on Unix systems as this is the URL protocol separator character.)
   */
  public  void   setWebsitePath           (String websitePath)           { this.websitePath = websitePath; }
  public  String getWebsitePath           ()                             { return websitePath; }
  private String websitePath              = "http://www.netobjectdays.org/index.html;http://www.net.objectdays.org/;http://net.objectdays.org/";

  /** InstallPath is the URL path prefix to the generated HTML-pages <b>after</b> publishing.
   *  (Anything past the last "/" is ignored.)
   *  This path is exactly one relative URL which must be relative to WebsitePath.
   *  An absolute path of a published HTML page therefore reads (asuming no mirrors):
   *  <code>(getWebsitePath()+getInstallPath()+getOverviewHtml())</code>
   */
  public  void   setInstallPath           (String installPath)           { this.installPath = cutAfterLast (installPath, '/'); }
  public  String getInstallPath           ()                             { return installPath; }
  private String installPath              = "node00/de/Conf/publish/";

  /** PreviewInstallPath is the URL path prefix to the generated HTML-pages <b>before</b> publishing.
   *  (Anything past the last "/" is ignored.)
   *  This path is exactly one relative URL which must be relative to the J2EE webserver's root (not the application root).
   */
  public  void   setPreviewInstallPath    (String previewInstallPath)    { this.previewInstallPath = cutAfterLast (previewInstallPath, '/'); }
  public  String getPreviewInstallPath    ()                             { return previewInstallPath; }
  private String previewInstallPath       = "livingtalks/publish/";

  /** StorePath is the path prefix of all names of files to be stored during publishing.
   *  This path should be an absolute file path prefix (not a file URL!) if the server's working directory is unknown.
   *  (Anything past the last "/", or "\\" resp., is ignored.)
   */
  public  void   setStorePath             (String storePath)             { this.storePath = cutAfterLast (storePath, java.io.File.separatorChar); }
  public  String getStorePath             ()                             { return storePath; }
  private String storePath                = "." + java.io.File.separator;

  /** RequestPath is the URL base relative which all names of files read during publishing are interpreted to.
   *  This path must be an absolute URL. It works like the HTML &lt;<code>base</code>&gt;-tag.
   *  (Anything past the last "/" is ignored.)
   *  RequestPath and StorePath must point to the same thing in browser, and server notations, resp.
   */
  public  void   setRequestPath           (String requestPath)           { this.requestPath = cutAfterLast (requestPath, '/'); }
  public  String getRequestPath           ()                             { return requestPath; }
  private String requestPath              = "";

  // Layout parameters (passed on to style processor, defined here only for "remembering values" within a session)
  public  void   setStyleCutlen           (String styleCutlen)           { this.styleCutlen = styleCutlen; }
  public  String getStyleCutlen           ()                             { return styleCutlen; }
  private String styleCutlen              = "";

  public  void   setStylePagenumber       (String stylePagenumber)       { this.stylePagenumber = stylePagenumber; }
  public  String getStylePagenumber       ()                             { return stylePagenumber; }
  private String stylePagenumber          = "";

  public  void   setStyleHourh            (String styleHourh)            { this.styleHourh = styleHourh; }
  public  String getStyleHourh            ()                             { return styleHourh; }
  private String styleHourh               = "160";

  public  void   setStyleCw               (String styleCw)               { this.styleCw = styleCw; }
  public  String getStyleCw               ()                             { return styleCw; }
  private String styleCw                  = "180";

  private boolean styleDraftsChecked              = false;
  public void    setStyleDraftsChecked            (boolean styleDraftsChecked)           { this.styleDraftsChecked = styleDraftsChecked; }
  public boolean isStyleDraftsChecked             ()                                     { return styleDraftsChecked; }
  public void    setStyleDraftsCheckBoxState      (String styleDraftsCheckBoxState)      { setStyleDraftsChecked ("checked".equals (styleDraftsCheckBoxState)); }
  public String  getStyleDraftsCheckBoxState      ()                                     { return isStyleDraftsChecked () ? "checked" : ""; }

  private boolean styleSessautChecked             = false;
  public void    setStyleSessautChecked           (boolean styleSessautChecked)          { this.styleSessautChecked = styleSessautChecked; }
  public boolean isStyleSessautChecked            ()                                     { return styleSessautChecked; }
  public void    setStyleSessautCheckBoxState     (String styleSessautCheckBoxState)     { setStyleSessautChecked ("checked".equals (styleSessautCheckBoxState)); }
  public String  getStyleSessautCheckBoxState     ()                                     { return isStyleSessautChecked () ? "checked" : ""; }

  private boolean styleDoublepageChecked          = false;
  public void    setStyleDoublepageChecked        (boolean styleDoublepageChecked)       { this.styleDoublepageChecked = styleDoublepageChecked; }
  public boolean isStyleDoublepageChecked         ()                                     { return styleDoublepageChecked; }
  public void    setStyleDoublepageCheckBoxState  (String styleDoublepageCheckBoxState)  { setStyleDoublepageChecked ("checked".equals (styleDoublepageCheckBoxState)); }
  public String  getStyleDoublepageCheckBoxState  ()                                     { return isStyleDoublepageChecked () ? "checked" : ""; }
}
