/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Living Talks Web Conference Tool using J2EE standards<p>
 *
 * <b>This file was automatically generated from a template!</b>
 *
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.web;

import com.livis.livingtalks.model.*;
import com.livis.livingtalks.ejb.*;
import com.livis.flabes.util.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;


public class TalkWebImpl extends TalkModel {

    private BlockWebImpl webblock = null;
    private Manage       mgr      = null;

    public TalkWebImpl () {
        mgr = LTUtils.createManage (); // keep for session
    }

    public void processRequest (String request) {
	if (request == null) request = "";
	if (request.equals ("clear"))  { clear    (); }
	if (request.equals ("edit"))   { retrieve (); }
	if (request.equals ("view"))   { retrieve (); }
	if (request.equals ("copy"))   { retrieve (); setKey (getAutoKey ()); }
	if (request.equals ("add"))    { add      (); }
	if (request.equals ("add++"))  { add      (); clear (); }
	if (request.equals ("add+="))  { add      (); setKey (getAutoKey ()); }
	if (request.equals ("change")) { change   (); }
	if (request.equals ("delete")) { delete   (); }
    }

    public void clear () {
      super.clear ();
      setBlockChecked (false);
    }

    public void retrieve () {
	try {
	    TalkModel talk = mgr.getTalkModel (getKey ());
            if (talk == null) throw new FinderException ("Item could not be found");
	    setModel (talk);
            setBlockChecked (!mgr.getBlockKeysByTalkList (getKey ()).isEmpty ()); // we assume that any matching block is ours
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null);
	} catch (FinderException fe) { LTUtils.handleException (fe, this, null, getKey ());
	}
    }

    public void add () {
	try {
	    String key = mgr.createTalk (getModel ());
	    TalkModel talk = mgr.getTalkModel (key);
            if (talk == null) throw new CreateException ("Created item disappeared");
	    setModel (talk); // the key may have been generated
            if (isBlockChecked ()) addBlock ();
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null);
	} catch (CreateException ce) { LTUtils.handleException (ce, this, null, getKey ());
	}
    }

    public void change () {
	try {
	    mgr.setTalkModel (getModel ());
            if (isBlockChecked ()) addBlock    ();
            else                   deleteBlock ();
            // /* internal check, removed:*/ setBlockChecked (!mgr.getBlockKeysByTalkList (getKey ()).isEmpty ());
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null);
	} catch (CreateException ce) { LTUtils.handleException (ce, this, null, getKey ());
	}
    }

    public void delete () {
	try {
	    mgr.removeTalk (getKey ());
            deleteBlock ();
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null);
	} catch (RemoveException de) { LTUtils.handleException (de, this, null, getKey ());
	}
    }

    /** @return a Collection of short TalkModel(s) */
    public Collection getModels () {
	try {
	    return mgr.getTalkShortmodelsAll ();
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null); return null;
	}
    }

    /** @return a Collection of short TalkModel(s) belonging to a given Type */
    public Collection getModels (String typeKey) {
	try {
	    Collection keys = mgr.getTalkKeysByTypeKey (typeKey);
	    ArrayList result = new ArrayList (keys.size ());
            for (Iterator i = keys.iterator (); i.hasNext (); )
              result.add (mgr.getTalkShortmodel ((String) i.next()));
            return result;
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null); return null;
	}
    }

    // Convenience web menues stuff follows
    private String sepDisplay    = "----------------";
    public  String getSepDisplay ()                  { return sepDisplay; }
    public  void   setSepDisplay (String sepDisplay) { this.sepDisplay = sepDisplay; }

    private String selected      = "";
    public  String getSelected   ()                  { return selected.equals ("") ? null : selected; }
    public  void   setSelected   (String selected)   { this.selected = selected; }

    /** @return a menue listing all talk keys, listing dangling talks (ie. in no block) first */
    public WebMenu getAllTalksMenu () {
	try {
	    return new WebMenu ("", sepDisplay, mgr.getDanglingTalkKeys (), mgr.getTalkKeysAll (), getSelected ());
	} catch (RemoteException re) { LTUtils.handleException (re, this, null, null); return null;
	}
    }

    /** @return a menue listing all authors of this talk */
    public WebMenu getAuthorsMenu () {
	return new WebMenu (getAuthors ());
    }

    /** @return a menue listing of all states with talk's state selected */
    public WebMenu getStateMenu () {
        WebMenu menue = new WebMenu ();
        menue.add ("private"); menue.add ("draft"); menue.add ("released"); menue.add ("final");
        menue.selectFirst (getState ());
	return menue;
    }

    // Implementation stuff for automatic block creation follows
    private boolean blockChecked = false;
    public void    setBlockChecked       (boolean blockChecked)      { this.blockChecked = blockChecked; }
    public boolean isBlockChecked        ()                          { return blockChecked; }
    public void    setBlockCheckBoxState (String blockCheckBoxState) { setBlockChecked ("checked".equals (blockCheckBoxState)); }
    public String  getBlockCheckBoxState ()                          { return isBlockChecked () ? "checked" : ""; }

    private void addBlock () {
        try {
            if (mgr.getBlockKeysByTalkList (getKey ()).isEmpty ()) {
                if (webblock == null) webblock = new BlockWebImpl ();
                webblock.clear ();
                webblock.setBlockType ("talk");
                webblock.setTalkList (getKey ());
                webblock.add ();
            }
        } catch (RemoteException re) { LTUtils.handleException (re, this, "Internal: problem while checking for existance of block for", getKey ());
        }
    }

    private void deleteBlock () {
        try {
            for (Iterator i = mgr.getBlockKeysByTalkList (getKey ()).iterator (); i.hasNext (); ) {
                mgr.removeBlock ((String) i.next ());
            }
        } catch (RemoteException re) { LTUtils.handleException (re, this, "Internal: problem while trying to delete block for",   getKey ());
        } catch (RemoveException de) { LTUtils.handleException (de, this, "Internal: could not delete corresponding block for",   getKey ());
        }
    }
}
