package com.livis.livingtalks.web;

import java.util.*;

public class WebMenu extends LinkedList {

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    public WebMenu (String sepValue, String sepDisplay, Collection c, String selected) {
	super ();
	setSepValue (sepValue);
	setSepDisplay (sepDisplay);
	addAll (c);
        selectFirst (selected);
    }
    public WebMenu (String sepValue, String sepDisplay, Collection c1, Collection c2, String selected) {
	this (sepValue, sepDisplay, c1);
        if (!c1.isEmpty () && !c2.isEmpty ()) addSeparator ();
	addAll (c2);
        selectFirst (selected);
    }
    public WebMenu (String sepValue, String sepDisplay, Collection c) { this (defaultSepValue, defaultSepDisplay, c, null); }
    public WebMenu (String sepValue, String sepDisplay)               { this (sepValue,        sepDisplay,        defaultItems); }
    public WebMenu (Collection c1, Collection c2, String selected)    { this (defaultSepValue, defaultSepDisplay, c1, c2, selected); }
    public WebMenu (Collection c1, Collection c2)                     { this (defaultSepValue, defaultSepDisplay, c1, c2, null); }
    public WebMenu (Collection c, String selected)                    { this (defaultSepValue, defaultSepDisplay, c, selected); }
    public WebMenu (Collection c)                                     { this (defaultSepValue, defaultSepDisplay, c); }
    public WebMenu ()                                                 { this (defaultSepValue, defaultSepDisplay, defaultItems); }

    private static String     defaultSepValue   = "";
    private static String     defaultSepDisplay = "----------------";
    private static Collection defaultItems      = new ArrayList (0);

    //----------------------------------------------------------------------
    // properties
    //----------------------------------------------------------------------
    private String sepValue;
    public String getSepValue () { return sepValue; }
    public void setSepValue (String sepValue) { this.sepValue = sepValue; }

    private String sepDisplay;
    public String getSepDisplay () { return sepDisplay; }
    public void setSepDisplay (String sepDisplay) { this.sepDisplay = sepDisplay; }

    //----------------------------------------------------------------------
    public void unselect () {
        for (Iterator i = this.iterator (); i.hasNext (); ) ((WebMenu.Item) i.next ()).setSelected (false);
    }

    public void selectFirst (String selected) {
        if (selected == null) return;
        for (Iterator i = this.iterator (); i.hasNext (); ) {
	    WebMenu.Item item = (WebMenu.Item) i.next ();
            if (selected.equals (item.getValue ())) {
                item.setSelected (true);
                break;
            }
        }
    }

    public String toString () {
        StringBuffer options = new StringBuffer (4000);
        for (Iterator i = this.iterator (); i.hasNext (); ) {
	    WebMenu.Item item = (WebMenu.Item) i.next ();
            options.append ("<option value=\"" + item.getValue () + "\"");
            if (item.isSelected ()) options.append (" selected");
            options.append (">" + item.getDisplay () + "</option>\n");
        }
        return options.toString ();
    }

    //----------------------------------------------------------------------
    public boolean add          (Item   item)                  { return super.add (item); }
    public boolean add          (String value, String display) { return add (new Item (value, display)); }
    public boolean add          (              String display) { return add (new Item (       display)); }
    public boolean addSeparator ()                             { return add (getSepValue (), getSepDisplay ()); }
    public boolean add          (Object object)                {
	if (object instanceof Item) {
	    return add ((Item) object);
	} else if (object instanceof String) {
	    return add ((String) object);
	} else {
	    throw new IllegalArgumentException ("cannot add non-Item/non-String objects (here: "
						+ object.getClass ().getName () + ") to Web menu");
	}
    }
    public boolean addAll (Collection c) {
	if (c.isEmpty ()) {
	    return false;
	}
	for (Iterator i = c.iterator (); i.hasNext (); ) {
	    add (i.next ());
	}
	return true;
    }

    //----------------------------------------------------------------------
    public static class Item {

	//----------------------------------------------------------------------
	// constructors
	//----------------------------------------------------------------------
	public Item (String value, String display, boolean selected) { setValue (value); setDisplay (display); setSelected (selected); }
	public Item (String value, String display)                   { this (value, display, false); }
	public Item (              String display)                   { this (null,  display, false); }
	public Item ()                                               { this (null,  null,    false); }

	//----------------------------------------------------------------------
	// properties
	//----------------------------------------------------------------------
	private String value;
	public String getValue () { return (null == value) ? getDisplay () : value; }
	public void setValue (String value) { this.value = value; }

	private String display;
	public String getDisplay () { return display; }
	public void setDisplay (String display) { this.display = display; }

	private boolean selected;
	public boolean isSelected () { return selected; }
	public void setSelected (boolean selected) { this.selected = selected; }
    }
}
