/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.xml;

import com.livis.livingtalks.model.*;
import com.livis.livingtalks.ejb.*;
import com.livis.flabes.util.StringUtils;
import com.livis.flabes.util.beans.*;
import com.livis.flabes.xml.*;
import java.util.*;
import java.lang.reflect.*;
import javax.ejb.*;
import org.w3c.dom.*;


public class DOMContainer implements DOMable {

    public static final String REVISION = "1.1";  // Import is only possible from export with same revision
    public static final int    IMPORT   = 1;      // replace entire database, counterpart to export
    public static final int    ADD      = 2;      // insert only new records, does not override existing ones
    public static final int    CHANGE   = 3;      // inserts new and overrides existing

    private Manage             mgr;
    private int                mode;
    private boolean            ignoreError;
    private ImportResult       result;
    private int                maxops;

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    public DOMContainer (Manage mgr)                                 { this (mgr, null, IMPORT, new ImportResult (), false, Integer.MAX_VALUE); }
    public DOMContainer (Manage mgr, String containerName)           { this (mgr, containerName, IMPORT, new ImportResult (), false, Integer.MAX_VALUE); }
    public DOMContainer (Manage mgr, String containerName, int mode, ImportResult result, boolean ignoreError, int maxops) {
      this.mgr         = mgr;
      setContainerName (containerName);
      this.mode        = mode;
      this.result      = result;
      this.ignoreError = ignoreError;
      this.maxops      = maxops;
    }

    //----------------------------------------------------------------------
    // properties
    //----------------------------------------------------------------------
    private String containerName, className;
    public final String getContainerName ()                     { return containerName; }
    public final void   setContainerName (String containerName) { this.containerName = containerName; className = StringUtils.capitalize (containerName); }

    //----------------------------------------------------------------------
    // interface DOMable
    //----------------------------------------------------------------------
    public void fromElement (Element elem) {
	try {
	    // FIXME: always clear, always clear all, insert (error if
	    // duplicate), keep-old-if-found, replace (error if not
	    // found), replace (ok if not found)

            if (mode == IMPORT) {
                // clear all old content...
    	        Method get_KeysAll = Manage.class.getMethod ("get" + className + "KeysAll", null);
                Collection oldcontent = (Collection) get_KeysAll.invoke (mgr, null);
    	        for (Iterator i = oldcontent.iterator (); i.hasNext (); ) {
                    try {
                        String key = (String) i.next ();
                        if (maxops <= 0) throw new LivingTalksException ("Import limit reached! You are not the superuser...<br>\n" + getContainerName () + " not deleted: " + key, this, null);
                        try {
                            Method remove_ = Manage.class.getMethod ("remove" + className, new Class [] { String.class });
                            remove_.invoke (mgr, new String [] { key });
                            result.deleted ++;
                            maxops --;
                        } catch (Exception ex) { LTUtils.handleException (ex, this, getContainerName () + " not deleted: ", key);
                        }
                    } catch (LivingTalksException le) {
                        result.notdeleted ++;
                        result.exceptions.add (le);
                        if (!ignoreError) throw le;
                    }
    	        }
            }
	    for (Iterator i = new ElementListIterator (elem); i.hasNext (); ) {
                boolean change = false;
                try {
                    String  key         = null;
                    try {
		        Element ejbElement  = (Element) i.next ();
	                Class modelClass    = Class.forName ("com.livis.livingtalks.model." + className + "Model");
                        AbstractModel model = (AbstractModel) modelClass.newInstance ();
		        new DOMToBean (model, ejbElement);
                        key = (model != null) ? model.getKey () : (ejbElement != null ? ejbElement.getAttribute ("name") : "?");
                        if (maxops <= 0) throw new LivingTalksException ("Import limit reached! You are not the superuser...<br>\n" + getContainerName () + " not inserted: " + key, this, null);
                        try {
	                    Method create_ = Manage.class.getMethod ("create" + className, new Class [] { model.getClass () });
                            create_.invoke (mgr, new Object [] { model });
                            result.created ++;
                            maxops --;
                        } catch (InvocationTargetException ie) {
                            change = mode == CHANGE && ie.getTargetException () instanceof DuplicateKeyException;
                            if (!change) throw ie;
                        }
                        if (change) {
	                    Method set_Model = Manage.class.getMethod ("set" + className + "Model", new Class [] { model.getClass () });
                            set_Model.invoke (mgr, new Object [] { model });
                            result.changed ++;
                            maxops --;
                        }
                    } catch (Exception ex) {
                        LTUtils.handleException (ex, this, getContainerName () + " not created: ", key);
                    }
                } catch (LivingTalksException le) {
                    if (change) result.notchanged ++;
                    else        result.notcreated ++;
                    result.exceptions.add (le);
                    if (!ignoreError) throw le;
                }
	    }
	} catch (Exception ex) {
            result.exceptions.add (new LivingTalksException ("Cannot create container " + getContainerName (), this, ex));
	    if (!ignoreError) LTUtils.handleException (ex, this, "Cannot create container " + getContainerName () + " from DOM element", null);
	}
    }

    public Element toElement (Document doc, String tagName) {
	try {
	    Element containerElement = doc.createElement (tagName);
	    containerElement.setAttribute ("declared-type", "com.livis.livingtalks.model." + className + "Model");
	    Method get_KeysAll = Manage.class.getMethod ("get" + className + "KeysAll", null);
            Collection allEJBKeys = (Collection) get_KeysAll.invoke (mgr, null);
	    for (Iterator i = allEJBKeys.iterator (); i.hasNext (); ) {
                String key = (String) i.next ();
	        Method get_Model = Manage.class.getMethod ("get" + className + "Model", new Class [] { String.class });
                AbstractModel model = (AbstractModel) get_Model.invoke (mgr, new String [] { key });
		Element ejbElement = new BeanToDOM (model, doc, getContainerName ()).toElement ();
		ejbElement.setAttribute ("name", key);
		containerElement.appendChild (ejbElement);
	    }
	    return containerElement;
	} catch (Exception e) {
	    LTUtils.handleException (e, this, "Cannot create DOM element for container " + getContainerName (), null);
	    return null;
	}
    }
    public Element toElement (Document doc) { return toElement (doc, getTagName ()); }

    public String getTagName () { return getContainerName () + "s"; }

    //----------------------------------------------------------------------
    // test
    //----------------------------------------------------------------------
    public static void main (String[] args) {
	try {
            Manage mgr  = LTUtils.createManage ();
	    Object bean = new DOMContainer (mgr, "author");

	    Element  asDOM = new BeanToDOM (bean).toElement ();
	    DOMUtils.writeAsXML (asDOM);

	    Object newBean = new DOMContainer (mgr, "author");
	    new DOMToBean (newBean, asDOM);

	    Element  newAsDOM = new BeanToDOM (newBean).toElement ();
	    DOMUtils.writeAsXML (newAsDOM);

	    System.exit (0);
	} catch (Exception e) {
	    e.printStackTrace (System.err);
	    System.exit (-1);
	}
    }
}
