/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Falk Langhammer
 * @version 1.0
 */
package com.livis.livingtalks.xml;

import com.livis.livingtalks.model.*;
import java.io.*;
import java.net.*;

import org.xml.sax.*;
import org.apache.xalan.xslt.*;
import org.apache.xalan.xslt.Process;

public class HtmlViewFromXml {

  XSLTProcessor processor = null;
  String base = null;
  URL xml_url = null;
  URL xsl_url = null;

  /** use this bean only within a JSP page of scope 'page'. It has not init() method!
   */
  public HtmlViewFromXml () {
    try {
      this.processor = XSLTProcessorFactory.getProcessor ();
    } catch (SAXException ex) { LTUtils.handleException (ex, this, null, null);
    }
  }

  public String getBase   ()            { return base; }
  public void   setBase   (String base) { this.base = base; }  // must be externalized URL or "./xxx"

  public URL    getXmlUrl ()            { return xml_url; }
  public void   setXmlUrl (URL xml_url) { this.xml_url = xml_url; } // must be absolute URL
  public void   setXml    (String xml)  { try { setXmlUrl (Process.getURLFromString (xml, getBase ())); } catch (SAXException ex) { LTUtils.handleException (ex, this, null, xml); } }

  public URL    getXslUrl ()            { return xsl_url; }
  public void   setXslUrl (URL xsl_url) { this.xsl_url = xsl_url; } // must be absolute URL
  public void   setXsl    (String xsl)  { try { setXslUrl (Process.getURLFromString (xsl, getBase ())); } catch (SAXException ex) { LTUtils.handleException (ex, this, null, xsl); } }

  // Layout parameters (passed on to XSLT style processor)
  public  void   setStyleCutlen           (String styleCutlen)           { this.styleCutlen = styleCutlen; }
  public  String getStyleCutlen           ()                             { return styleCutlen; }
  private String styleCutlen              = "";

  public  void   setStylePagenumber       (String stylePagenumber)       { this.stylePagenumber = stylePagenumber; }
  public  String getStylePagenumber       ()                             { return stylePagenumber; }
  private String stylePagenumber          = "";

  public  void   setStyleHourh            (String styleHourh)            { this.styleHourh = styleHourh; }
  public  String getStyleHourh            ()                             { return styleHourh; }
  private String styleHourh               = "120";

  public  void   setStyleCw               (String styleCw)               { this.styleCw = styleCw; }
  public  String getStyleCw               ()                             { return styleCw; }
  private String styleCw                  = "180";

  private boolean styleDraftsChecked              = false;
  public void    setStyleDraftsChecked            (boolean styleDraftsChecked)           { this.styleDraftsChecked = styleDraftsChecked; }
  public boolean isStyleDraftsChecked             ()                                     { return styleDraftsChecked; }
  public void    setStyleDraftsCheckBoxState      (String styleDraftsCheckBoxState)      { setStyleDraftsChecked ("checked".equals (styleDraftsCheckBoxState)); }
  public String  getStyleDraftsCheckBoxState      ()                                     { return isStyleDraftsChecked () ? "checked" : ""; }

  private boolean styleSessautChecked             = false;
  public void    setStyleSessautChecked           (boolean styleSessautChecked)          { this.styleSessautChecked = styleSessautChecked; }
  public boolean isStyleSessautChecked            ()                                     { return styleSessautChecked; }
  public void    setStyleSessautCheckBoxState     (String styleSessautCheckBoxState)     { setStyleSessautChecked ("checked".equals (styleSessautCheckBoxState)); }
  public String  getStyleSessautCheckBoxState     ()                                     { return isStyleSessautChecked () ? "checked" : ""; }

  private boolean styleDoublepageChecked          = false;
  public void    setStyleDoublepageChecked        (boolean styleDoublepageChecked)       { this.styleDoublepageChecked = styleDoublepageChecked; }
  public boolean isStyleDoublepageChecked         ()                                     { return styleDoublepageChecked; }
  public void    setStyleDoublepageCheckBoxState  (String styleDoublepageCheckBoxState)  { setStyleDoublepageChecked ("checked".equals (styleDoublepageCheckBoxState)); }
  public String  getStyleDoublepageCheckBoxState  ()                                     { return isStyleDoublepageChecked () ? "checked" : ""; }

  // utility methods
  public void setStylesheetParam (String name, String expression) {
    processor.setStylesheetParam (name, expression);
  }
  public void unsetStylesheetParams () {
    processor.reset ();
  }

  public void write () {
    write (new PrintWriter (System.out));
  }

  // must call setters first!
  public void write (Writer out) {
    try {
      if (getXmlUrl () == null || getXslUrl () == null || processor == null)
        throw new LivingTalksException ("Creation of HTML from XML failed!<br>with XML=[" + getXmlUrl ().toExternalForm () + "]<br>and XSLT=[" + getXslUrl ().toExternalForm () + "]", this, null);

      setStylesheetParam ("sessaut",    isStyleSessautChecked    () ? "true()"  : "false()");
      setStylesheetParam ("doublepage", isStyleDoublepageChecked () ? "true()"  : "false()");
      setStylesheetParam ("nodrafts",   isStyleDraftsChecked     () ? "false()" : "true()" ); // <<< inverted!
      if (!getStylePagenumber ().equals ("")) setStylesheetParam ("pagenumber", getStylePagenumber ());
      if (!getStyleCutlen     ().equals ("")) setStylesheetParam ("cutlen",     getStyleCutlen     ());
      if (!getStyleHourh      ().equals ("")) setStylesheetParam ("hourh",      getStyleHourh      ());
      if (!getStyleCw         ().equals ("")) setStylesheetParam ("cw",         getStyleCw         ());

      processor.process (new XSLTInputSource (getXmlUrl ().toExternalForm ()),
                         new XSLTInputSource (getXslUrl ().toExternalForm ()),
                         new XSLTResultTarget (out));
      out.flush ();
      processor.reset ();
    } catch (SAXException ex) { LTUtils.handleException (ex, this, null, null);
    } catch (IOException ie)  { LTUtils.handleException (ie, this, null, null);
    }
  }

  public String getToday () {
    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat ("MMM d, yyyy");
    return formatter.format (new java.util.Date ());
  }

  public static void main (String argv []) {
    HtmlViewFromXml html = new HtmlViewFromXml ();
    System.out.println ("test is run today=" + html.getToday ());
    // this works when in (JBuilder's) project root...
    html.setBase ("./index.html"); // optional, must be absolute URL if not "./xxx" or null
    html.setXml  ("webroot/publish/overview.xml");
    html.setXsl  ("webroot/publish/styles/overview.xsl");
    html.setStylesheetParam ("standalone", "false()");  html.setStylesheetParam ("gxoffset", "10"); html.setStylesheetParam ("gyoffset", "10");
    html.write   ();
  }
}
