/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.xml;

import com.livis.livingtalks.model.*;
import com.livis.livingtalks.ejb.Manage;
import com.livis.flabes.util.beans.*;
import com.livis.flabes.util.ejb.*;
import com.livis.flabes.xml.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class Import {

    private String       conferenceName;
    private String []    containerNames;
    private Manage       mgr;
    private int          mode;
    private int          maxops;
    private boolean      ignoreError;

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    public Import () {
	this ("Net.ObjectDays 2000", DOMContainer.IMPORT, false, Integer.MAX_VALUE, Export.allContainerNames);
    }

    public Import (String conference, int mode, boolean ignoreError) {
	this (conference, mode, ignoreError, Integer.MAX_VALUE, Export.allContainerNames);
    }

    public Import (String conference, int mode, boolean ignoreError, int maxops) {
	this (conference, mode, ignoreError, maxops, Export.allContainerNames);
    }

    public Import (String conferenceName, int mode, boolean ignoreError, int maxops, String [] containerNames) {
        this.mgr            = LTUtils.createManage ();
        this.mode           = mode;
        this.ignoreError    = ignoreError;
        this.maxops         = maxops;
	this.conferenceName = conferenceName;
	this.containerNames = containerNames;
    }

    //----------------------------------------------------------------------
    public ImportResult read (String inSpec) {
	try {
	    return read (new InputSource (new URL (inSpec).openConnection ().getInputStream ()));
	} catch (Exception ex) {
	    LTUtils.handleException (ex, this, null, null); return null;
	}
    }

    public ImportResult read (InputSource in) {
        ImportResult result = new ImportResult ();
	try {
	    Document doc            = DOMParserFactory.createDOMParser ().parse (in, false);
	    Element  saveSetElement = (Element) doc.getFirstChild ();
	    for (Iterator i = new ElementListIterator (saveSetElement); i.hasNext (); ) {
		Element child = (Element) i.next ();
		if ("entities".equals (child.getNodeName ())) {
		    Map nodeMap = new HashMap ();
		    for (Iterator c = new ElementListIterator (child); c.hasNext (); ) {
			Element containerNode = (Element) c.next ();
			String  containerName = containerNode.getNodeName ();
			if (containerName.endsWith ("s")) {
			    containerName = containerName.substring (0, containerName.length () - 1);
			}
			nodeMap.put (containerName, containerNode);
		    }
                    System.out.print ("importing");
		    for (int c = 0; c < containerNames.length; ++c) {
			Element containerNode = (Element) nodeMap.get (containerNames [c]);
			if (null != containerNode) {
                            System.out.print ("," + containerNames [c]);
                            int ops = maxops - (result.created + result.deleted + result.changed);
			    DOMContainer container = new DOMContainer (mgr, containerNames [c], mode, result, ignoreError, ops);
			    new DOMToBean (container, containerNode);
			} else if (mode == DOMContainer.IMPORT) {
                            result.exceptions.add (new LivingTalksException ("Warning: file contained no " + containerNames [c] + "s (its old data was not modified)", this));
			}
		    }
                    System.out.println (".");
		}
	    }
            return result;
	} catch (Exception ex) {
            result.exceptions.add (new LivingTalksException ("XML input (" + in + ") not read", this, ex));
            for (Iterator i = result.exceptions.iterator (); i.hasNext (); )
              System.err.println (((Exception) i.next ()).getMessage ());
	    if (!ignoreError) LTUtils.handleException (ex, this, null, null);
            return result;
	}
    }

    public ImportResult read () { return read (new InputSource (System.in)); }
}
