package com.livis.livingtalks.xml;

import com.livis.flabes.util.*;
import com.livis.flabes.util.beans.*;
import com.livis.flabes.xml.*;
import com.livis.livingtalks.ejb.*;
import com.livis.livingtalks.model.*;
import java.io.*;
import java.net.*;
import java.rmi.*;
import java.util.*;
import javax.ejb.*;
import org.w3c.dom.*;

public class Overview
    extends XMLOutput {

    public static final int CELLRASTER = 5;   // minimum difference of different rastervalues (in minutes)

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    public Overview (String fromSpec, String[] contextSpecs) { super (fromSpec, contextSpecs); }

    //----------------------------------------------------------------------
    // class XMLExport
    //----------------------------------------------------------------------
    public void print (PrintWriter out, Indenter indenter) throws Exception {
	if (isPrintHeader ()) out.println ("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
	out.println (indenter + "<overview>");

	printPages (out, indenter);

	out.println ("</overview>");
    }

    //----------------------------------------------------------------------
    // utilities
    //----------------------------------------------------------------------
    private void printPages (PrintWriter out, Indenter indenter)
	throws FinderException, RemoteException, MalformedURLException {

	Collection allTrackModels = mgr.getTrackShortmodelsAll ();
	SortedSet  pageNumbers    = new TreeSet (ArraysUtils.extract (allTrackModels, "getPage"));

	if (!pageNumbers.isEmpty ()) {
	    out.println (indenter.incLevel () + "<pages>");
	    indenter.incLevel ();
	    for (Iterator p = pageNumbers.iterator (); p.hasNext (); ) {
		int        pageNumber = ((Integer) p.next ()).intValue ();

                Tablemap tablemap = new Tablemap (pageNumber);

		out.println (indenter + "<page>");
		out.println (indenter.incLevel () + "<number>" + pageNumber + "</number>");

		int [] pageRasterStart = new int [] { Integer.MAX_VALUE }; // pass by ref
		int [] pageRasterEnd   = new int [] { Integer.MIN_VALUE }; // pass by ref
		int [] maxCol          = new int [] { 0 };                 // pass by ref

		printColumns (out, indenter, pageNumber, pageRasterStart, pageRasterEnd, maxCol, tablemap);

		if (Integer.MAX_VALUE == pageRasterStart [0]) pageRasterStart [0] = 0;
		if (Integer.MIN_VALUE == pageRasterEnd   [0]) pageRasterEnd   [0] = 0;

		out.println (indenter + "<rasterStart>" + pageRasterStart [0] + "</rasterStart>");
		out.println (indenter + "<rasterLength>" + (pageRasterEnd [0] - pageRasterStart [0])+ "</rasterLength>");
		out.println (indenter + "<maxColumn>" + maxCol [0] + "</maxColumn>");

                tablemap.print (out, indenter, "blockKey", 1, 1, maxCol [0], CELLRASTER, pageRasterStart [0], pageRasterEnd [0] - pageRasterStart [0]);

		out.println (indenter.decLevel () + "</page>");
	    }
	    out.println (indenter.decLevel () + "</pages>");
	    indenter.decLevel ();
	}
    }

    private void printColumns (PrintWriter out, Indenter indenter, int pageNumber,
			       int [] pageRasterStartRef, int [] pageRasterEndRef, int [] maxCol, Tablemap tablemap)
	throws FinderException, RemoteException, MalformedURLException {

        Collection pageTrackKeys = mgr.getTrackKeysByPage (pageNumber);
	SortedSet columnNumbers  = new TreeSet ();
        for (Iterator i = pageTrackKeys.iterator (); i.hasNext (); )
          columnNumbers.add (new Integer (mgr.getTrackShortmodel ((String) i.next ()).getTrackCol ()));

	if (!columnNumbers.isEmpty ()) {
	    out.println (indenter + "<columns>");
	    indenter.incLevel ();
	    for (Iterator c = columnNumbers.iterator (); c.hasNext (); ) {
		out.println (indenter + "<column>");
		int columnNumber = ((Integer) c.next ()).intValue ();
		out.println (indenter.incLevel () + "<number>" + columnNumber + "</number>");

                Collection trackKeys   = mgr.getTrackKeysByPageAndTrackCol (pageNumber, columnNumber);
		Collection trackModels = new ArrayList (trackKeys.size ());
                for (Iterator i = trackKeys.iterator (); i.hasNext (); )
                  trackModels.add (mgr.getTrackShortmodel ((String) i.next ()));
		String [] columnTitle = new String [1];

		printTracks (out, indenter, trackModels, pageRasterStartRef, pageRasterEndRef, columnTitle, maxCol, tablemap);

		if (null != columnTitle [0])
		    out.println (indenter + "<title>" + columnTitle [0] + "</title>");
		out.println (indenter.decLevel () + "</column>");
	    }
	    out.println (indenter.decLevel () + "</columns>");
	}
    }
}
