/**
 * Title:        Living Talks Web Conference Tool<p>
 * Description:  Tool using J2EE standards<p>
 * Copyright:    Copyright (c) Dr. Falk Langhammer<p>
 * Company:      Living Pages Research GmbH<p>
 * @author Oliver Imbusch
 * @version 1.0
 */
package com.livis.livingtalks.xml;

import com.livis.flabes.util.*;
import com.livis.flabes.util.beans.*;
import com.livis.flabes.xml.*;
import com.livis.livingtalks.ejb.*;
import com.livis.livingtalks.model.*;
import java.io.*;
import java.net.*;
import java.rmi.*;
import java.util.*;
import javax.ejb.*;
import org.w3c.dom.*;

public abstract class XMLOutput {

    protected Manage    mgr;
    protected String    fromSpec;
    protected String [] contextSpecs;

    public XMLOutput ()                                        { this (null, null); }
    public XMLOutput (String fromSpec, String [] contextSpecs) {
        this.mgr          = LTUtils.createManage ();
	this.fromSpec     = fromSpec;
	this.contextSpecs = contextSpecs;
    }

    private boolean printHeader    = true;
    public  boolean isPrintHeader  ()                    { return printHeader; }
    public  void    setPrintHeader (boolean printHeader) { this.printHeader = printHeader; }

    public abstract void print (PrintWriter out, Indenter indenter) throws Exception;

    public void write (Writer out) {
	try {
	    PrintWriter pout = (out instanceof PrintWriter) ? (PrintWriter) out : new PrintWriter (out);
	    pout.flush ();
	    print (pout, new Indenter ("", "  "));
	    pout.flush ();
	} catch (Exception e) {
	    LTUtils.handleException (e, this, "Cannot create XML output", null);
	}
    }

    public void write () { write (new PrintWriter (System.out)); }

    public void write (String outSpec) {
	try {
	    OutputStream os = URLUtils.isRelative (outSpec)
		// use absolute path to see useful error messages
		? new FileOutputStream (new File (outSpec).getAbsolutePath ())
		: new URL (outSpec).openConnection ().getOutputStream ();
	    write (new PrintWriter (os));
	    os.close ();
	} catch (Exception e) {
	    LTUtils.handleException (e, this, "Cannot create XML output", null);
	}
    }

    //----------------------------------------------------------------------
    // utilities
    //----------------------------------------------------------------------
    protected void printContainer (PrintWriter out, Indenter indenter, String containerName) {
	DOMContainer container = new DOMContainer (mgr, containerName);
        Document doc           = DocumentFactory.createDocument ();
        Element  containerElem = container.toElement (doc);
	new DOMWriter (out, indenter).print (containerElem).println ();
    }

    protected void printTracks (PrintWriter out, Indenter indenter, Collection trackModels,
				int[] pageRasterStartRef, int[] pageRasterEndRef,
				String[] columnTitleRef, int[] maxColRef)
	throws FinderException, RemoteException, MalformedURLException {
        printTracks (out, indenter, trackModels, pageRasterStartRef, pageRasterEndRef, columnTitleRef, maxColRef, null);
    }

    protected void printTracks (PrintWriter out, Indenter indenter, Collection trackModels,
				int[] pageRasterStartRef, int[] pageRasterEndRef,
				String[] columnTitleRef, int[] maxColRef, Tablemap tablemap)
	throws FinderException, RemoteException, MalformedURLException {

	int minExtraTrackCols = Integer.MAX_VALUE;

	if (!trackModels.isEmpty ()) {
	    Document doc           = DocumentFactory.createDocument ();
	    Element  tracksElement = doc.createElement ("tracks");
	    for (Iterator tm = trackModels.iterator (); tm.hasNext (); ) {
		TrackModel trackModel = (TrackModel) tm.next ();

		if (null != maxColRef) {
		    maxColRef [0] = Math.max (maxColRef [0], trackModel.getTrackCol ());
		}

	        if (columnTitleRef != null) {
		    if (trackModel.getExtraTrackCols () < minExtraTrackCols) {
                        if (!StringUtils.isNullOrEmpty (trackModel.getTitle ()))
			    columnTitleRef [0] = trackModel.getTitle ();
                            minExtraTrackCols = trackModel.getExtraTrackCols ();
		    }
		}

		Element trackElement = new BeanToDOM (trackModel, doc, "track").toElement ();
		tracksElement.appendChild (trackElement);

		appendBlocks (trackElement, trackModel, doc, pageRasterStartRef, pageRasterEndRef, tablemap);
	    }
	    new DOMWriter (out, indenter).print (tracksElement).println ();
	}
    }

    protected void appendBlocks (Element trackElement, TrackModel trackModel, Document doc,
				 int [] pageRasterStartRef, int [] pageRasterEndRef)
	throws FinderException, RemoteException, MalformedURLException {
        appendBlocks (trackElement, trackModel, doc, pageRasterStartRef, pageRasterEndRef, null);
    }

    protected void appendBlocks (Element trackElement, TrackModel trackModel, Document doc,
				 int [] pageRasterStartRef, int [] pageRasterEndRef, Tablemap tablemap)
	throws FinderException, RemoteException, MalformedURLException {

	SortedSet pitemModels = new TreeSet  (new
            Comparator () {
		public int compare (Object lhsO, Object rhsO) {
		    PitemModel lhs = (PitemModel) lhsO;
		    PitemModel rhs = (PitemModel) rhsO;
		    int             order = lhs.getStart ().compareTo (rhs.getStart ());
		    if (order == 0) order = lhs.getKey   ().compareTo (rhs.getKey   ());
		    return order;
		}
	    }
        );
        Collection pitems = mgr.getPitemKeysByTrackKey (trackModel.getKey ());
        for (Iterator i = pitems.iterator (); i.hasNext (); )
          pitemModels.add (mgr.getPitemShortmodel ((String) i.next ()));
	if (!pitemModels.isEmpty ()) {
	    Element blocksElement = doc.createElement ("blocks");
	    trackElement.appendChild (blocksElement);
	    for (Iterator pm = pitemModels.iterator (); pm.hasNext (); ) {
		PitemModel pitemModel = (PitemModel) pm.next ();
		java.sql.Time nextTalkStart = pitemModel.getStart ();
                BlockModel blockModel = mgr.getBlockShortmodel (pitemModel.getBlockKey ());
		if (null != blockModel) {
		    Element blockElement = new BeanToDOM (blockModel, doc, "block").toElement ();
		    DOMUtils.remove (blockElement, "authorKey");
		    AuthorModel authorModel = mgr.getAuthorShortmodel (blockModel.getAuthorKey ());
		    appendAuthor (blockElement, authorModel, doc, "author");

		    SQLTimeEditor timeEditor   = new SQLTimeEditor ();
		    timeEditor.setValue (pitemModel.getStart ());
		    DOMUtils.append (blockElement, "startTime", timeEditor.getAsText ());
		    java.sql.Time stopTime = new java.sql.Time (DateUtils.addMinutes (pitemModel.getStart (),
										      blockModel.getDuration ()).getTime ());
		    timeEditor.setValue (stopTime);
		    int rasterStart  = DateUtils.getMinuteOfDay (pitemModel.getStart ());
		    int rasterLength = blockModel.getDuration ();
		    DOMUtils.append (blockElement, "stopTime",     timeEditor.getAsText ());
		    DOMUtils.append (blockElement, "rasterStart",  rasterStart);
		    DOMUtils.append (blockElement, "rasterLength", rasterLength);

		    blocksElement.appendChild (blockElement);

		    appendTalks (blockElement, blockModel.getTalks (), nextTalkStart, doc);

		    if (null != pageRasterStartRef)
			pageRasterStartRef [0] = Math.min (pageRasterStartRef [0], rasterStart);
		    if (null != pageRasterEndRef)
			pageRasterEndRef   [0] = Math.max (pageRasterEndRef   [0], rasterStart + rasterLength);
                    if (tablemap != null)
                        tablemap.add (blockModel.getKey (), trackModel.getPage (), trackModel.getTrackCol (), trackModel.getExtraTrackCols () + 1, rasterStart, rasterLength);
		}
	    }
	}
    }

    protected boolean printTalkAbstract = false;
    protected boolean printTalkType     = true;
    protected boolean printTalkPitem    = false;

    protected void appendTalks (Element parent, Collection talkKeys, java.sql.Time nextTalkStart, Document doc)
	throws FinderException, RemoteException, MalformedURLException {

	if (!talkKeys.isEmpty ()) {
	    Element talksElement = doc.createElement ("talks");
	    parent.appendChild (talksElement);

	    Collection allBlockModels = null;
            if (printTalkPitem) allBlockModels = mgr.getBlockShortmodelsAll ();
	    for (Iterator t = talkKeys.iterator (); t.hasNext (); ) {
		TalkModel talkModel = mgr.getTalkModel ((String) t.next ());
		if (null != talkModel) {
		    // don't touch homePageUrl
		    talkModel.setSlidesUrl   (relativeURLSpec (talkModel.getSlidesUrl ()));
		    talkModel.setPaperUrl    (relativeURLSpec (talkModel.getPaperUrl ()));
		    talkModel.setDraftUrl    (relativeURLSpec (talkModel.getDraftUrl ()));

		    if (printTalkAbstract) {
			talkModel.setAbstractHtml (LTUtils.preformattedToHTML (talkModel.getAbstractHtml ()));
		    }

		    Element talkElement = new BeanToDOM (talkModel, doc, "talk").toElement ();
		    if (!printTalkAbstract) {
			DOMUtils.remove (talkElement, "abstractHtml");
		    }
		    talksElement.appendChild (talkElement);

		    appendAuthors (talkElement, talkModel, doc);
		    if (printTalkType) {
			nextTalkStart = appendType (talkElement, talkModel, nextTalkStart, doc);
		    }

		    if (printTalkPitem) {
			Element pitemsElement = doc.createElement ("pitems");
			talkElement.appendChild (pitemsElement);

			for (Iterator bm = allBlockModels.iterator (); bm.hasNext (); ) {
			    BlockModel blockModel = (BlockModel)bm.next ();
			    if (blockModel.getTalks ().contains (talkModel.getKey ())) {
                                Collection pitems = mgr.getPitemKeysByBlockKey (blockModel.getKey ());
				for (Iterator pm = pitems.iterator (); pm.hasNext (); ) {
				    PitemModel pitemModel = mgr.getPitemShortmodel ((String) pm.next ());

				    Element pitemElement = doc.createElement ("pitem");
				    pitemsElement.appendChild (pitemElement);

				    TrackModel trackModel = mgr.getTrackShortmodel (pitemModel.getTrackKey ());
				    if (null != trackModel) {
					pitemElement.appendChild (new BeanToDOM (trackModel, doc, "track").toElement ());
				    }

				    Element blockElement = new BeanToDOM (blockModel, doc, "block").toElement ();
				    pitemElement.appendChild (blockElement);

				    SQLTimeEditor timeEditor = new SQLTimeEditor ();
				    timeEditor.setValue (pitemModel.getStart ());
				    DOMUtils.append (blockElement, "startTime", timeEditor.getAsText ());

				    java.sql.Time blockStop = new java.sql.Time (DateUtils.addMinutes (pitemModel.getStart (),
												       talkModel.getDuration ()).getTime ());
				    timeEditor.setValue (blockStop);
				    DOMUtils.append (blockElement, "stopTime", timeEditor.getAsText ());

				    int rasterStart  = DateUtils.getMinuteOfDay (pitemModel.getStart ());
				    int rasterLength = talkModel.getDuration ();
				    DOMUtils.append (blockElement, "rasterStart",  rasterStart);
				    DOMUtils.append (blockElement, "rasterLength", rasterLength);

				    java.sql.Time talkStart = new java.sql.Time (pitemModel.getStart ().getTime ());
				    for (Iterator tk = blockModel.getTalks ().iterator (); tk.hasNext (); ) {
					String earlyTalkKey = (String) tk.next ();
					if (earlyTalkKey.equals (talkModel.getKey ())) {
					    break;
					}
                                        TalkModel earlyTalkModel = mgr.getTalkShortmodel (earlyTalkKey);
					if (earlyTalkModel != null)
                                          talkStart = new java.sql.Time (DateUtils.addMinutes (talkStart, earlyTalkModel.getDuration ()).getTime ());
				    }

				    timeEditor.setValue (talkStart);
				    DOMUtils.append (pitemElement, "startTime", timeEditor.getAsText ());
				    java.sql.Time talkStop
					= new java.sql.Time (DateUtils.addMinutes (talkStart,
										   talkModel.getDuration ()).getTime ());
				    timeEditor.setValue (talkStop);
				    DOMUtils.append (pitemElement, "stopTime", timeEditor.getAsText ());
				}
			    }
			}
		    }
		}
	    }
	}
    }

    private static Map          authorPropertyNames = new LinkedHashMap ();
    private static FastBeanInfo authorBI            = new FastBeanInfo (AuthorModel.class, true);
    static {
	authorPropertyNames.keySet ().addAll (Arrays.asList (new String[] { "key", "firstName", "lastName", "email", "affiliation" }));
    }

    protected void appendAuthors (Element talkElement, TalkModel talkModel, Document doc) throws FinderException, RemoteException {
	Collection authorKeys = talkModel.getAuthors ();
	if (!authorKeys.isEmpty ()) {
	    Element authorsElement = doc.createElement ("authors");
	    talkElement.appendChild (authorsElement);
	    for (Iterator a = authorKeys.iterator (); a.hasNext (); ) {
	        AuthorModel authorModel = mgr.getAuthorShortmodel ((String) a.next ());
	        appendAuthor (authorsElement, authorModel, doc, "author");
	    }
	}
    }

    protected void appendAuthor (Element parentElement, AuthorModel authorModel, Document doc, String tag) {
	if (null != authorModel) {
	    authorModel = (AuthorModel) BeanUtils.clone (authorModel, authorBI, authorPropertyNames);
	    parentElement.appendChild (new BeanToDOM (authorModel, doc, tag).toElement ());
	}
    }

    protected java.sql.Time appendType (Element talkElement, TalkModel talkModel, java.sql.Time talkStart, Document doc)
	throws FinderException, RemoteException, MalformedURLException {

	TypeModel typeModel = mgr.getTypeShortmodel (talkModel.getTypeKey ());
	if (null != typeModel) {
	    typeModel.setIconUrl     (relativeURLSpec (typeModel.getIconUrl ()));
	    typeModel.setTypePageUrl (relativeURLSpec (typeModel.getTypePageUrl ()));

	    Element   typeElement = new BeanToDOM (typeModel, doc, "type").toElement ();
	    DOMUtils.remove (typeElement, "editorInDutyList");
	    talkElement.appendChild (typeElement);
	}
	SQLTimeEditor timeEditor   = new SQLTimeEditor ();
	timeEditor.setValue (talkStart);
	DOMUtils.append (talkElement, "startTime", timeEditor.getAsText ());
	java.sql.Time nextTalkStart = new java.sql.Time (DateUtils.addMinutes (talkStart, talkModel.getDuration ()).getTime ());
	timeEditor.setValue (nextTalkStart);
	DOMUtils.append (talkElement, "stopTime", timeEditor.getAsText ());

	return nextTalkStart;
    }

    protected String relativeURLSpec (String spec) throws MalformedURLException {
	return StringUtils.isNullOrEmpty (spec) ? spec : URLUtils.relativeURLSpec (fromSpec, spec, contextSpecs);
    }
}
