<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <!--
	authors.xsl v1.0; July 24, 2000; tested with Xalan
	This stylesheet creates a list of all authors.
	LivingTalks conference content management tool (c) 2000 Living Pages Research GmbH, Munich, Germany
  -->
  <xsl:output encoding="iso-8859-1" method="html" indent="yes" omit-xml-declaration="yes"/>

  <xsl:param name="standalone" select="true()"/><!-- produce html file and not fragment -->
  <xsl:param name="css-gfile"  select="'../styles/node.css'"/><!-- path when published if standalone -->
  <xsl:param name="css-lfile"  select="'styles/authors.css'"/><!-- path when published if standalone -->
  <xsl:param name="title"      select="'Directory of Authors'"/><!-- title if standalone -->
  <xsl:param name="ppf"        select="''"/><!-- global path prefix for links to resources like images or help pages or stylesheets -->

  <xsl:template match="@*|node()" />

  <xsl:template match="/">
    <!--if--><xsl:choose><xsl:when test="$standalone">
      <html>
        <head>
          <xsl:comment>
		    This HTML file is automatically generated from XML. Do not edit!
		  </xsl:comment>
          <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
          <title><xsl:value-of select="$title"/></title>
		  <link rel="stylesheet" href="{$ppf}{$css-gfile}"/>
		  <link rel="stylesheet" href="{$css-lfile}"/>
        </head>
        <body bgcolor="#FFFFFF">
          <h1><xsl:value-of select="$title"/></h1>
	      <p class="tochint">(Click on any item to jump to the corresponding entry...)</p>
	      <p class="tochint">(Then, click on an author to go to additional info...)</p>
	      <hr/>
          <xsl:apply-templates/>
        </body>
      </html>
	<!--else--></xsl:when><xsl:otherwise>
      <xsl:comment>
		The following content is automatically generated from XML. Do not edit!
      </xsl:comment>
      <xsl:apply-templates/>
	<!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="authors">
    <h2 class="toctitle">Table of contents:</h2>
	<p class="tocline"><b>Information is available for the following <xsl:value-of select="count (author)"/> authors...</b></p>
    <xsl:for-each select="author">
      <p class="tocline">
        <a href="#{key}" class="plink"><xsl:apply-templates select="." mode="shortname"/></a>
		<xsl:if test="string (affiliation)">
		  <xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;(</xsl:text>
 		  <xsl:value-of select="affiliation"/>)
		</xsl:if>
      </p>
    </xsl:for-each>
	<hr/>
	<p><xsl:text> </xsl:text></p>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="author">
    <table width="90%" align="center" class="author"><tr><td>
      <a name="{key}"/>
      <h2 class="title">
        <xsl:if test="string (photoUrl)">
          <img alt="Photo" src="{$ppf}{photoUrl}" align="right" hspace="10" vspace="4" border="0"/>
        </xsl:if>
	    <xsl:apply-templates select="." mode="shortname"/>
      </h2>
      <xsl:apply-templates select="talks"/><!-- FIXME (delete) --><xsl:apply-templates select="talkKeys"/>
      <p class="addresstitle"><xsl:text>Address: </xsl:text></p>
      <div class="address">
        <xsl:apply-templates select="." mode="name"/>
        <xsl:apply-templates select="affiliation"/>
        <xsl:apply-templates select="address"/>
	  </div>
      <xsl:apply-templates select="email"/>
      <xsl:apply-templates select="homePageUrl"/>
      <xsl:apply-templates select="cvHtml"/>
	  <p class="url"><xsl:text>Go to </xsl:text><a href="#" class="simpleLink">top of page</a></p>
    </td></tr></table>
  </xsl:template>

  <xsl:template match="affiliation"><xsl:if test="string (.)">
    <p class="a-affiliation"><xsl:value-of select="."/></p>
  </xsl:if></xsl:template>

  <xsl:template match="address"><xsl:if test="string (.)">
    <pre class="a-address"><xsl:value-of select="."/></pre>
  </xsl:if></xsl:template>

  <xsl:template match="email"><xsl:if test="string (.)">
	<p class="url"><xsl:text>Email: </xsl:text><span class="a-email"><a href="mailto:{.}" class="simpleLink"><xsl:value-of select="."/></a></span></p>
  </xsl:if></xsl:template>

  <xsl:template match="homePageUrl"><xsl:if test="string (.)">
	<p class="url"><xsl:text>Homepage: </xsl:text><span class="a-homePageUrl">
	  <a href="{.}" target="_blank" class="simpleLink">
	    <xsl:if test="not (starts-with (., 'http'))">
          <xsl:attribute name="href">http://<xsl:value-of select="."/></xsl:attribute>
	    </xsl:if>
	    <xsl:value-of select="."/>
	  </a>
    </span></p>
  </xsl:if></xsl:template>

  <xsl:template match="cvHtml"><xsl:if test="string (.)">
    <div class="acv">
      <p class="acvtitle">Curriculum vitae:</p>
      <div class="acvbody">
	    <xsl:value-of select="." disable-output-escaping="yes"/>
	  </div>
    </div>
  </xsl:if></xsl:template>

  <xsl:template match="author" mode="name">
    <p class="a-name">
      <xsl:if test="starts-with (title, 'Prof') or starts-with (title, 'Dr')">
        <xsl:value-of select="title"/>
	    <xsl:text> </xsl:text>
	  </xsl:if>
      <xsl:if test="string (firstName)">
        <xsl:value-of select="firstName"/>
	    <xsl:text> </xsl:text>
	  </xsl:if>
      <xsl:value-of select="lastName"/>
      <xsl:if test="starts-with (title, 'Ph') or starts-with (title, 'M.D')">
	    <xsl:text> (</xsl:text>
        <xsl:value-of select="title"/>
	    <xsl:text>)</xsl:text>
	  </xsl:if>
	</p>
  </xsl:template>

  <xsl:template match="author" mode="shortname">
    <xsl:value-of select="lastName"/>
    <xsl:if test="string (firstName)">
	  <xsl:text>, </xsl:text>
      <xsl:value-of select="firstName"/>
	</xsl:if>
  </xsl:template>

  <xsl:template match="talks">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="talk">
	<p class="url"><xsl:text>Go to her/his talk: </xsl:text><span class="talk"><a href="talks.html#{key}.t" target="talks" class="simpleLink"><xsl:value-of select="title"/></a></span></p>
  </xsl:template>

  <!-- FIXME: (delete) --><xsl:template match="talkKeys"><xsl:apply-templates/></xsl:template><xsl:template match="talkKey"><p class="url"><xsl:text>Go to her/his </xsl:text><span class="talk"><a href="talks.html#{.}.t" target="talks" class="simpleLink">talk</a></span></p></xsl:template>

</xsl:stylesheet>