<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <!--
	overview.xsl v1.0; July 24, 2000; tested with Xalan
	This stylesheet creates an overview view using HTML 4.0.
	LivingTalks conference content management tool (c) 2000 Living Pages Research GmbH, Munich, Germany
  -->
  <xsl:output encoding="iso-8859-1" method="html" indent="yes" omit-xml-declaration="yes"/>

  <xsl:param name="standalone" select="true()"/><!-- produce html file and not fragment -->
  <xsl:param name="css-gfile"  select="'../styles/node.css'"/><!-- path when published if standalone -->
  <xsl:param name="css-lfile"  select="'styles/overview.css'"/><!-- path when published if standalone -->
  <xsl:param name="title"      select="'Conference program overview'"/><!-- title if standalone -->
  <xsl:param name="nodrafts"   select="true()"/><!-- suppress talks in sessions with status 'draft' too (not only 'private') -->
  <xsl:param name="sessaut"    select="false()"/><!-- print authors also in subtalks of sessions -->
  <xsl:param name="doublepage" select="false()"/><!-- layout pages such that they print on two pages side by side-->
  <xsl:param name="pagenumber" select="0"/><!-- only geneate HTML for a single page (if greater than 0) -->
  <xsl:param name="cutlen"     select="0"/><!-- cut titles longer than this (if greater than 0) -->
  <xsl:param name="gxoffset"   select="10"/><!-- left to upper left corner of first page in px -->
  <xsl:param name="gyoffset"   select="70"/><!-- top to upper left corner of first page in px -->
  <xsl:param name="cw"         select="180"/><!-- width of one track column -->
  <xsl:param name="hourw"      select="50"/><!-- width of one timebar column -->
  <xsl:param name="hourh"      select="120"/><!-- height of 1 hour -->
  <xsl:param name="pskip"      select="20"/><!-- distance between pages -->
  <xsl:param name="pmargin"    select="3"/><!-- padding around pages -->
  <xsl:param name="phead"      select="40"/><!-- height of page header -->
  <xsl:param name="chead"      select="30"/><!-- height of track header -->
  <xsl:param name="hfrac"      select="0.5"/><!-- height of time background rectangle as fraction -->
  <xsl:param name="cfrac"      select="0.5"/><!-- width of track background rectangle as fraction -->
  <xsl:param name="hskip"      select="5"/><!-- horizontal skip between columns -->
  <xsl:param name="vskip"      select="5"/><!-- vertical skip between rows, not boxes -->
  <xsl:param name="vmargin"    select="2"/><!-- vertical margin around boxes -->
  <xsl:param name="iconh"      select="16"/><!-- height of category icon -->
  <xsl:param name="iconw"      select="33"/><!-- width of category icon -->
  <xsl:param name="ppf"        select="''"/><!-- global path prefix for links to resources like images or help pages or stylesheets -->
  <xsl:param name="ns_bdfix"   select="6"/><!-- empirical fix for NS 4.7 (layer pad of 6) -->

  <xsl:template match="@*|node()" />

  <xsl:template match="/">
    <!--if--><xsl:choose><xsl:when test="$standalone">
      <html>
        <head>
          <xsl:comment>
		    This HTML file is automatically generated from XML. Do not edit!
		  </xsl:comment>
          <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
          <title><xsl:value-of select="$title"/></title>
		  <link rel="stylesheet" href="{$ppf}{$css-gfile}"/>
		  <link rel="stylesheet" href="{$css-lfile}"/>
        </head>
        <body bgcolor="#FFFFCC">
          <h1><xsl:value-of select="$title"/></h1>
          <xsl:apply-templates/>
        </body>
      </html>
	<!--else--></xsl:when><xsl:otherwise>
      <xsl:comment>
		The following content is automatically generated from XML. Do not edit!
      </xsl:comment>
      <xsl:apply-templates/>
	<!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="overview">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="pages">
    <!--if--><xsl:choose><xsl:when test="$pagenumber = 0">
      <xsl:apply-templates/>
	 <!--else--></xsl:when><xsl:otherwise>
      <xsl:apply-templates select="page [number = $pagenumber] [1]"/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="pages/page">
    <!-- compute how much space we have consumed before this page -->
	 <!-- this would fail when pages are unordered: select="sum (preceding-sibling::page/rasterLength)"/> instead, we use: -->
    <xsl:variable name="mynum">
      <!--if--><xsl:choose><xsl:when test="$pagenumber = 0">
        <xsl:value-of select="round (number)"/>
	   <!--else--></xsl:when><xsl:otherwise>
        <xsl:value-of select="-9999"/>
	   <!--endif--></xsl:otherwise></xsl:choose>
    </xsl:variable>
    <xsl:variable name="sumh"  select="sum   (../page [number &lt; $mynum]/rasterLength)"/>
    <xsl:variable name="cnth"  select="count (../page [number &lt; $mynum]/rasterLength)"/>
    <xsl:variable name="px" select="$gxoffset"/>
    <xsl:variable name="py" select="$gyoffset + round (($sumh * $hourh) div 60) + $cnth * ($pskip + 2 * ($phead + $chead + 2 * $vskip))"/>
    <xsl:variable name="ph" select="round ((rasterLength * $hourh) div 60) + 2 * ($phead + $chead + $pmargin + 2 * $vskip)"/>
    <xsl:variable name="pw" select="2 * $hourw + 2 * $pmargin + (maxColumn * $cw) + ((maxColumn + 1) * $hskip)"/>
    <!-- write page background -->
    <div class ="lypage" style="position:absolute; z-index:1; left:{$px}px; top:{$py}px; width:{$pw}px; height:{$ph}px"/>
    <!-- write page headers -->
    <xsl:apply-templates select="columns/column/tracks/track/date">
      <xsl:with-param name="x0" select="$px + $pmargin"/>
      <xsl:with-param name="y0" select="$py + $pmargin"/>
      <xsl:with-param name="w0" select="$pw - 2 * $pmargin"/>
      <xsl:with-param name="h0" select="$phead"/>
    </xsl:apply-templates>
    <xsl:apply-templates select="columns/column/tracks/track/date">
      <xsl:with-param name="x0" select="$px + $pmargin"/>
      <xsl:with-param name="y0" select="$py + $ph - $pmargin - $phead"/>
      <xsl:with-param name="w0" select="$pw - 2 * $pmargin"/>
      <xsl:with-param name="h0" select="$phead"/>
    </xsl:apply-templates>
    <!-- write timebars -->
    <xsl:call-template name="timeboxes">
      <xsl:with-param name="x0" select="$px + $pmargin"/>
      <xsl:with-param name="y0" select="$py + $pmargin + $phead + $chead + 2 * $vskip"/>
      <xsl:with-param name="w0" select="$pw - 2 * $pmargin"/>
      <xsl:with-param name="n"  select="."/>
    </xsl:call-template>
    <!-- write columns -->
    <xsl:apply-templates select="columns/column">
      <xsl:with-param name="x0" select="$px + $pmargin + $hourw + $hskip"/>
      <xsl:with-param name="y0" select="$py + $pmargin + $phead + $vskip"/>
      <xsl:with-param name="h0" select="$ph - 2 * ($pmargin + $phead + $vskip)"/>
      <xsl:with-param name="n"  select="."/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="columns/column">
    <xsl:variable name="px" select="$x0 + (number - 1) * ($cw + $hskip)"/>
    <div class ="lycolbar" style="position:absolute; z-index:5; left:{round ($px + $cw * $cfrac div 2)}px; top:{$y0}px; width:{round ($cw * $cfrac)}px; height:{$h0}px"/>
    <!-- write column headers -->
    <xsl:apply-templates select="title">
      <xsl:with-param name="x0" select="$px"/>
      <xsl:with-param name="y0" select="$y0"/>
    </xsl:apply-templates>
    <xsl:apply-templates select="title">
      <xsl:with-param name="x0" select="$px"/>
      <xsl:with-param name="y0" select="$y0 + $h0 - $chead"/>
    </xsl:apply-templates>
    <!-- write tracks -->
    <xsl:apply-templates select="tracks/track">
      <xsl:with-param name="x0" select="$px"/>
      <xsl:with-param name="y0" select="$y0 + $chead + $vskip"/>
      <xsl:with-param name="n"  select="$n"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="tracks/track">
    <xsl:apply-templates select="blocks/block">
      <xsl:with-param name="x0" select="$x0"/>
      <xsl:with-param name="y0" select="$y0"/>
      <xsl:with-param name="w0" select="$cw + extraTrackCols * ($cw + $hskip)"/>
      <xsl:with-param name="n"  select="$n"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="blocks/block">
    <xsl:variable name="py" select="$y0 + round (((rasterStart - $n/rasterStart) * $hourh) div 60)"/>
    <xsl:variable name="h0" select="round ((rasterLength * $hourh) div 60)"/>
    <div class ="ly-{blockType}" style="position:absolute; z-index:7; left:{$x0}px; top:{$py + $vmargin}px; width:{$w0}px; height:{$h0 - 2 * $vmargin}px; overflow: hidden">
      <table width="100%" height="100%" cellspacing="0" cellpadding="0" border="0">
	    <tr>
	      <!--NS4.7 fix--><td><img src="{$ppf}images/shim.gif" width="1" height="{$h0 - 2 * ($vmargin + $ns_bdfix)}"/></td>
	      <td valign="top" class="box">
		    <!-- first paragraph -->
		    <p class="boxp">
			  <span class="boxtime"><xsl:value-of select="startTime"/><xsl:text> </xsl:text></span>
	          <xsl:if test="(blockType = 'talk' or blockType = 'keynote') and string (talks/talk/type/iconUrl)">
                <a href="{$ppf}{talks/talk/type/typePageUrl}"><img alt="{talks/talk/type/description}" src="{$ppf}{talks/talk/type/iconUrl}" width="{$iconw}" height="{$iconh}" align="absmiddle" border="0"/></a>
	            <xsl:text> </xsl:text>
	          </xsl:if>
		      <span class="boxtype"><xsl:choose>
		        <xsl:when test="blockType = 'keynote'">Keynote:<xsl:text> </xsl:text></xsl:when>
			    <xsl:when test="blockType = 'session'">Session:<xsl:text> </xsl:text></xsl:when>
			  </xsl:choose></span>
		      <span class="title-{blockType}"><xsl:choose>
		       <xsl:when test="(blockType = 'talk' or blockType = 'keynote') and talks/talk"><a href="talks.html#{talks/talk/key}.t" target="talks" class="plink"><xsl:value-of select="title"/></a></xsl:when>
			    <xsl:when test="blockType = 'keynote' or (blockType = 'other' or blockType = 'event') and talks/talk"><a href="agenda.html#{key}.b" target="agenda" class="plink"><xsl:value-of select="title"/></a></xsl:when>
			    <xsl:when test="blockType = 'session' and talks/talk"><a href="agenda.html#{key}.b" target="agenda" class="plink"><xsl:apply-templates select="title"/></a></xsl:when>
		       <xsl:otherwise><xsl:value-of select="title"/></xsl:otherwise>
			  </xsl:choose></span>
              <xsl:if test="blockType != 'session'">
		        <span class="boxduration"><xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;(</xsl:text><xsl:value-of select="duration"/><xsl:text> min)</xsl:text></span>
              </xsl:if>
			</p>
		    <!-- second and more paragraphs -->
    		<xsl:choose>
			  <xsl:when test="blockType = 'keynote'">
		        <xsl:apply-templates select="author" mode="keynote"/>
			  </xsl:when>
			  <xsl:when test="blockType = 'session'">
		        <xsl:apply-templates select="talks/talk"/>
			  </xsl:when>
			  <xsl:otherwise>
		        <xsl:apply-templates select="author" mode="talk"/>
			  </xsl:otherwise>
			</xsl:choose>
		  </td>
	    </tr>
	  </table>
	</div>
  </xsl:template>

  <xsl:template match="talks/talk">
    <xsl:if test="not (state = 'private' or ($nodrafts and state = 'draft'))">
      <div class="sessiontalk">
        <span class="titlesessiontalktime"><xsl:value-of select="startTime"/><xsl:text> </xsl:text></span>
	     <xsl:if test="string (type/iconUrl)">
          <a href="{$ppf}{type/typePageUrl}"><img alt="{type/description}" src="{$ppf}{type/iconUrl}" width="{$iconw}" height="{$iconh}" align="absmiddle" border="0"/></a>
	      <xsl:text> </xsl:text>
	    </xsl:if>
        <span class="titlesessiontalk"><a href="talks.html#{key}.t" target="talks" class="plink"><xsl:apply-templates select="title"/></a></span>
	     <xsl:if test="authors/author and $sessaut">
	       <xsl:text> </xsl:text>
          <!--if--><xsl:choose><xsl:when test="authors/author [key = ../../speaker or key = ../../speaker/key]">
            <xsl:apply-templates select="authors/author [key = ../../speaker or key = ../../speaker/key] [1]"/>
	       <!--else--></xsl:when><xsl:otherwise>
            <xsl:apply-templates select="authors/author [1]"/>
	       <!--endif--></xsl:otherwise></xsl:choose>
	     </xsl:if>
	  </div>
    </xsl:if>
  </xsl:template>

  <xsl:template match="author">
    <span class="sessspeaker"><a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
      <xsl:if test="count (../author) > 1">
	     <xsl:text> et al.</xsl:text>
	   </xsl:if>
	 </span>
    <xsl:if test="string (affiliation)">
	   <span class="sessaffil"><xsl:text> (</xsl:text><xsl:apply-templates select="affiliation"/>)</span>
	 </xsl:if>
  </xsl:template>

  <xsl:template match="author" mode="keynote">
    <div class="keyspeaker"><a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
	  <xsl:if test="string (affiliation)">
	    <span class="keyaffil"><xsl:text> (</xsl:text><xsl:value-of select="affiliation"/>)</span>
	  </xsl:if>
	</div>
  </xsl:template>

  <xsl:template match="author" mode="talk">
    <div class="talkspeaker"><a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
	  <xsl:if test="string (affiliation)">
	    <span class="talkaffil"><xsl:text> (</xsl:text><xsl:value-of select="affiliation"/>)</span>
	  </xsl:if>
	</div>
  </xsl:template>

  <xsl:template match="talk/title">
    <!--if--><xsl:choose><xsl:when test="$cutlen > 0 and string-length (.) > $cutlen">
      <xsl:value-of select="substring (string (.), 1, $cutlen)"/>...
    <!--else--></xsl:when><xsl:otherwise>
      <xsl:value-of select="."/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="block/title">
    <!--if--><xsl:choose><xsl:when test="$cutlen > 0 and string-length (.) > $cutlen">
      <xsl:value-of select="substring (string (.), 1, $cutlen)"/>...
    <!--else--></xsl:when><xsl:otherwise>
      <xsl:value-of select="."/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="author/affiliation">
    <!--if--><xsl:choose><xsl:when test="$cutlen > 0 and string-length (.) > $cutlen">
      <xsl:value-of select="substring (string (.), 1, $cutlen)"/>...
    <!--else--></xsl:when><xsl:otherwise>
      <xsl:value-of select="."/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="column/title">
    <div class ="lycolhead" style="position:absolute; z-index:6; left:{$x0}px; top:{$y0}px; width:{$cw}px; height:{$chead}px; overflow: hidden">
      <table width="100%" height="100%" cellspacing="0" cellpadding="0" border="0">
	    <tr>
	      <td class="colhead"><xsl:value-of select="."/></td>
	    </tr>
	  </table>
	</div>
  </xsl:template>

  <xsl:template match="date">
    <div class ="lyhead" style="position:absolute; z-index:2; left:{$x0}px; top:{$y0}px; width:{$w0}px; height:{$h0}px; overflow: hidden">
      <table width="100%" height="100%" cellspacing="0" cellpadding="0" border="0">
	    <tr>
	       <td class="pagehead"><xsl:value-of select="."/></td>
			<xsl:if test="$doublepage">
	       <td class="pagehead"><xsl:value-of select="."/></td>
			</xsl:if>
	    </tr>
	  </table>
	</div>
  </xsl:template>

  <xsl:template name="timebar">
    <xsl:variable name="hourmin" select="ceiling (($n/rasterStart) div 60)"/>
    <xsl:variable name="hourmax" select="floor (($n/rasterStart + $n/rasterLength - 60) div 60)"/>
    <xsl:variable name="py" select="$y0 + round ((($hh * 60 - $n/rasterStart) * $hourh) div 60)"/>
	<xsl:if test="$hourmax >= $hh and $hh >= $hourmin">
      <div class ="lyhourbar" style="position:absolute; z-index:3; left:{$x0}px; top:{round ($py + ($hourh * $hfrac div 2))}px; width:{$w0}px; height:{round ($hourh * $hfrac)}px"/>
      <xsl:call-template name="timebox">
        <xsl:with-param name="hh" select="$hh"/>
        <xsl:with-param name="x0" select="$x0"/>
        <xsl:with-param name="y0" select="$py"/>
      </xsl:call-template>
      <xsl:call-template name="timebox">
        <xsl:with-param name="hh" select="$hh"/>
        <xsl:with-param name="x0" select="$x0 + $w0 - $hourw"/>
        <xsl:with-param name="y0" select="$py"/>
      </xsl:call-template>
	</xsl:if>
  </xsl:template>

  <xsl:template name="timebox">
    <div class ="lyhour" style="position:absolute; z-index:4; left:{$x0}px; top:{$y0 + $vmargin}px; width:{$hourw}px; height:{$hourh - 2 * $vmargin}px; overflow: hidden">
      <table width="100%" height="100%" cellspacing="0" cellpadding="0" border="0">
        <tr>
	      <!--NS4.7 fix--><td><img src="{$ppf}images/shim.gif" width="1" height="{$hourh - 2 * ($vmargin + $ns_bdfix)}"/></td>
	      <td class="timebox"><span class="timeboxhour"><xsl:value-of select="$hh"/></span><span class="timeboxsuff">:00<br/>-<br/><xsl:value-of select="$hh + 1"/>:00</span></td>
	    </tr>
	  </table>
    </div>
  </xsl:template>
  
  <xsl:template name="timeboxes">
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="00"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="01"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="02"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="03"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="04"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="05"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="06"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="07"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="08"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="09"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="10"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="11"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="12"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="13"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="14"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="15"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="16"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="17"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="18"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="19"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="20"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="21"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="22"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
    <xsl:call-template name="timebar"><xsl:with-param name="hh" select="23"/><xsl:with-param name="n" select="$n"/><xsl:with-param name="x0" select="$x0"/><xsl:with-param name="y0" select="$y0"/><xsl:with-param name="w0" select="$w0"/></xsl:call-template>
  </xsl:template>

</xsl:stylesheet>