<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <!--
	print.xsl v1.0; July 24, 2000; tested with Xalan
	This stylesheet creates an overview view suitable for printing (adopted from overview.xsl).
	LivingTalks conference content management tool (c) 2000 Living Pages Research GmbH, Munich, Germany
  -->
  <xsl:output encoding="iso-8859-1" method="html" indent="yes" omit-xml-declaration="yes"/>

  <xsl:param name="standalone" select="true()"/><!-- produce html file and not fragment -->
  <xsl:param name="css-gfile"  select="'../styles/node.css'"/><!-- path when published if standalone -->
  <xsl:param name="css-lfile"  select="'styles/print.css'"/><!-- path when published if standalone -->
  <xsl:param name="title"      select="'Conference program printable overview'"/><!-- title if standalone -->
  <xsl:param name="nodrafts"   select="true()"/><!-- suppress talks in sessions with status 'draft' too (not only 'private') -->
  <xsl:param name="sessaut"    select="false()"/><!-- print authors also in subtalks of sessions -->
  <xsl:param name="doublepage" select="false()"/><!-- layout pages such that they print on two pages side by side-->
  <xsl:param name="pagenumber" select="0"/><!-- only geneate HTML for a single page (if greater than 0) -->
  <xsl:param name="cutlen"     select="0"/><!-- cut titles longer than this (if greater than 0) -->
  <xsl:param name="iconh"      select="16"/><!-- height of category icon -->
  <xsl:param name="iconw"      select="33"/><!-- width of category icon -->
  <xsl:param name="ppf"        select="''"/><!-- global path prefix for links to resources like images or help pages or stylesheets -->
  <xsl:template match="@*|node()" />

  <xsl:template match="/">
    <!--if--><xsl:choose><xsl:when test="$standalone">
      <html>
        <head>
          <xsl:comment>
		    This HTML file is automatically generated from XML. Do not edit!
		  </xsl:comment>
          <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
          <title><xsl:value-of select="$title"/></title>
		  <link rel="stylesheet" href="{$ppf}{$css-gfile}"/>
		  <link rel="stylesheet" href="{$css-lfile}"/>
        </head>
        <body bgcolor="#FFFFCC">
          <h1><xsl:value-of select="$title"/></h1>
          <xsl:apply-templates/>
        </body>
      </html>
	<!--else--></xsl:when><xsl:otherwise>
      <xsl:comment>
		The following content is automatically generated from XML. Do not edit!
      </xsl:comment>
      <xsl:apply-templates/>
	<!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="overview">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="pages">
    <!--if--><xsl:choose><xsl:when test="$pagenumber = 0">
      <xsl:apply-templates/>
	 <!--else--></xsl:when><xsl:otherwise>
      <xsl:apply-templates select="page [number = $pagenumber] [1]"/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="pages/page">
    <a name="p.{number}"/>
    <!-- write page background -->
    <table width="100%" border="0" cellspacing="0" cellpadding="0" class ="lypage">
    <!-- write page headers -->
    <xsl:apply-templates select="(columns/column/tracks/track/date)[1]">
      <xsl:with-param name="ncols" select="tablemap/@ncols"/>
    </xsl:apply-templates>
    <!-- write columns -->
    <xsl:apply-templates select="tablemap"/>
    <!-- write page headers -->
    <xsl:apply-templates select="(columns/column/tracks/track/date)[1]">
      <xsl:with-param name="ncols" select="tablemap/@ncols"/>
    </xsl:apply-templates>
    </table>
    <p class="pageskip"><xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text></p>
  </xsl:template>

  <xsl:template match="tablemap">
    <!-- write track headers -->
    <xsl:apply-templates select="thr"/>
    <!-- write all block bodies -->
    <xsl:apply-templates select="tr"/>
    <!-- write track headers -->
    <xsl:apply-templates select="thr"/>
  </xsl:template>

  <xsl:template match="thr">
    <tr><xsl:apply-templates select="thd"/></tr>
  </xsl:template>

  <xsl:template match="tr">
    <tr><xsl:apply-templates select="td"/></tr>
  </xsl:template>

  <xsl:template match="thd">
    <td width="{round (100 div ../../@ncols)}%">
		<xsl:variable name="colref" select="col-ref"/>
      <xsl:apply-templates select="(../../../columns/column[number = $colref]/title)[1]"/>
	 </td>
  </xsl:template>

  <xsl:template match="td">
    <td class="bodycell">
	   <xsl:if test="@colspan"><xsl:attribute name="colspan"><xsl:value-of select="@colspan"/></xsl:attribute></xsl:if>
	   <xsl:if test="@rowspan"><xsl:attribute name="rowspan"><xsl:value-of select="@rowspan"/></xsl:attribute></xsl:if>
		<xsl:variable name="bkey"   select="blockKey"/>
		<xsl:variable name="tcol"   select="colStart"/>
		<xsl:variable name="textra" select="colLength - 1"/>
		<xsl:variable name="bstart" select="rowStart"/>
		<xsl:variable name="blen"   select="rowLength"/>
	   <xsl:apply-templates select="(../../../columns/column/tracks/track [trackCol = $tcol and extraTrackCols = $textra] [1]/blocks/block [key = $bkey and rasterStart = $bstart and rasterLength = $blen])[1]"/>
	 </td>
  </xsl:template>

  <xsl:template match="blocks/block">
    <table valing="top" height="100%" width="100%" cellspacing="0" cellpadding="0" border="0" class ="ly-{blockType}">
	    <tr>
	      <td valign="top" class="box">
		    <!-- first paragraph -->
		    <p class="boxp">
			  <span class="boxtime"><xsl:value-of select="startTime"/><xsl:text> </xsl:text></span>
	          <xsl:if test="(blockType = 'talk' or blockType = 'keynote') and string (talks/talk/type/iconUrl)">
                <a href="{$ppf}{talks/talk/type/typePageUrl}"><img alt="{talks/talk/type/description}" src="{$ppf}{talks/talk/type/iconUrl}" width="{$iconw}" height="{$iconh}" align="absmiddle" border="0"/></a>
	            <xsl:text> </xsl:text>
	          </xsl:if>
		      <span class="boxtype"><xsl:choose>
		        <xsl:when test="blockType = 'keynote'">Keynote:<xsl:text> </xsl:text></xsl:when>
			    <xsl:when test="blockType = 'session'">Session:<xsl:text> </xsl:text></xsl:when>
			  </xsl:choose></span>
		      <span class="title-{blockType}"><xsl:choose>
		       <xsl:when test="(blockType = 'talk' or blockType = 'keynote') and talks/talk"><a href="talks.html#{talks/talk/key}.t" target="talks" class="plink"><xsl:value-of select="title"/></a></xsl:when>
			    <xsl:when test="blockType = 'keynote' or (blockType = 'other' or blockType = 'event') and talks/talk"><a href="agenda.html#{key}.b" target="agenda" class="plink"><xsl:value-of select="title"/></a></xsl:when>
			    <xsl:when test="blockType = 'session' and talks/talk"><a href="agenda.html#{key}.b" target="agenda" class="plink"><xsl:apply-templates select="title"/></a></xsl:when>
		       <xsl:otherwise><xsl:value-of select="title"/></xsl:otherwise>
			  </xsl:choose></span>
              <xsl:if test="blockType != 'session'">
		        <span class="boxduration"><xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;(</xsl:text><xsl:value-of select="duration"/><xsl:text> min)</xsl:text></span>
              </xsl:if>
			</p>
		    <!-- second and more paragraphs -->
    		<xsl:choose>
			  <xsl:when test="blockType = 'keynote'">
		        <xsl:apply-templates select="author" mode="keynote"/>
			  </xsl:when>
			  <xsl:when test="blockType = 'session'">
		        <xsl:apply-templates select="talks/talk"/>
			  </xsl:when>
			  <xsl:otherwise>
		        <xsl:apply-templates select="author" mode="talk"/>
			  </xsl:otherwise>
			</xsl:choose>
		  </td>
	    </tr>
    </table>
  </xsl:template>

  <xsl:template match="talks/talk">
    <xsl:if test="not (state = 'private' or ($nodrafts and state = 'draft'))">
      <div class="sessiontalk">
        <span class="titlesessiontalktime"><xsl:value-of select="startTime"/><xsl:text> </xsl:text></span>
	     <xsl:if test="string (type/iconUrl)">
          <a href="{$ppf}{type/typePageUrl}"><img alt="{type/description}" src="{$ppf}{type/iconUrl}" width="{$iconw}" height="{$iconh}" align="absmiddle" border="0"/></a>
	      <xsl:text> </xsl:text>
	    </xsl:if>
        <span class="titlesessiontalk"><a href="talks.html#{key}.t" target="talks" class="plink"><xsl:apply-templates select="title"/></a></span>
	     <xsl:if test="authors/author and $sessaut">
	       <xsl:text> </xsl:text>
          <!--if--><xsl:choose><xsl:when test="authors/author [key = ../../speaker or key = ../../speaker/key]">
            <xsl:apply-templates select="authors/author [key = ../../speaker or key = ../../speaker/key] [1]"/>
	       <!--else--></xsl:when><xsl:otherwise>
            <xsl:apply-templates select="authors/author [1]"/>
	       <!--endif--></xsl:otherwise></xsl:choose>
	     </xsl:if>
	  </div>
    </xsl:if>
  </xsl:template>

  <xsl:template match="author">
    <span class="sessspeaker"><a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
      <xsl:if test="count (../author) > 1">
	     <xsl:text> et al.</xsl:text>
	   </xsl:if>
	 </span>
    <xsl:if test="string (affiliation)">
	   <span class="sessaffil"><xsl:text> (</xsl:text><xsl:apply-templates select="affiliation"/>)</span>
	 </xsl:if>
  </xsl:template>

  <xsl:template match="author" mode="keynote">
    <div class="keyspeaker"><a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
	  <xsl:if test="string (affiliation)">
	    <span class="keyaffil"><xsl:text> (</xsl:text><xsl:value-of select="affiliation"/>)</span>
	  </xsl:if>
	</div>
  </xsl:template>

  <xsl:template match="author" mode="talk">
    <div class="talkspeaker"><a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
	  <xsl:if test="string (affiliation)">
	    <span class="talkaffil"><xsl:text> (</xsl:text><xsl:value-of select="affiliation"/>)</span>
	  </xsl:if>
	</div>
  </xsl:template>

  <xsl:template match="talk/title">
    <!--if--><xsl:choose><xsl:when test="$cutlen > 0 and string-length (.) > $cutlen">
      <xsl:value-of select="substring (string (.), 1, $cutlen)"/>...
    <!--else--></xsl:when><xsl:otherwise>
      <xsl:value-of select="."/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="block/title">
    <!--if--><xsl:choose><xsl:when test="$cutlen > 0 and string-length (.) > $cutlen">
      <xsl:value-of select="substring (string (.), 1, $cutlen)"/>...
    <!--else--></xsl:when><xsl:otherwise>
      <xsl:value-of select="."/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="author/affiliation">
    <!--if--><xsl:choose><xsl:when test="$cutlen > 0 and string-length (.) > $cutlen">
      <xsl:value-of select="substring (string (.), 1, $cutlen)"/>...
    <!--else--></xsl:when><xsl:otherwise>
      <xsl:value-of select="."/>
    <!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="column/title">
    <table width="100%" height="100%" cellspacing="0" cellpadding="0" border="0" class ="lycolhead">
      <tr>
	     <td class="colhead"><xsl:value-of select="."/></td>
	   </tr>
	 </table>
  </xsl:template>

  <xsl:template match="date">
    <tr>
      <!--if--><xsl:choose><xsl:when test="$doublepage">
		  <td colspan="{$ncols div 2}">
        <table width="100%" cellspacing="0" cellpadding="0" border="0" class ="lyhead">
        <tr><td class="pagehead"><xsl:value-of select="."/></td></tr></table></td>
		  <td colspan="{$ncols - $ncols div 2}">
        <table width="100%" cellspacing="0" cellpadding="0" border="0" class ="lyhead">
        <tr><td class="pagehead"><xsl:value-of select="."/></td></tr></table></td>
	   <!--else--></xsl:when><xsl:otherwise>
		  <td colspan="{$ncols}">
        <table width="100%" cellspacing="0" cellpadding="0" border="0" class ="lyhead">
        <tr><td class="pagehead"><xsl:value-of select="."/></td></tr></table></td>
	   <!--endif--></xsl:otherwise></xsl:choose>
    </tr>
  </xsl:template>

</xsl:stylesheet>