<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <!--
	talks.xsl v1.0; July 24, 2000; tested with Xalan
	This stylesheet creates a list of all talks.
	LivingTalks conference content management tool (c) 2000 Living Pages Research GmbH, Munich, Germany
  -->
  <xsl:output encoding="iso-8859-1" method="html" indent="yes" omit-xml-declaration="yes"/>

  <xsl:param name="standalone" select="true()"/><!-- produce html file and not fragment -->
  <xsl:param name="css-gfile"  select="'../styles/node.css'"/><!-- path when published if standalone -->
  <xsl:param name="css-lfile"  select="'styles/talks.css'"/><!-- path when published if standalone -->
  <xsl:param name="title"      select="'Directory of Talks'"/><!-- title if standalone -->
  <xsl:param name="nodrafts"   select="true()"/><!-- suppress talks in sessions with status 'draft' too (not only 'private') -->
  <xsl:param name="iconh"      select="16"/><!-- height of category icon -->
  <xsl:param name="iconw"      select="33"/><!-- width of category icon -->
  <xsl:param name="ppf"        select="''"/><!-- global path prefix for links to resources like images or help pages or stylesheets -->

  <xsl:template match="@*|node()" />

  <xsl:template match="/">
    <!--if--><xsl:choose><xsl:when test="$standalone">
      <html>
        <head>
          <xsl:comment>
		    This HTML file is automatically generated from XML. Do not edit!
		  </xsl:comment>
          <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
          <title><xsl:value-of select="$title"/></title>
		  <link rel="stylesheet" href="{$ppf}{$css-gfile}"/>
		  <link rel="stylesheet" href="{$css-lfile}"/>
        </head>
        <body bgcolor="#FFFFFF">
          <h1><xsl:value-of select="$title"/></h1>
	      <p class="tochint">(Click on any item to jump to the corresponding entry...)</p>
	      <p class="tochint">(Then, click on an author to go to additional info...)</p>
	      <hr/>
          <xsl:apply-templates/>
        </body>
      </html>
	<!--else--></xsl:when><xsl:otherwise>
      <xsl:comment>
		The following content is automatically generated from XML. Do not edit!
      </xsl:comment>
      <xsl:apply-templates/>
	<!--endif--></xsl:otherwise></xsl:choose>
  </xsl:template>

  <xsl:template match="/talks">
    <h2 class="toctitle">Table of contents:</h2>
    <xsl:for-each select="types/type">
      <xsl:if test="talks">
        <p class="typetocline">
          <a href="#{key}.type" class="plink"><xsl:value-of select="description"/></a>:
        </p>
      </xsl:if>
      <xsl:for-each select="talks/talk">
        <xsl:if test="not (state = 'private' or ($nodrafts and state = 'draft'))">
          <p class="tocline">
	        <a href="#{key}.t" class="plink"><xsl:value-of select="title"/></a>
	      </p>
        </xsl:if>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="types">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="type">
    <xsl:if test="talks">
	  <hr/>
      <h2 class="typetitle">
	    <a name="{key}.type"/>
        <xsl:if test="string (iconUrl)">
          <a href="{$ppf}{typePageUrl}"><img alt="{description}" src="{$ppf}{iconUrl}" width="{2 * $iconw}" height="{2 * $iconh}" align="absmiddle" hspace="10" border="0"/></a>
        </xsl:if>
	    <xsl:value-of select="description"/>:
	  </h2>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="type/talks">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="talk">
    <xsl:if test="not (state = 'private' or ($nodrafts and state = 'draft'))">
	  <table width="90%" align="center" class="talk"><tr><td>
        <a name="{key}.t"/>
        <h3 class="title-default"><span>
          <xsl:if test="string (pitems/pitem/block/blockType)">
		    <xsl:attribute name="class">title-<xsl:value-of select="pitems/pitem/block/blockType"/></xsl:attribute>
		  </xsl:if>
          <xsl:value-of select="title"/>
          <xsl:if test="string (../../iconUrl)">
            <a href="{$ppf}{../../typePageUrl}"><img alt="{../../description}" src="{$ppf}{../../iconUrl}" width="{$iconw}" height="{$iconh}" align="absmiddle" hspace="5" border="0"/></a>
          </xsl:if>
        </span></h3>
        <xsl:apply-templates select="pitems"/>
        <xsl:apply-templates select="authors"/>
	    <xsl:apply-templates select="slidesUrl"/>
	    <xsl:apply-templates select="paperUrl"/>
        <xsl:apply-templates select="abstractHtml"/>
        <xsl:apply-templates select="homePageUrl"/>
		<p class="url"><xsl:text>Go to </xsl:text><a href="#" class="simpleLink">top of page</a></p>
  	  </td></tr></table>
    </xsl:if>
  </xsl:template>

  <!-- reusable portion to print author information starts here -->

  <xsl:template match="authors">
    <div class="authors">
      <xsl:apply-templates/>
	</div>
  </xsl:template>

  <xsl:template match="author">
    <p class="author">
      <xsl:if test="key = ../../speaker or key = ../../speaker/key">
        <xsl:attribute name="class">speaker</xsl:attribute>
	  </xsl:if>
      <a href="authors.html#{key}" target="authors" class="plink"><xsl:value-of select="firstName"/><xsl:text> </xsl:text><xsl:value-of select="lastName"/></a>
      <xsl:apply-templates select="affiliation"/>
	</p>
  </xsl:template>

  <xsl:template match="affiliation"><xsl:if test="string (.)">
    <span class="aff">
      <xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;</xsl:text>
	  (<xsl:value-of select="."/>)
	</span>
  </xsl:if></xsl:template>

  <xsl:template match="abstractHtml"><xsl:if test="string (.)">
    <div class="abstract">
      <p class="abstracttitle">Abstract:</p>
      <div class="abstractbody">
	    <xsl:value-of select="." disable-output-escaping="yes"/>
	  </div>
    </div>
  </xsl:if></xsl:template>

  <!-- reusable portion to print author information ends here -->
  <!-- reusable portion to print time and room information starts here -->

  <xsl:template match="pitems">
    <ul class="pitems">
      <xsl:apply-templates/>
	</ul>
  </xsl:template>

  <xsl:template match="pitem">
    <li class="pitem">
      <xsl:apply-templates select="startTime" mode="time"/>
      <xsl:apply-templates select="track" mode="time"/>
      <xsl:apply-templates select="block" mode="btime"/>
	</li>
  </xsl:template>

  <xsl:template match="startTime" mode="time"><xsl:if test="string (.)">
    <span class="time"><xsl:value-of select="."/></span>
    <xsl:apply-templates select="../stopTime" mode="time"/>
  </xsl:if></xsl:template>

  <xsl:template match="stopTime" mode="time"><xsl:if test="string (.)">
    <span class="time"><xsl:text> - </xsl:text><xsl:value-of select="."/></span>
  </xsl:if></xsl:template>

  <xsl:template match="track" mode="time">
    <span class="track">
      <xsl:apply-templates select="date" mode="time"/>
      <xsl:apply-templates select="title" mode="time"/>
      <xsl:apply-templates select="room" mode="time"/>
    </span>
  </xsl:template>

  <xsl:template match="track/date" mode="time"><xsl:if test="string (.)">
    <xsl:text> at </xsl:text><span class="date"><xsl:value-of select="."/></span>
  </xsl:if></xsl:template>

  <xsl:template match="track/title" mode="time"><xsl:if test="string (.)">
    <xsl:text>, in track "</xsl:text><span class="ttitle"><xsl:value-of select="."/></span>"
  </xsl:if></xsl:template>

  <xsl:template match="track/room" mode="time"><xsl:if test="string (.)">
    <span class="troom"><xsl:text>, in room '</xsl:text><xsl:value-of select="."/>'</span>
  </xsl:if></xsl:template>

  <xsl:template match="block" mode="btime"><xsl:if test="string (.)">
    <!--if--><xsl:choose><xsl:when test="blockType = 'session'">
	  <span class="bsess"><xsl:text> [in </xsl:text>
	    <xsl:apply-templates select="blockType" mode="btime"/>
        <xsl:apply-templates select="title" mode="btime"/>
        <xsl:apply-templates select="startTime" mode="btime"/>
	  ]</span>
	<!--else--></xsl:when><xsl:otherwise>
	  <xsl:text> [</xsl:text><xsl:apply-templates select="blockType" mode="btime"/>]
	<!--endif--></xsl:otherwise></xsl:choose>
  </xsl:if></xsl:template>

  <xsl:template match="block/blockType" mode="btime">
    <span class="btype"><xsl:value-of select="."/></span>
  </xsl:template>

  <xsl:template match="block/title" mode="btime"><xsl:if test="string (.)">
    <xsl:text> "</xsl:text><span class="btitle"><xsl:value-of select="."/></span>"
  </xsl:if></xsl:template>

  <xsl:template match="block/startTime" mode="btime"><xsl:if test="string (.)">
    <xsl:text> starting at </xsl:text><span class="btime"><xsl:value-of select="."/></span>
    <xsl:apply-templates select="../stopTime" mode="btime"/>
  </xsl:if></xsl:template>

  <xsl:template match="block/stopTime" mode="btime">
  </xsl:template>

  <!-- reusable portion to print time and room information ends here -->
  <!-- reusable portion to print URLs starts here -->

  <xsl:template match="slidesUrl"><xsl:if test="string (.)">
    <p class="url">
      <a href="{ppf}{.}" target="pdf" class="simpleLink">Sesson slides</a>
	  <xsl:text> (</xsl:text>
	  <xsl:value-of select="translate (substring-after (string (.), '.'), 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
      <xsl:apply-templates select="../slidesSize"/>
	  <xsl:text>)</xsl:text>
	</p>
  </xsl:if></xsl:template>

  <xsl:template match="slidesSize"><xsl:if test="string (.)">
    <xsl:text>, </xsl:text>
    <xsl:value-of select="."/>
  </xsl:if></xsl:template>

  <xsl:template match="paperUrl"><xsl:if test="string (.)">
    <p class="url">
      <a href="{ppf}{.}" target="pdf" class="simpleLink">Published paper</a>
	  <xsl:text> (</xsl:text>
	  <xsl:value-of select="translate (substring-after (string (.), '.'), 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
	  <xsl:text>)</xsl:text>
	</p>
  </xsl:if></xsl:template>

  <xsl:template match="homePageUrl"><xsl:if test="string (.)">
    <p class="url"><xsl:text>Go to </xsl:text>
      <a href="{.}" target="_blank" class="simpleLink">
	    <xsl:if test="not (starts-with (., 'http'))">
          <xsl:attribute name="href">http://<xsl:value-of select="."/></xsl:attribute>
	    </xsl:if>
	    website with additional information regarding this talk
	  </a>
      <xsl:text> (</xsl:text><xsl:value-of select="."/>)
	</p>
  </xsl:if></xsl:template>

</xsl:stylesheet>