package com.livis.flabes.util;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

public class ArraysUtils {

    private ArraysUtils () { }

    //----------------------------------------------------------------------
    public static List asList (Object array) {
	return Arrays.asList (ClassUtils.wrap (array));
    }

    //----------------------------------------------------------------------
    public static Object extract (Object[] src, Method extractor) {
	Object result = Array.newInstance (extractor.getReturnType (), src.length);
	for (int i = 0; i < src.length; ++i) {
	    Array.set (result, i, MethodUtils.invokeCatching (src [i], extractor));
	}
	return result;
    }

    public static Object extract (Object[] src, String extractorName) {
	// FIXME: null==src, 0==src.length
	return extract (src, ClassUtils.getMethodCatching (src [0].getClass (), extractorName));
    }

    public static Collection extract (Collection srcCollection, Method extractor) {
	Class    componentType = extractor.getDeclaringClass ();
	Object[] srcArray      = (Object[])Array.newInstance (componentType, srcCollection.size ());
	srcCollection.toArray (srcArray);
	return Arrays.asList (ClassUtils.wrap (extract (srcArray, extractor)));
    }

    public static Collection extract (Collection src, String extractorName) {
	if (0 >= src.size ()) {
	    return new ArrayList (0);
	}
	return extract (src, ClassUtils.getMethodCatching (src.iterator ().next ().getClass (), extractorName));
    }

    //----------------------------------------------------------------------
    public static Map addAll (Map result, Object[] src, Method keyGetter) {
	result.clear ();

	for (int i = 0; i < src.length; ++i) {
	    Object key = MethodUtils.invokeCatching (src [i], keyGetter);
	    result.put (key, src [i]);
	}
 	return result;
    }

    public static Map addAll (Map result, Collection src, Method keyGetter) {
	return addAll (result, (null == src) ? new Object [0] : src.toArray (), keyGetter);
    }

    public static Map addAll (Map result, Object[] src, String keyGetterName) {
	return addAll (result, src, ClassUtils.getMethodCatching (src.getClass ().getComponentType (), keyGetterName));
    }

    public static Map addAll (Map result, Collection srcCollection, String keyGetterName) {
	Object[] srcArray      = (null == srcCollection) ? new Object [0] : srcCollection.toArray ();
	Class    componentType = (0    == srcArray.length) ? Object.class : srcArray [0].getClass ();
	return addAll (result, srcArray, ClassUtils.getMethodCatching (componentType, keyGetterName));
    }

    public static MapMap addAll (MapMap result, Object[]src, Method mapKeyGetter, Method keyGetter) {
	// FIXME: initial capacity known!
//  	MapMap result = new HashMapHashMap ();

	result.clear ();

	for (int i = 0; i < src.length; ++i) {
	    Object mapKey = MethodUtils.invokeCatching (src [i], mapKeyGetter);
	    Object key    = MethodUtils.invokeCatching (src [i], keyGetter);
	    result.put (mapKey, key, src [i]);
	}
 	return result;
    }

    //----------------------------------------------------------------------
    public static int commonPrefixLength (Object arraysO) {
	if (null == arraysO) {
	    return 0;
	}

	if (!ClassUtils.isArray (arraysO)) {
	    throw new IllegalStateException ("expected two-dimensional array (type found: " + arraysO.getClass ().getName () + ")");
	}

	Object[] arrays = new Object [Array.getLength (arraysO)];

	for (int i = 0; i < arrays.length; ++i) {
	    arrays [i] = Array.get (arraysO, i);

	    if (!ClassUtils.isArray (arrays [i])) {
		throw new IllegalStateException ("expected two-dimensional array (type found at index " + i + ": "
						 + arrays [i].getClass ().getName () + ")");
	    }

	    if (null == arrays [i]) {
		return 0;
	    }
	}

	int result;
	for (result = 0; ; ++result) {
	    if (result == Array.getLength (arrays [0])) {
		break;
	    }

	    Object component = Array.get (arrays [0], result);
	    int    i;
	    for (i = 1; i < arrays.length; ++i) {
		if ((result == Array.getLength (arrays [i]) || (!ObjectUtils.equal (component, Array.get (arrays [i], result))))) {
		    break;
		}
	    }
	    if (i != arrays.length) {
		break;
	    }
	}

	return result;
    }

    public static int commonPrefixLength (Object array0, Object array1) { return commonPrefixLength (new Object[] { array0, array1, }); }

    public static Object[] commonPrefix (Object arrays) {
	if (null == arrays) {
	    return null;
	}
	int      prefixLength = commonPrefixLength (arrays);
	Object[] result       = new Object [prefixLength];
	Object   array        = Array.get (arrays, 0);
	for (int i = 0; i < prefixLength; ++i) {
	    result [i] = Array.get (array, i);
	}
	return result;
    }

    public static Object[] commonPrefix (Object array0, Object array1) { return commonPrefix (new Object[] { array0, array1, }); }

    public static Object cutCommonPrefix (Object arrays) {
	if (null == arrays) {
	    return null;
	}
	
	int prefixLength = commonPrefixLength (arrays);
	for (int i = 0; i < Array.getLength (arrays); ++i) {
	    Object array = Array.get (arrays, i);
	    if (null != array) {
		int    newLength = Array.getLength (array) - prefixLength;
		Object newArray  = Array.newInstance (array.getClass ().getComponentType (), newLength);
		System.arraycopy (array, prefixLength, newArray, 0, newLength);
		Array.set (arrays, i, newArray);
	    }
	}
	return arrays;
    }

    public static Object cutCommonPrefix (Object array0, Object array1) {
	return cutCommonPrefix (new Object[] { array0, array1, });
    }

    //----------------------------------------------------------------------
    public static String getJavaInitializationString (byte[] b) {
	if (true) {
	    StringBuffer result  = new StringBuffer ("com.livis.flabes.util.StringUtils.toByteArray (\"\"\n\t\t+ \"");
	    char[]       plain   = StringUtils.toString (b).toCharArray ();
	    StringBuffer current = new StringBuffer ();

	    for (int i = 0; i < plain.length; ++i) {

		// FIXME: braindead editors (like JBuilder's) cannot
		// handle long lines 8(
		if (false)
		if (70 <= current.length ()) {
		    current.append ("\"\n\t\t+ \"");
		    result.append (current);
		    current = new StringBuffer ();
		}

		// Unicode escapes are expanded prior to lexical
		// analysis. The fact that the Unicode escape appears
		// within a string literal is irrelevant. See JLS
		// 3.2. and 3.10.6
		switch (plain [i]) {
		    case '\b': /* \u0008: backspace       BS */ { current.append ("\\b");  } break;
		    case '\t': /* \u0009: horizontal tab  HT */ { current.append ("\\t");  } break;
		    case '\n': /* \u000a: linefeed        LF */ { current.append ("\\n");  } break;
		    case '\f': /* \u000c: form feed       FF */ { current.append ("\\f");  } break;
		    case '\r': /* \u000d: carriage return CR */ { current.append ("\\r");  } break;
		    case '\"': /* \u0022: double quote    "  */ { current.append ("\\\""); } break;
		    case '\'': /* \u0027: single quote    '  */ { current.append ("\\'");  } break;
		    case '\\': /* \u005c: backslash       \  */ { current.append ("\\\\"); } break;
		    default: {
			// FIXME: braindead editors/compilers (like
			// JBuilder's) cannot "non-printable"
			// characters 8(
			if (Character.isISOControl (plain [i])) {
			    String hexString = Integer.toHexString ((int)plain [i]);
			    current.append ("\\u" + StringUtils.stringOf ("0", 4 - hexString.length ()) + hexString);
			} else {
			    current.append (plain [i]);
			}
		    }
		}
	    }
	    result.append (current + "\")");
	    return result.toString ();
	} 


// 	String encoding = "UTF8";
// 	String encoding = "8859_1";
//  	String encoding = "Unicode";
//   	String encoding = "ASCII";
//   	String encoding = "SingleByte";
  	String encoding = "Cp1252";
	try {
	    return "\"" + new String (b, encoding) + "\".getBytes (\"" + encoding + "\")";
	} catch (UnsupportedEncodingException e) {
	    e.printStackTrace (System.err);

	    StringBuffer result = new StringBuffer ("(new byte[] {");
	    result.append ("\n\t\t");
        
	    for (int i = 0; i < b.length; ++i) {
		// result.append (((0 == (i % 5)) ? "\n\t\t" : "") + "(byte)0x" + Integer.toHexString ((b [i] + 256) % 256) + ", ");
		// result.append (((0 == (i % 5)) ? "\n\t\t" : "") + "(byte)"   +                      b [i]                + ", ");

		// one long line...
		result.append ("(byte)" + b [i] + ", ");
	    }
    
	    result.append ("\n\t})");
	    return result.toString ();
 	}
    }
}
