package com.livis.flabes.util;

import java.io.*;
import java.lang.reflect.*;

public class ClassUtils {
    private ClassUtils () { }

    private static PrintWriter defaultOut = new PrintWriter (System.err);

    public static Object newInstanceCatching (Class cls, PrintWriter out) {
 	try {
	    return cls.newInstance ();
	} catch (Exception e) {
	    if (null != out) {
		e.printStackTrace (out);
		out.flush ();
	    }
	    return null;
	}
    }

    public static Object newInstanceCatching (Class cls) {
	return newInstanceCatching (cls, defaultOut);
    }

    public static Class forNameCatching (String className, PrintWriter out) {
	try {
	    return Class.forName (className);
	} catch (Exception e) {
	    if (null != out) {
		e.printStackTrace (out);
		out.flush ();
	    }
	    return null;
	}
    }

    public static Class forNameCatching (String className) {
	return forNameCatching (className, defaultOut);
    }

    public static Object newInstanceForNameCatching (String className, PrintWriter out) {
	return newInstanceCatching (forNameCatching (className, out), out);
    }

    public static Object newInstanceForNameCatching (String className) {
	return newInstanceForNameCatching (className, defaultOut);
    }

    // "java.lang.StringBuffer" -> "StringBuffer"
    public static String classNameWithoutPackage (String className) {
	return className.substring (className.lastIndexOf ('.') + 1);
    }

    public static String innerClassName (String className) {
	String result  = classNameWithoutPackage (className);
	return result.substring (result.lastIndexOf ('$') + 1);
    }

    // we need this since Class.getPackage() may return null
    public static String packageName (String className) {
	int lastDot = className.lastIndexOf ('.');
	if (-1 == lastDot) {
	    return "";
	} else {
	    return className.substring (0, lastDot);
	}
    }

    //----------------------------------------------------------------------
    // see "The Java Language Specification", Section 4.2 "Primitive
    // Types and Values"
    //----------------------------------------------------------------------
    public static boolean isIntegralPrimitive (Class type) {
	return (   (Byte.TYPE      == type)
		|| (Short.TYPE     == type)
		|| (Integer.TYPE   == type)
		|| (Long.TYPE      == type)
		|| (Character.TYPE == type));
    }

    public static boolean isIntegralWrapper (Class type) {
	return (   (Byte.class      == type)
		|| (Short.class     == type)
		|| (Integer.class   == type)
		|| (Long.class      == type)
		|| (Character.class == type));
    }

    public static boolean isIntegral (Class type) {
	return isIntegralPrimitive (type) || isIntegralWrapper (type);
    }

    public static boolean isFloatingPointPrimitive (Class type) {
	return (   (Float.TYPE   == type)
		|| (Double.TYPE  == type));
    } 

    public static boolean isFloatingPointWrapper (Class type) {
	return (   (Float.class   == type)
		|| (Double.class  == type));
    }

    public static boolean isFloatingPoint (Class type) {
	return isFloatingPointPrimitive (type) || isFloatingPointWrapper (type);
    }

    public static boolean isNumericPrimitive (Class type) {
	return isIntegralPrimitive (type) || isFloatingPointPrimitive (type);
    }

    public static boolean isNumericWrapper (Class type) {
	return isIntegralWrapper (type) || isFloatingPointWrapper (type);
    }

    public static boolean isNumeric (Class type) {
	return isNumericPrimitive (type) || isNumericWrapper (type);
    }

    // corresponds to Class.isPrimitive
    public static boolean isWrapper (Class type) {
	return (   isNumericWrapper (type)
	        || (Boolean.class   == type)
		|| (Void.class      == type));
    }

    // FIXME: array?
    public static boolean isBasic (Class type) {
	return type.isPrimitive () || isWrapper (type) || (String.class == type);
    }

    //----------------------------------------------------------------------
    public static boolean isArray (Class c) { return (null != c) && (null != c.getComponentType ()); }
    public static boolean isArray (Object o) { return (null != o) && isArray (o.getClass ()); }

    //----------------------------------------------------------------------
    public static Class wrapperType (Class primitiveType) {
	if      (Boolean.TYPE 	== primitiveType) { return Boolean.class; }
	else if (Byte.TYPE    	== primitiveType) { return Byte.class; }
	else if (Character.TYPE == primitiveType) { return Character.class; }
	else if (Short.TYPE   	== primitiveType) { return Short.class; }
	else if (Integer.TYPE 	== primitiveType) { return Integer.class; }
	else if (Long.TYPE    	== primitiveType) { return Long.class; }
	else if (Float.TYPE   	== primitiveType) { return Float.class; }
	else if (Double.TYPE  	== primitiveType) { return Double.class; }
	else if (Void.TYPE    	== primitiveType) { return Void.class; }
	else {
	    // input not a primitive type, return it; FIXME: throw an
	    // exception here?
	    return primitiveType;
	}
    }

    public static Class primitiveType (Class wrapperType) {
	if      (Boolean.class 	 == wrapperType) { return Boolean.TYPE; }
	else if (Byte.class    	 == wrapperType) { return Byte.TYPE; }
	else if (Character.class == wrapperType) { return Character.TYPE; }
	else if (Short.class   	 == wrapperType) { return Short.TYPE; }
	else if (Integer.class 	 == wrapperType) { return Integer.TYPE; }
	else if (Long.class    	 == wrapperType) { return Long.TYPE; }
	else if (Float.class   	 == wrapperType) { return Float.TYPE; }
	else if (Double.class  	 == wrapperType) { return Double.TYPE; }
	else if (Void.class    	 == wrapperType) { return Void.TYPE; }
	else {
	    throw new IllegalArgumentException ("not a wrapper type: " + wrapperType.getName ());
	}
    }

    public static Object[] wrap (Object srcArray) {
	if (null == srcArray) {
	    return null;
	}
	if (!isArray (srcArray)) {
	    throw new IllegalArgumentException ("expected array (found: " + srcArray.getClass ().getName () + ")");
	}
	Class componentType = srcArray.getClass ().getComponentType ();
	if (!componentType.isPrimitive ()) {
	    return (Object[])srcArray;
	}
	int    length = Array.getLength (srcArray);
	Object result = Array.newInstance (wrapperType (componentType), length);
	for (int i = 0; i < length; ++i) {
	    Array.set (result, i, Array.get (srcArray, i));
	}
	return (Object[])result;
    }

    public static Object unwrap (Object srcArray) {
	if (null == srcArray) {
	    return null;
	}
	if (!isArray (srcArray)) {
	    throw new IllegalArgumentException ("expected array (found: " + srcArray.getClass ().getName () + ")");
	}
	Class componentType = srcArray.getClass ().getComponentType ();
	if (!isWrapper (componentType)) {
	    throw new IllegalArgumentException ("not a wrapper type: " + componentType.getName ());
// 	    return srcArray;
	}
	int    length = Array.getLength (srcArray);
	Object result = Array.newInstance (primitiveType (componentType), length);
	for (int i = 0; i < length; ++i) {
	    Array.set (result, i, Array.get (srcArray, i));
	}
	return result;
    }

    //----------------------------------------------------------------------
    public static Method getMethodCatching (Class cls, String methodName, Class[] parameterTypes, PrintWriter out) {
	try {
	    return cls.getMethod (methodName, parameterTypes);
	} catch (Exception e) {
	    if (null != out) {
		e.printStackTrace (out);
		out.flush ();
	    }
	    return null;
	}
    }
    public static Method getMethodCatching (Class cls, String methodName, Class[] parameterTypes) {
	return getMethodCatching (cls, methodName, parameterTypes, defaultOut);
    }
    public static Method getMethodCatching (Class cls, String methodName, PrintWriter out) {
	return getMethodCatching (cls, methodName, null, out);
    }
    public static Method getMethodCatching (Class cls, String methodName) {
	return getMethodCatching (cls, methodName, null, defaultOut);
    }

    //----------------------------------------------------------------------
    public static void assureDerivedClassHasDefined (Class cls, String methodName) {
	// this is often used to check whether the outermost derived
	// class has defined the method, eg to provide static storage;
	// such methods oftern are protected, so we don't use
	// getMethod (which only returns public methods) but
	// getDeclaredMethod (which does not included inherited
	// methods, but that's what we want here)
	try {
	    cls.getDeclaredMethod (methodName, new Class [0]);
	} catch (NoSuchMethodException e) {
	    throw new IllegalArgumentException ("method " + methodName + " not defined in " + cls);
	}
    }
}
