package com.livis.flabes.util;

import java.io.*;
import java.util.*;

public class IOUtils {

    private IOUtils () {}

    //----------------------------------------------------------------------
    public static byte[] readComplete (InputStream in) throws IOException {
	int  buffSize = 8* 1024;

	// read chunks of data from InputStream until EOF is reached;
	// store chunks in a linked list and append each to result
	// byte array afterwards; we could have used a
	// ByteArrayOutputStream instead with auto-resize, but that
	// would use lot's of arraycopys for the grow which we want to
	// avoid (there's already too much copying here, but we cannot
	// know the total number of bytes in advance but have to try
	// until EOF)
	//
	// CAUTION: if we ever change this to use the number of bytes
	// to be read (which may be known in advance) especially check
	// whether the number of remaining bytes equals 0 as
	// read(...,0) returns 0 at EOF, not -1!

	int  totalBytesRead = 0;
	// use LinkedList for fast append
	List buffs          = new LinkedList ();
	int  bytesRead;
	do {
	    Buffer buffer = new Buffer (new byte [buffSize]);
	    bytesRead = in.read (buffer.data);

// System.out.println(bytesRead + "/" + buffSize);

	    if (0 < bytesRead) {
		totalBytesRead += bytesRead;
		buffer.size    =  bytesRead;
		buffs.add (buffer);
	    }
	} while (-1 != bytesRead);

	byte[] result   = new byte [totalBytesRead];
	int    writePos = 0;
	for (Iterator i = buffs.iterator (); i.hasNext (); ) {
	    Buffer buffer = (Buffer)i.next ();
	    System.arraycopy (buffer.data, 0, result, writePos, buffer.size);
	    writePos += buffer.size;

	    // prepare for GC
	    buffer.data = null;
	}
	buffs.clear ();
	buffs = null;

	return result;
    }

    public static class Buffer {
	public byte[] data;
	public int    size;
	public Buffer (byte[] data) { this.data = data; size = data.length; }
    }
    //----------------------------------------------------------------------

    public static int copy (OutputStream out, InputStream in, int maxBytes, byte[] b) throws IOException {
	if ((null == b) || (0 >= b.length)) {
	    throw new IllegalArgumentException ("buffer must be non-null and of size > 0");
	}
	Buffer buff        = new Buffer (b);
	int    bytesCopied = 0;
	int    bytesRead   = -1; 
	do {
	    int bytesToRead = ((0 > maxBytes) || (buff.size < maxBytes)) ? buff.size : maxBytes;
	    if (0 >= bytesToRead) {
		bytesRead = -1;
	    } else {
		bytesRead = in.read (buff.data, 0, bytesToRead);
		if (0 < bytesRead) {
		    out.write (buff.data, 0, bytesRead);
		    bytesCopied += bytesRead;
		}
	    }
	} while (-1 != bytesRead);

	return bytesCopied;
    }

    // FIXME: make buff size property/class variable, ...
    private static int defaultBuffSize = 10 * 1024;
    private static int defaultMaxBytes = -1;
    public static int copy (OutputStream out, InputStream in, int maxBytes) throws IOException {
	return copy (out, in, maxBytes, new byte [defaultBuffSize]);
    }
    public static int copy (OutputStream out, InputStream in,               byte[] b) throws IOException {
	return copy (out, in, defaultMaxBytes, b);
    }
    public static int copy (OutputStream out, InputStream in) throws IOException {
	return copy (out, in, defaultMaxBytes, new byte [defaultBuffSize]);
    }
}
