package com.livis.flabes.util;

import java.io.*;
import java.util.*;

// like java.util.HashSet, but uses LinkedHashMap instead of HashMap
// internally to preserve insertion order as iteration order

public class LinkedHashSet
    extends AbstractSet
    implements Set, Cloneable, Serializable {

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    public LinkedHashSet () { map = new LinkedHashMap (); }
    public LinkedHashSet (Collection c) {
	map = new LinkedHashMap (Math.max (2 * c.size (), 11));
	addAll (c);
    }
    public LinkedHashSet (int initialCapacity, float loadFactor) { map = new LinkedHashMap (initialCapacity, loadFactor); }
    public LinkedHashSet (int initialCapacity) { map = new LinkedHashMap (initialCapacity); }

    //----------------------------------------------------------------------
    // interface Set/class AbstractSet
    //----------------------------------------------------------------------
    public boolean  add      (Object o) { return null == map.put (o, PRESENT); }
    public void     clear    ()         { map.clear (); }
    public boolean  contains (Object o) { return map.containsKey (o); }
    public boolean  isEmpty  ()         { return map.isEmpty (); }
    public Iterator iterator ()         { return map.keySet ().iterator (); }
    public boolean  remove   (Object o) { return PRESENT == map.remove (o); }
    public int      size     ()         { return map.size (); }

    //----------------------------------------------------------------------
    // marking interface Cloneable
    //----------------------------------------------------------------------
    public Object clone () {
	try { 
	    LinkedHashSet newSet = (LinkedHashSet)super.clone ();
	    newSet.map = (LinkedHashMap)map.clone ();
	    return newSet;
	} catch (CloneNotSupportedException e) { 
	    throw new InternalError ();
	}
    }

    //----------------------------------------------------------------------
    // marking interface Serializable
    //----------------------------------------------------------------------
    private synchronized void writeObject (ObjectOutputStream s) throws IOException {
	s.defaultWriteObject ();
        s.writeInt (map.capacity ());
        s.writeFloat (map.loadFactor ());
        s.writeInt (map.size ());
	for (Iterator i = map.keySet ().iterator (); i.hasNext (); ) {
            s.writeObject (i.next ());
	}
    }

    private synchronized void readObject (ObjectInputStream s) throws IOException, ClassNotFoundException {
	s.defaultReadObject ();
        int   capacity   = s.readInt ();
        float loadFactor = s.readFloat ();
        map              = new LinkedHashMap (capacity, loadFactor);
        int   size       = s.readInt ();
	for (int i = 0; i < size; ++i) {
            Object e = s.readObject ();
            map.put (e, PRESENT);
        }
    }

    //----------------------------------------------------------------------
    private transient    LinkedHashMap map;
    private static final Object        PRESENT = new Object ();
}
