package com.livis.flabes.util;

import java.io.*;
import java.util.*;

public class ObjectUtils {

    private ObjectUtils () { }

    /**
     * Checks whether to objects are "equal".
     *
     * Like {@link java.lang.Object#equals(Object) equals}, with
     * <code>null</null> references gracefully handled (ie, considered
     * "equal").
     *
     * @param   lhs               left hand side object of check
     * @param   rhs               right hand side object of check
     * @returns <code>true</code> if both objects are
     *                            <code>null</code>, identical
     *                            (<code>==</code>) or match the
     *                            {@link
     *                            java.lang.Object#equals(Object)
     *                            equals} relation
     * @see java.lang.Object#equals(Object)
     **/
    public static boolean equal (Object lhs, Object rhs) {
	return (lhs == rhs) || ((null != lhs) && lhs.equals (rhs));
    }

    public static byte[] toSerialized (Object o) {
	ByteArrayOutputStream baos = new ByteArrayOutputStream ();
	try {
	    ObjectOutputStream oos = new ObjectOutputStream (baos);
	    oos.writeObject (o);
	    oos.close ();
	    baos.close ();
	} catch (IOException e) {
	    e.printStackTrace (System.err);
	}
	byte[] result = baos.toByteArray ();

	if (debug) {
	    System.out.println (o + " serialized to " + result.length + " bytes");
	    
	    // byte[] is not Object[]  8(
	    //    	System.out.println (Arrays.asList (result));
	    // FIXME: hexDump
	    for (int i = 0; i < result.length; ++i) {
		System.out.print (result [i] + " ");
	    }
	    System.out.println ();
	    
	    {
		Object newO = fromSerialized (result);
	    }
	}

	return result;
    }

    public static Object fromSerialized (byte[] ser) {
	Object o = null;
	try {
	    o = new ObjectInputStream (new ByteArrayInputStream (ser)).readObject ();
	} catch (Exception e) {
	    e.printStackTrace (System.err);
	}

	if (debug) {
	    // byte[] is not Object[]  8(
	    //    	System.out.println (Arrays.asList (ser));
	    // FIXME: hexDump
	    for (int i = 0; i < ser.length; ++i) {
		System.out.print (ser [i] + " ");
	    }
	    System.out.println ();
	    
	    System.out.println (o + " deserialized from " + ser.length + " bytes");
	}

	return o;
    }

    //----------------------------------------------------------------------
    private static boolean debug = "true".equals (System.getProperty (ObjectUtils.class.getName () + ".debug"));
}
