package com.livis.flabes.util;

import java.io.*;
import java.util.*;

public class SizeManagingMap
    extends AbstractMap
    implements Serializable {

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    public SizeManagingMap (Map map, int[] size) {
	super ();
	this.map  = map;
	this.size = size;
    }

    //----------------------------------------------------------------------
    // class AbstractMap
    //----------------------------------------------------------------------
    public Set entrySet () { return map.entrySet (); }

    //----------------------------------------------------------------------
    // size-changing methods
    //----------------------------------------------------------------------
    public void clear () {
	int oldSize = map.size ();
	map.clear ();
	changeSize (oldSize, map.size ());
    }
    public Object put (Object key, Object value) {
	int    oldSize = map.size ();
	Object result  = map.put (key, value);
	changeSize (oldSize, map.size ());
	return result;
    }
    public void putAll (Map t) {
	int oldSize = map.size ();
	map.putAll (t);
	changeSize (oldSize, map.size ());
    }
    public Object remove (Object key) {
	int    oldSize = map.size ();
	Object result  = map.remove (key);
	changeSize (oldSize, map.size ());
	return result;
    }

    //----------------------------------------------------------------------
    private int[] size;
    private void changeSize (int oldSize, int newSize) { size [0] += (newSize - oldSize); }

    //----------------------------------------------------------------------
    private Map map;
}
