package com.livis.flabes.util;

import java.util.*;

/**
 * A collection of static methods that deal with strings.
 **/
public class StringUtils {

    /** Don't let anyone instantiate this class */
    private StringUtils () { }

    /**
     * Returns a string with charcaters exceeding the specified
     * maximum length cut off.
     *
     * @param   orig   original string
     * @param   maxLen maximum length
     * @return         trimmed String
     **/
    public static String trimToLength (String orig, int maxLen) {
	return (orig.length () > maxLen) ? orig.substring (0, maxLen) : orig;
    }

    /**
     * Returns a string which contains the given base string several
     * times.
     *
     * @param  base  string to concatenate
     * @param  times number of base strings to concatenate
     * @return       concatenation string
     **/
    public static String stringOf (String base, int times) {
	StringBuffer result = new StringBuffer (times * base.length ());
	for (int i = 0; i < times; ++i) {
	    result.append (base);
	}
	return result.toString ();
    }

    /**
     * Returns a string which contains the given base character
     * several times.
     *
     * @param base  character to concatenate
     * @param times number of base characters to concatenate
     * @return      concatenation string
     **/
    public static String stringOf (char base, int times) {
	return stringOf (new Character (base).toString (), times);
    }

    public static String concat (String[] s, String glue, int beginIndex, int endIndex) {
	StringBuffer result = new StringBuffer ();
	for (int i = beginIndex; i < endIndex; ++i) {
	    if (i > beginIndex) {
		result.append (glue);
	    }
	    result.append (s [i]);
	}
	return result.toString ();
    }
    public static String concat (String[] s, String glue, int beginIndex) { return concat (s, glue, beginIndex, s.length); }
    public static String concat (String[] s, String glue)                 { return concat (s, glue, 0,          s.length); }
    public static String concat (String[] s)                              { return concat (s, "",   0,          s.length); }

    public static String concat (Collection c, String glue, int beginIndex, int endIndex) {
	return (null == c) ? null : concat ((String[])c.toArray (new String [0]), glue, beginIndex, endIndex);
    }
    public static String concat (Collection c, String glue, int beginIndex) {
	return (null == c) ? null : concat ((String[])c.toArray (new String [0]), glue, beginIndex);
    }
    public static String concat (Collection c, String glue) {
	return (null == c) ? null : concat ((String[])c.toArray (new String [0]), glue);
    }
    public static String concat (Collection c) {
	return (null == c) ? null : concat ((String[])c.toArray (new String [0]), "");
    }

    //----------------------------------------------------------------------
    public static final int CASE_TOGGLE_KEEP     = 0;
    public static final int CASE_TOGGLE_TO_UPPER = 1;
    public static final int CASE_TOGGLE_TO_LOWER = 2;
    public static String caseToggleTo (String s, int conversionMode, String insert) {
	if (isNullOrEmpty (s)) {
	    return s;
	}
	StringBuffer result = new StringBuffer ();
	char         lastC  = ' '; // compiler wants to see it inited
	for (int i = 0; i < s.length (); ++i) {
	    char c = s.charAt (i);

	    if ((0 < i) && (Character.isUpperCase (c)
			    && (Character.isLowerCase (lastC)
				|| ((i + 1) < s.length () && (Character.isLowerCase (s.charAt (i + 1))))))) {
		result.append (insert);
	    }
	    if (CASE_TOGGLE_TO_UPPER == conversionMode) {
		result.append (Character.toUpperCase (c));
	    } else if (CASE_TOGGLE_TO_LOWER == conversionMode) {
		result.append (Character.toLowerCase (c));
	    } else {
		result.append (c);
	    }
	    lastC = c;
	}

	return result.toString ();
    }

    public static String toCaseToggle (String s, String remove) {
	if (null == s) {
	    return null;
	}
	String[] parts = splitByString (s, remove, false);
	// FIXME: 0/1, flag
	for (int i = 1; i < parts.length; ++i) {
	    parts [i] = capitalize (parts [i]);
	}
	return concat (parts);
    }

    //----------------------------------------------------------------------
    public static String shortest (String[] s) {
	if (null == s) {
	    return null;
	}
	String result = null;
	for (int i = 0; i < s.length; ++i) {
	    if (null == s [i]) {
		return null;
	    }
	    if ((null == result) || (s [i].length () < result.length ())) {
		result = s [i];
	    }
	}
	return result;
    }

    public static String shortest (String s0, String s1) { return shortest (new String[] { s0, s1, }); }

    //----------------------------------------------------------------------
    public static int commonPrefixLength (String[] s) {
	if (null == s) {
	    return 0;
	}

	for (int i = 0; i < s.length; ++i) {
	    if (null == s [i]) {
		return 0;
	    }
	}

	int result;
	for (result = 0; ; ++result) {
	    if (result == s [0].length ()) {
		break;
	    }

	    char c = s [0].charAt (result);
	    int  i;
	    for (i = 1; (i < s.length) && (result < s [i].length ()) && (c == s [i].charAt (result)); ++i) {
		// nothing else
	    }
	    if (i != s.length) {
		break;
	    }
	}

	return result;
    }

    public static int commonPrefixLength (String s0, String s1) { return commonPrefixLength (new String[] { s0, s1, }); }

    public static String commonPrefix (String[] s) {
	return (null == s) ? null : s [0].substring (0, commonPrefixLength (s));
    }

    public static String commonPrefix (String s0, String s1) { return commonPrefix (new String[] { s0, s1, }); }

    public static void cutCommonPrefix (String[] s) {
	if (null == s) {
	    return;
	}

	int prefixLength = commonPrefixLength (s);
	for (int i = 0; i < s.length; ++i) {
	    if (null != s [i]) {
		s [i] = s [i].substring (prefixLength);
	    }
	}
    }

    public static String[] cutCommonPrefix (String s0, String s1) {
	String[] result = new String [] { s0, s1, };
	cutCommonPrefix (result);
	return result;
    }

    //----------------------------------------------------------------------
    public static int count (String in, String occurence) {
	int result = 0;
	if ((null != in) && (null != occurence) && !"".equals (occurence)) {
	    int step = occurence.length ();
	    for (int index = 0; -1 != index; ) {
		index = in.indexOf (occurence, index);

// 		System.err.println ("<" + in + "> <" + occurence + "> " + index);

		if (-1 != index) {
		    ++result;
		    index += step;
		}
	    } 
	}
	return result;
    }

    //----------------------------------------------------------------------

    /**
     * Returns the given string which is either cut to the specified
     * length (if it's longer) or appended with characters (if it's
     * shorter).
     *
     * @param base   string to cut/fill
     * @param length desired length of the result
     * @param fill   character to append to base string with
     * @return       cut/extended string
     **/
    public static String flushLeft (String base, int length, char fill) {
	base = trimToLength (base, length);
	return base + stringOf (fill, length - base.length ());
    }
    /**
     * Returns the given string which is either cut to the specified
     * length (if it's longer) or appended with blanks (if it's
     * shorter).
     *
     * @param base   string to cut/fill
     * @param length desired length of the result
     * @return       cut/extended string
     **/
    public static String flushLeft (String base, int length) { return flushLeft (base, length, ' '); }

    /**
     * Returns the given string which is either cut to the specified
     * length (if it's longer) or prepended with characters (if it's
     * shorter).
     *
     * @param base   string to cut/fill
     * @param length desired length of the result
     * @param fill   character to prepend to base string with
     * @return       cut/extended string
     **/
    public static String flushRight (String base, int length, char fill) {
	base = trimToLength (base, length);
	return stringOf (fill, length - base.length ()) + base;
    }

    /**
     * Returns the given string which is either cut to the specified
     * length (if it's longer) or prepended with blanks (if it's
     * shorter).
     *
     * @param base   string to cut/fill
     * @param length desired length of the result
     * @return       cut/extended string
     **/
    public static String flushRight (String base, int length) { return flushRight (base, length, ' '); }

    /**
     * Returns the given string which is either cut to the specified
     * length (if it's longer) or filled with characters (if it's
     * shorter). If the number of characters to add is odd, one more
     * character is added to the end.
     *
     * @param base   string to cut/fill
     * @param length desired length of the result
     * @param fill   character to prepend and append to the base string
     * @return       cut/extended string
     **/
    public static String center (String base, int length, char fill) {
	base = trimToLength (base, length);
	int postFillLength = (length - base.length ()) / 2;
	return stringOf (fill, length - postFillLength - base.length ()) + base + stringOf (fill, postFillLength);
    }

    /**
     * Returns the given string which is either cut to the specified
     * length (if it's longer) or filled with blanks (if it's
     * shorter). If the number of blanks to add is odd, one more
     * blank is added to the end.
     *
     * @param base   string to cut/fill
     * @param length desired length of the result
     * @return       cut/extended string
     **/
    public static String center (String base, int length) { return center (base, length, ' '); }

    public static String s (int n)   { return (1 == n) ? ""   : "s"; }
    public static String S (int n)   { return (1 == n) ? ""   : "S"; }
    public static String ys (int n)  { return (1 == n) ? "y"  : "ies"; }
    public static String YS (int n)  { return (1 == n) ? "Y"  : "IES"; }
    // class -> classes
    public static String es (int n)  { return (1 == n) ? ""   : "es"; }
    public static String ES (int n)  { return (1 == n) ? ""   : "ES"; }
    public static String uma (int n) { return (1 == n) ? "um" : "a"; }
    public static String UMA (int n) { return (1 == n) ? "UM" : "A"; }

    public static String no (int n)  { return (0 == n) ? "no" : String.valueOf (n); }
    public static String NO (int n)  { return (0 == n) ? "NO" : String.valueOf (n); }

    public static String replace (String in, String from, String to) {
	/* FIXME: faster with StringBuffer and estimated initial worst case
	   (replace ("aaaa", "a", "bbb")) capacity of

	   `(in.length () * to.length ()) / to.length ()'

	   ?
	**/
	String result;
	int    lastPos = 0;
	int    pos;

	if (from.equals ("")) {
	    result = in;
	} else {
	    result = "";
	    do {
		pos = in.indexOf (from, lastPos);
		if (-1 == pos) {
		    result = result + in.substring (lastPos, in.length ());
		} else {
		    result = result + in.substring (lastPos, pos) + to;
		    lastPos = pos + from.length ();
		}
	    } while (-1 != pos);
	}
	return result;
    }

    public static String cutLeading (String s, char[] trash) {
	if (isNullOrEmpty (s) || (null == trash)) {
	    return s;
	}

	String trashString = new String (trash);
	int    i;
	for (i = 0; i < s.length (); ++i) {
	    if (-1 == trashString.indexOf (s.charAt (i))) {
		return s.substring (i);
	    }
	}
	return "";
    }

    public static String cutLeading (String s, char trash) { return cutLeading (s, new char[] { trash }); }

    public static String cutTrailing (String s, char[] trash) {
	if (isNullOrEmpty (s) || (null == trash)) {
	    return s;
	}

	String trashString = new String (trash);
	int    i;
	for (i = s.length () - 1; 0 <= i; --i) {
	    if (-1 == trashString.indexOf (s.charAt (i))) {
		return s.substring (0, i + 1);
	    }
	}
	return "";
    }


    public static String cutTrailing (String s, char trash) { return cutTrailing (s, new char[] { trash }); }

    public static String removeOccurrences (String s, char[] trash) {
	if (isNullOrEmpty (s)) {
	    return s;
	}

	String       trashString = new String (trash);
	StringBuffer result      = new StringBuffer ();
	for (int i = 0; i < s.length (); ++i) {
	    char c = s.charAt (i);
	    if (-1 == trashString.indexOf (c)) {
		result.append (c);
	    }
	}

// 	System.err.println ("\"" + "\"/" + ArraysUtils.asList (trash) + " -> " result);

	return result.toString ();
    }

    // FIXME: emptyBeforeFirstDelim?
    public static String[] splitByString (String s, String delimString, boolean emptyAfterLastDelim) {
        if (null == s) {
            return null;
        }
	if (isNullOrEmpty (delimString)) {
	    return new String[] { s, };
	}

	List result      = new ArrayList ();
	int  delimLength = delimString.length ();
	int  index       = 0;
	do {
	    index = s.indexOf (delimString);
	    if (-1 != index) {
		result.add (s.substring (0, index));
		s = s.substring (index + delimLength);
	    } else {
		result.add (s);
	    }
	} while ((-1 != index) && (0 < s.length ()));

	if ((-1 != index) && emptyAfterLastDelim) {
	    result.add ("");
	}

//   	System.err.println ("splitByString \"" + s + "\"/\"" + delimString + "\" -> " + result);

	return (String[])result.toArray (new String [0]);
    }

    public static String[] split (String s, char[] delimChars, boolean emptyAfterLastDelim) {
        if (null == s) {
            return null;
        }
	List            result 	     = new ArrayList ();
	// FIXME: delim holds A SET OF delimiters
	String          delimString  = new String (delimChars);
	StringTokenizer st     	     = new StringTokenizer (s, delimString, true);
	boolean         lastWasDelim = false;
	while (st.hasMoreElements ()) {
	    String token = st.nextToken ();

	    if (token.equals (delimString)) {
		result.add ("");
		lastWasDelim = true;
	    } else {
		result.add (token);
		lastWasDelim = false;
		if (st.hasMoreElements ()) {
		    st.nextToken ();
		    lastWasDelim = true;
		}
	    }
	}

	if (lastWasDelim && emptyAfterLastDelim) {
	    result.add ("");
	}

	// 	System.err.println ("split \"" + s + "\" -> " + result);

	return (String[])result.toArray (new String [0]);
    }

    public static String[] split (String s, char delimChar, boolean emptyAfterLastDelim ) {
	return split (s, new char[] { delimChar }, true);
    }
    public static String[] split (String s, char[] delimChars) { return split (s, delimChars, true); }
    public static String[] split (String s, char   delimChar) { return split (s, new char[] { delimChar }, true); }

    //----------------------------------------------------------------------
    public static String clone (String s) { return (null == s) ? null : new String (s); }

    //----------------------------------------------------------------------
    public static boolean isNullOrEmpty (String s) { return (null == s) || "".equals (s); }

    public static String defaultIfNull        (String s, String dflt) { return (null == s)       ? dflt : s; }
    public static String defaultIfEmpty       (String s, String dflt) { return ("".equals (s))   ? dflt : s; }
    public static String defaultIfNullOrEmpty (String s, String dflt) { return isNullOrEmpty (s) ? dflt : s; }

    public static String emptyIfNull (String s) { return defaultIfNull (s, ""); }
    public static String nullIfEmpty (String s) { return defaultIfEmpty (s, null); }

    //----------------------------------------------------------------------
    public static String capitalize (String s) {
	if (null == s) {
	    return null;
	} else if (0 == s.length ()) {
	    return "";
	} else {
	    return s.substring (0, 1).toUpperCase () + s.substring (1);
	}
    }

    //----------------------------------------------------------------------
    //sun.beans.editors.StringEditor.getJavaInitializationString()
    //doesn't handle null properly
    public static String getJavaInitializationString (String s) { return (null == s) ? "null" : "\"" + s + "\""; }

    //----------------------------------------------------------------------
    // plain convert byte array to String and vice versa; note that we
    // cannot use String(byte[]) and toByteArray() since encodings
    // (that is, interpretation) are envolved and that conversion is
    // not symmetric
    public static String toString (byte[] b) {
	char[] c = new char [b.length];
	for (int i = 0; i < c.length; ++i) {
	    // byte is [-128,127], char ['\u0000','\uffff'] (ie,
	    // [0,65535]); converting negative bytes to char delivers
	    // undefined results, ie, (char)(byte)-2 = '?'; the other
	    // way round is OK
	    c [i] = (char)((b [i] + 256) % 256);
	    //  	    System.out.println (i + " " + b [i] + " " + (byte)c[i]);
	}

	// 	for (int i = 0; i < c.length; ++i) {
	// 	    System.out.println (i +  " " + c [i]);
	// 	}

	return new String (c);
    }

    public static byte[] toByteArray (String s) {
	byte[] b = new byte [s.length()];
	for (int i = 0; i < b.length; ++i) {
	    b [i] = (byte)s.charAt (i);
	}

	char[] c = s.toCharArray ();
	for (int i = 0; i < b.length; ++i) {
	    b [i] = (byte)c [i];
	}



	return b;
    }
}
