package com.livis.flabes.util.beans;

import com.livis.flabes.util.*;
import com.livis.flabes.xml.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import org.w3c.dom.*;

public class BeanToDOM {

    public BeanToDOM (Object bean, Document doc, String tag, boolean groupIndexed) { this (bean, doc, tag, groupIndexed, null, null); }
    public BeanToDOM (Object bean, Document doc, String tag)                       { this (bean, doc, tag, true); }
    public BeanToDOM (Object bean, Document doc,             boolean groupIndexed) { this (bean, doc, (null == bean) ? "tag" : DOMUtils.beanShortTagName (bean.getClass ().getName ()), groupIndexed); }
    public BeanToDOM (Object bean, Document doc)                                   { this (bean, doc, true); }
    public BeanToDOM (Object bean,               String tag, boolean groupIndexed) { this (bean, DocumentFactory.createDocument (), tag, groupIndexed); }
    public BeanToDOM (Object bean,               String tag)                       { this (bean, tag, true); }
    public BeanToDOM (Object bean,                           boolean groupIndexed) { this (bean, DocumentFactory.createDocument (), groupIndexed); }
    public BeanToDOM (Object bean)                                                 { this (bean, true); }

    public BeanToDOM (Object bean, Document doc, String tag, boolean groupIndexed, Map knownEditors, DOMConverterUtils knownDOMConverterUtils) {
	if (null == bean) {
	    return;
	}

	this.bean = bean;

	if (bean instanceof DOMable) {
	    element = ((DOMable)bean).toElement (doc);
	} else {
	    // take read/write properties only to enable bean
	    // reconstruction from DOM
	    bi                = new FastBeanInfo (bean, true);
	    editors           = (null == knownEditors)           ? bi.newPropertyEditorMap ()   : knownEditors;
	    domConverterUtils = (null == knownDOMConverterUtils) ? new DOMConverterUtils (bean) : knownDOMConverterUtils;
	    
	    // FIXME: make this a parameter; LivingTalks: false, FormBeans: true
	    this.dashifyTags = false;
	    if (dashifyTags) {
		tag = StringUtils.caseToggleTo (tag, StringUtils.CASE_TOGGLE_TO_LOWER, "-");
	    }
	    
	    element = doc.createElement (tag);

	    if (debug) {
		System.err.println (bean.getClass ().getName () + " tag: <" + tag + ">");
		System.err.println ("properties: " + Arrays.asList (bi.getPropertyNames ()));
	    }
	    
	    PropertyDescriptor[] attributeDescs = domConverterUtils.getAttributePropertyDescriptors ();
	    PropertyDescriptor[] tagDescs       = domConverterUtils.getTagPropertyDescriptors ();
	    
	    for (int i = 0; i < attributeDescs.length; ++i) {
		String text = valueAsText (bean, attributeDescs [i], (PropertyEditor)editors.get (attributeDescs [i].getName ()));
		if (null != text) {
		    element.setAttribute (attributeDescs [i].getName (), text);
		    domConverterUtils.cont ();
		}
	    }
	    
	    for (int i = 0; i < tagDescs.length; ++i) {
		if (tagDescs [i] instanceof IndexedPropertyDescriptor) {
		    appendIndexed (element, bean, (IndexedPropertyDescriptor)tagDescs [i], doc, groupIndexed);
		} else {
		    appendPlain   (element, bean, tagDescs [i],                            doc, groupIndexed,  (PropertyEditor)editors.get (tagDescs [i].getName ()));
		}
		domConverterUtils.cont ();
	    }
	}

	domConverterUtils.cont ();
    }

    private static Object value (Object bean, PropertyDescriptor desc) {
	Method getter = desc.getReadMethod ();
 	Method setter = desc.getWriteMethod ();

	// only accept read/write properties
 	if ((null == getter) || (null == setter)) {
	    return null;
	} else {
	    return MethodUtils.invokeCatching (bean, getter);
	}
    }

    private String valueAsText (Object bean, PropertyDescriptor desc, PropertyEditor editor) {
	if (null != editor) {
	    Object val = value (bean, desc);
	    if (null != val) {
		editor.setValue (val);
		return editor.getAsText ();
	    }
	}
	return null;
    }

    //----------------------------------------------------------------------
    private Element element;
    public Element toElement () { return element; }

    private Map editors;
    public Map getEditors () { return editors; }

    private DOMConverterUtils domConverterUtils;
    public DOMConverterUtils getDOMConverterUtils () { return domConverterUtils; }

    //----------------------------------------------------------------------
    private void appendIndexed (Element element, Object bean, IndexedPropertyDescriptor desc, Document doc, boolean groupIndexed) {
	String property = desc.getName ();

	if (debug) System.err.println ("appendIndexed (" + bean.getClass ().getName () + "." + property + ")");

	// FIXME: bug in JDK-1.2.2 with overloaded set methods, so we
	// have to re-create the PropertyDescriptor
	try {
	    desc = new IndexedPropertyDescriptor (property, bean.getClass ());
	} catch (IntrospectionException e) {
 	    System.err.println ("appendIndexed: " + e);
	    return;
	}

	Method getter  = null;
	Method setter  = null;
	Method iGetter = null;
	Method iSetter = null;
	try {
	    getter  = desc.getReadMethod ();
	    setter  = desc.getWriteMethod ();
	    iGetter = desc.getIndexedReadMethod ();
	    iSetter = desc.getIndexedWriteMethod ();
	} catch (NoSuchMethodError e) {
// 	    System.err.println (e);
	}

	if ((null == getter) && (null == iGetter)) {
// 	    System.err.println ("no read method");
	    // cannot read
	    return;
	}
	if ((null == setter) && (null == iSetter)) {
// 	    System.err.println ("no write method");
	    // only read/write allowed
	    return;
	}

	Object valueArray;
	if (null != getter) {
 	    if (debug) System.err.println ("getting all at once (getter = " + getter + ")");
	    valueArray = MethodUtils.invokeCatching (bean, getter);
 	    domConverterUtils.cont ();
	} else {
 	    if (debug) System.err.println ("getting separately");
	    List values = new ArrayList ();
	    // FIXME: where to get the length from if only indexed
	    // access is allowed? just try until we get an Exception
	    // (ignore that that's not necessarily an
	    // IndexOutOfBoundsException, I've seen
	    // InvokationTragetException)
	    for (int i = 0; ; ++i) {
		Object value;
		try {
		    value = iGetter.invoke (bean, new Object[] { new Integer (i) });
 		    domConverterUtils.cont ();
		} catch (Exception e) {
		    // e.printStackTrace (System.err);
		    break;
		}
		values.add (value);
	    }
	    valueArray = values.toArray ();
	}

	if (null != valueArray) {
	    if (groupIndexed) {
		String  tag          = domConverterUtils.getPropertyMap ().getTag (property);
		if (dashifyTags) {
		    tag = StringUtils.caseToggleTo (tag, StringUtils.CASE_TOGGLE_TO_LOWER, "-");
		}
		Element arrayElement = doc.createElement (tag);
		element.appendChild (arrayElement);
		element = arrayElement;
	    }

	    Map               arrayEditors           = null;
	    DOMConverterUtils arrayDOMConverterUtils = null;
	    PropertyEditor    elementEditor          = PropertyEditorManager.findEditor (desc.getIndexedPropertyType ());
	    for (int i = 0; i < Array.getLength (valueArray); ++i) {
		Object value = Array.get (valueArray, i);
		if (null != value) {
		    // FIXME: singularify: ies -> y, a -> um, ...; let
		    // class config
		    String singularTag = domConverterUtils.getPropertyMap ().getTag (property);
		    if (singularTag.endsWith ("s")) {
			singularTag = singularTag.substring (0, singularTag.length () - 1);
		    }
		    if (dashifyTags) {
			singularTag = StringUtils.caseToggleTo (singularTag, StringUtils.CASE_TOGGLE_TO_LOWER, "-");
		    }
		    
		    if (null != elementEditor) {
			elementEditor.setValue (value);
			DOMUtils.append (element, singularTag, elementEditor.getAsText ());
		    } else {
			// recursion
			BeanToDOM arrayDOM = new BeanToDOM (value, doc, singularTag, groupIndexed, arrayEditors, arrayDOMConverterUtils);
			element.appendChild (arrayDOM.toElement ());
			arrayEditors           = arrayDOM.getEditors ();
			arrayDOMConverterUtils = arrayDOM.getDOMConverterUtils ();
		    }
		}
	    }
	}
    }

    private void appendPlain (Element element, Object bean, PropertyDescriptor desc, Document doc, boolean groupIndexed, PropertyEditor editor) {
	String property = desc.getName ();

	// this is for Model.getModel() and its sub-classes
	if (desc.getPropertyType ().isAssignableFrom (bean.getClass ())) {
	    System.err.println ("property \"" + property + "\" is of same type as (or super-type of) bean ("
				+ bean.getClass ().getName () + "); skipping to avoid endless recursion");
	    return;
	}

	String asText = valueAsText (bean, desc, editor);

 	if (debug) System.err.println (property + " (" + asText + ")");

	if (null != asText) {
	    String tag = domConverterUtils.getPropertyMap ().getTag (property);
	    if (dashifyTags) {
		tag = StringUtils.caseToggleTo (tag, StringUtils.CASE_TOGGLE_TO_LOWER, "-");
	    }
	    DOMUtils.append (element, tag, asText);
	} else {
	    // FIXME: bug in JDK-1.2.2 with overloaded set methods, so we
	    // have to re-create the PropertyDescriptor
	    try {
		desc = new PropertyDescriptor (property, bean.getClass ());
	    } catch (IntrospectionException e) {
// 		e.printStackTrace (System.err);
		return;
	    }

	    // recursion
	    Element child = new BeanToDOM (value (bean, desc), doc, property, groupIndexed).toElement ();
	    if (null != child) {
		element.appendChild (child);
	    }
	}
    }

    //----------------------------------------------------------------------
    private        Object       bean;
    private	   FastBeanInfo bi;
    private        boolean      dashifyTags;
    private static boolean      debug = "true".equals (System.getProperty (BeanToDOM.class.getName () + ".debug"));
}
