package com.livis.flabes.util.beans;

import com.livis.flabes.util.*;
import java.beans.*;
import java.lang.reflect.*;
import java.util.*;

public class BeanUtils {

    private BeanUtils () { }

    public static Object assign (Object lhsBean, Object rhsBean, FastBeanInfo lhsBI, FastBeanInfo rhsBI, Map propertyNames) {
	if (null == rhsBean) {
	    return null;
	}
	Object result = (null == lhsBean) ? ClassUtils.newInstanceCatching (lhsBI.getBeanDescriptor ().getBeanClass ()) : lhsBean;
	for (Iterator i = propertyNames.entrySet ().iterator (); i.hasNext (); ) {
	    Map.Entry entry           = (Map.Entry)i.next (); 
	    String    lhsPropertyName = (String)entry.getKey ();
	    String    rhsPropertyName = (String)entry.getValue ();
	    if (null == rhsPropertyName) {
		rhsPropertyName = lhsPropertyName;
	    }

	    Method setter = lhsBI.getPropertyDescriptor (lhsPropertyName).getWriteMethod ();
	    if (null != setter) {
		Method getter = rhsBI.getPropertyDescriptor (rhsPropertyName).getReadMethod ();
		if (null != getter) {
		    try {
			// MethodUtils.invokeCatching (result, setter, new Object [] { MethodUtils.invokeCatching (rhsBean, getter), });
			
			Object value = getter.invoke (rhsBean, new Object [0]);
			setter.invoke (result, new Object [] { value, });
		    } catch (Exception e) {
			e.printStackTrace (System.err);
			System.err.println ("LHS      " + lhsBean);
			System.err.println ("LHS-PROP " + lhsPropertyName);
			System.err.println ("RHS      " + rhsBean);
			System.err.println ("RHS-PROP " + rhsPropertyName);
			System.err.println ("RESULT   " + rhsBean);
			System.err.println ("GETTER   " + getter);
			System.err.println ("SETTER   " + setter);
		    }
		}
	    }
	}
	return result;
    }

    public static Object clone (Object rhsBean, FastBeanInfo lhsBI, FastBeanInfo rhsBI, Map propertyNames) {
	return assign (null, rhsBean, lhsBI, rhsBI, propertyNames);
    }

    public static Object clone (Object rhsBean, FastBeanInfo bi, Map propertyNames) {
	return clone (rhsBean, bi, bi, propertyNames);
    }

// 	if (null == propertyNames) {
// 	    propertyNames = new LinkedHashMap ();
// 	    propertyNames.keySet ().addAll (lhsBI.getPropertyNameSet ());
// 	}


    public static Object clone (Object rhsBean, BeanInfo bi) {
	if (null == rhsBean) {
	    return null;
	}
	Object               result = ClassUtils.newInstanceCatching (bi.getBeanDescriptor ().getBeanClass ());
	PropertyDescriptor[] pds    = bi.getPropertyDescriptors ();
	for (int i = 0; i < pds.length; ++i) {
	    MethodUtils.invokeCatching (result, pds [i].getWriteMethod (),
					new Object [] { MethodUtils.invokeCatching (rhsBean, pds [i].getReadMethod ()), });
	}
	return result;
    }

    public static Object clone (Object rhsBean) {
	if (null == rhsBean) {
	    return null;
	}
	return clone (rhsBean, new FastBeanInfo (rhsBean.getClass (), true));
    }

    public static PropertyEditor newEditor (PropertyDescriptor pd) {
	PropertyEditor result      = null;
	Class          editorClass = pd.getPropertyEditorClass ();
	Class          type        = pd.getPropertyType ();

	// FIXME: indexed propterties
	if (null != type) {
	    // non-indexed property or indexed property with
	    // non-indexed access
	    if (null == editorClass) {
		result = PropertyEditorManager.findEditor (type);
	    } else {
		try {
		    result = (PropertyEditor)editorClass.newInstance ();
		} catch (Exception e) {
		    e.printStackTrace (System.err);
		}
	    }
	}
	return result;
    }

    public static PropertyEditor newElementEditor (PropertyDescriptor pd) {
	if (!(pd instanceof IndexedPropertyDescriptor)) {
	    return null;
	}

	return PropertyEditorManager.findEditor (((IndexedPropertyDescriptor)pd).getIndexedPropertyType ());
    }

    public static FeatureDescriptor findFeatureDescriptor (FeatureDescriptor[] fds, String name) {
// 	System.out.println ("looking for feature " + name);
	for (int i = 0; i < fds.length; ++i) {
// 	    System.out.println ("trying " + fds [i].getName ());
	    if (name.equals (fds [i].getName ())) {
		return fds [i];
	    }
	}
// 	System.out.println ("not found");
	return null;
    }
    public static PropertyDescriptor findPropertyDescriptor (PropertyDescriptor[] pds, String name) {
	return (PropertyDescriptor)findFeatureDescriptor (pds, name);
    }
    public static EventSetDescriptor findPropertyChanger (EventSetDescriptor[] esds) {
	return (EventSetDescriptor)findFeatureDescriptor (esds, "propertyChange");
    }

    // PropertyChangeSupport.add/removePropertyChangeListener don't
    // check whether the given property name exists
    public static String checkedPropertyName (Object bean, String propertyName) {
	try {
	    return new PropertyDescriptor (propertyName, bean.getClass ()).getName ();
	} catch (IntrospectionException e) {
	    throw new IllegalArgumentException (propertyName + ": no such property (" + e + ")");
	}
    }
}

