package com.livis.flabes.util.beans;

import com.livis.flabes.util.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;

public class DOMConverterUtils {

    public DOMConverterUtils (Object bean) {
	this.bean = bean;

	// only read/write properties are usable in two way conversion
	FastBeanInfo bi = new FastBeanInfo (bean, true);

	// the bean may provide methods returning Lists of XML
	// property names
	Method tagPropertiesGetter       = ClassUtils.getMethodCatching (bean.getClass (), "XMLTagProperties",       (PrintWriter)null);
	Method attributePropertiesGetter = ClassUtils.getMethodCatching (bean.getClass (), "XMLAttributeProperties", (PrintWriter)null);
	Method propertyMapGetter         = ClassUtils.getMethodCatching (bean.getClass (), "XMLpropertyMap",         (PrintWriter)null);

	// get the the list of XML properties from the bean itself; if
	// it doesn't provide the necessary methods, the result are
	// null
	List tagProperties       = (null == tagPropertiesGetter)       ? null : (List)MethodUtils.invokeCatching (bean, tagPropertiesGetter);
	List attributeProperties = (null == attributePropertiesGetter) ? null : (List)MethodUtils.invokeCatching (bean, attributePropertiesGetter);
	propertyMap = (null == propertyMapGetter) ? new PropertyMap () : (PropertyMap)MethodUtils.invokeCatching (bean, propertyMapGetter);

	// create a set version of the provided list to check for
	// duplicates
	Set tagPropertiesSet       = (null == tagProperties)       ? null : new HashSet (tagProperties);
	Set attributePropertiesSet = (null == attributeProperties) ? null : new HashSet (attributeProperties);

	if (debug) {
	    System.err.println ("tag       list provided by " + bean.getClass ().getName () + ": " + tagProperties);
	    System.err.println ("attribute list provided by " + bean.getClass ().getName () + ": " + attributeProperties);
	    System.err.println ("property  map  provided by " + bean.getClass ().getName () + ": " + propertyMap);
	}

	// if the list and the set version of a property collection
	// differ in size, there must be duplicate elements in the
	// list (user provided) version
	{
	    if ((null != tagProperties) && (tagPropertiesSet.size () != tagProperties.size ())) {
		throw new IllegalArgumentException (tagPropertiesGetter + " returns duplicate element: " + tagProperties);
	    }
	    
	    if ((null != attributeProperties) && (attributePropertiesSet.size () != attributeProperties.size ())) {
		throw new IllegalArgumentException (attributePropertiesGetter + " returns duplicate element: " + attributeProperties);
	    }
	}

	// a property must not be in both the attribute and the tag
	// set
	if ((null != tagProperties) && (null != attributeProperties)) {
	    Set intersection = new HashSet (tagPropertiesSet);
	    intersection.retainAll (attributePropertiesSet);
	    if (!intersection.isEmpty ()) {
		throw new IllegalArgumentException (tagPropertiesGetter + " and " + attributePropertiesGetter
						   + " return common elements: " + intersection);
	    }
	}

	// if the list of tag properties is not explicitly provided,
	// it defaults to the complete list of properties minus the
	// attribute properties
	if (null == tagProperties) {
	    // must be wrapped by LinkedList as
	    // Arrays.asList(..).removeAll is not supported
	    tagProperties = new LinkedList (Arrays.asList (bi.getPropertyNames ()));
	    if (null != attributeProperties) {
		tagProperties.removeAll (attributeProperties);
	    }
	}

	if (debug) System.err.println ("tag       list: " + tagProperties);
	if (debug) System.err.println ("attribute list: " + attributeProperties);

	// collect the PropertyDescriptors for attribute and tag
	// properties in two separate arrays
	{
	    tagDescs       = new PropertyDescriptor [(null == tagProperties)       ? 0 : tagProperties.size ()];
	    attributeDescs = new PropertyDescriptor [(null == attributeProperties) ? 0 : attributeProperties.size ()];

	    if (null != tagProperties) {
		int d = 0;
		for (Iterator i = tagProperties.iterator (); i.hasNext (); ++d) {
		    String             property = (String)i.next ();
		    PropertyDescriptor pd       = bi.getPropertyDescriptor (property);

		    // FIXME: was
		    //    new PropertyDescriptor (property, bean.getClass ());
		    // } catch (IntrospectionException ignore) {
		
		    tagDescs [d] = pd;
		}
	    }
	    
	    if (null != attributeProperties) {
		int d = 0;
		for (Iterator i = attributeProperties.iterator (); i.hasNext (); ++d) {
		    String             property = (String)i.next ();
		    PropertyDescriptor pd       = null;
		    
		    try {
			pd = new IndexedPropertyDescriptor (property, bean.getClass ());
		    } catch (IntrospectionException e) {
			if (debug) System.err.println (property + " not indexed: " + e); 
			pd = bi.getPropertyDescriptor (property);
		    }
		    
		    // FIXME: was
		    //     result.add (new IndexedPropertyDescriptor (property, bean.getClass ()));
		    // } catch (IntrospectionException ignore) {
		    //     if (debug) System.err.println (property + " not indexed: " + ignore); 
		    //     try {
		    //         result.add (new PropertyDescriptor (property, bean.getClass ()));
		    //     } catch (IntrospectionException ignoreAgain) {
		    
		    attributeDescs [d] = pd;
		}
	    }
	}
    }

    private PropertyDescriptor[] attributeDescs;
    /* package */ PropertyDescriptor[] getAttributePropertyDescriptors () { return attributeDescs; }
    private PropertyDescriptor[] tagDescs;
    /* package */ PropertyDescriptor[] getTagPropertyDescriptors       () { return tagDescs; }
    private PropertyMap          propertyMap;
    /* package */ PropertyMap          getPropertyMap                  () { return propertyMap; }

    //----------------------------------------------------------------------
    /* package */ static void cont () { if (cont) System.err.print ("."); }

    //----------------------------------------------------------------------
    private 	   Object  bean;
    private static boolean debug = "true".equals (System.getProperty (DOMConverterUtils.class.getName () + ".debug"));
    private static boolean cont  = "true".equals (System.getProperty (DOMConverterUtils.class.getName () + ".cont"));
}
