package com.livis.flabes.util.beans;

import com.livis.flabes.util.*;
import com.livis.flabes.xml.*;
import java.beans.*;
import java.lang.reflect.*;
import java.util.*;
import org.w3c.dom.*;

public class DOMToBean {

    public DOMToBean (Object bean, Element element) {
	if (null == bean) {
	    return;
	}

	this.bean = bean;

	if (bean instanceof DOMable) {
	    ((DOMable)bean).fromElement (element);
	} else {
	    domConverterUtils = new DOMConverterUtils (bean);
	    // only read/write properties have been taken when
	    // contructiong DOM from bean
	    beanInfo          = new FastBeanInfo (bean, true);

	    PropertyDescriptor[] attributeDescs = domConverterUtils.getAttributePropertyDescriptors ();
	    PropertyDescriptor[] tagDescs       = domConverterUtils.getTagPropertyDescriptors ();

	    for (int i = 0; i < attributeDescs.length; ++i) {
		PropertyDescriptor desc = attributeDescs [i];
		String             text = element.getAttribute (desc.getName ());
		if (null != text) {
		    if (debug) System.err.println ("attribute: " + text);
		    PropertyEditor editor = BeanUtils.newEditor (desc);
		    if (null != editor) {
			if (debug) System.err.println ("editor: " + editor);
			editor.setAsText (text);
			Method setter = desc.getWriteMethod ();
			Object value  = editor.getValue ();
			MethodUtils.invokeCatching (bean, setter, new Object [] { value });
			domConverterUtils.cont ();
		    }
		}
	    }

	    NodeList children = element.getChildNodes ();
	    int      nChildren = children.getLength ();

	    Map      iProps = new LinkedHashMap ();

	    if (debug) System.err.println ("top: " + nChildren + " children:");


	    String             oldTag = null;
	    PropertyDescriptor desc   = null;
	    PropertyEditor     editor = null;
	    for (int c = 0; c < nChildren; ++c) {
		Node child = children.item (c);

		if (! (child instanceof Element)) {
		    if (debug) System.err.println ("NOT AN ELEMENT: " + child); 
		    continue;
		}

		NodeList grandChildren  = child.getChildNodes ();
		int      nGrandChildren = grandChildren.getLength ();
		String   tag            = child.getNodeName ();
		String   propertyName   = null;

		if ((null != tag) && !tag.equals (oldTag)) {
		    // tag has changed (ie, not an indexed property)

		    // "-" -> must have been dashified, revert to case
		    // toggle
		    propertyName = (-1 != tag.indexOf ("-")) ? StringUtils.toCaseToggle (tag, "-") : tag;

		    desc = beanInfo.getPropertyDescriptor (propertyName);

		    if (debug) System.err.println ("PROP: " + propertyName);

		    if (null == desc) {
			// pluralize (FIXME) and retry
			propertyName += "s";
			desc = beanInfo.getPropertyDescriptor (propertyName);
		    }
		
		    if (null == desc) {
			System.err.println ("no property descriptior of node <" + child.getNodeName () + "> in " + bean.getClass () + "; skipping");
			continue;
		    }

		    editor = beanInfo.newPropertyEditor (desc.getName ());
		    if (debug) System.err.println ("editor: " + editor);

		    oldTag = tag;
		}

		//  	    System.err.println ("  " + property + ": " + nGrandChildren + " grand children:");
		// 	    for (int gc = 0; gc < nGrandChildren; ++gc) {
		// 		Node grandChild = grandChildren.item (gc);
		//  		System.err.println ("    " + grandChild.getNodeName () + " " + grandChild.getClass () + " inst of Text: " + (grandChild instanceof Text));
		// 	    }

		if ((null != editor) && (1 == nGrandChildren) && (grandChildren.item (0) instanceof Text)) {
		    // System.err.println("EDIT: " + editor);
		    // System.err.println("NODE TYPE: " + grandChildren.item (0));
		    editor.setAsText (grandChildren.item (0).getNodeValue ());
		    Object value = editor.getValue ();
		    Method setter = desc.getWriteMethod ();
		    MethodUtils.invokeCatching (bean, setter, new Object [] { value });
		    domConverterUtils.cont ();
		} else {
		    if (desc instanceof IndexedPropertyDescriptor) {
			// 		    System.err.println ("INDEXED");
			IndexedPropertyDescriptor iDesc = (IndexedPropertyDescriptor)desc;
			Class type = iDesc.getIndexedPropertyType ();

			// 		    editor = PropertyEditorManager.findEditor (type);
			editor = beanInfo.newPropertyElementEditor (desc.getName ());


			if (debug) System.err.println ("editor: " + editor);

			// System.err.println("TEXT: " + grandChildren.item (0).getNodeValue ());

			List iValues = (List)iProps.get (propertyName);
			if (null == iValues) {
			    iValues = new ArrayList ();
			    iProps.put (propertyName, iValues);
			}
		    
			if (null != editor) {
			    System.out.println("editor.setAsText (" + grandChildren.item (0).getNodeValue () + ")");
			    editor.setAsText (grandChildren.item (0).getNodeValue ());
			    Object value = editor.getValue ();
			    iValues.add (value);
			} else {
			    Object newBean = null;
			    try {
			    
				// 			    System.err.println("TYPE: " + type);
			    
				newBean = type.newInstance ();
			    } catch (Exception e) {
				e.printStackTrace ();
			    }
			    // recursion
			    new DOMToBean (newBean, (Element)child);
			    iValues.add (newBean);
			}
		    } else {
			Class type = desc.getPropertyType ();
			Object newBean = null;
			try {
			    newBean = type.newInstance ();
			} catch (Exception e) {
			    e.printStackTrace ();
			}
			// recursion
			new DOMToBean (newBean, (Element)child);
			Method setter = desc.getWriteMethod ();
			MethodUtils.invokeCatching (bean, setter, new Object [] { newBean });
			domConverterUtils.cont ();
		    }
		}

		// 	    if (desc instanceof IndexedPropertyDescriptor) {
		// 		appendIndexed (element, bean, (IndexedPropertyDescriptor)desc, doc);
		// 	    } else {
		// 		appendPlain   (element, bean, desc, doc);
		// 	    }
	    }

	    // 	System.err.println (iProps);
	    for (Iterator i = iProps.entrySet ().iterator (); i.hasNext (); ) {
		Map.Entry                 pair       = (Map.Entry)i.next ();
		String                    property   = (String)pair.getKey ();
		List                      values     = (List)pair.getValue ();
		IndexedPropertyDescriptor iDesc      = (IndexedPropertyDescriptor)beanInfo.getPropertyDescriptor (property);
		Object                    valueArray = Array.newInstance (iDesc.getIndexedPropertyType (), values.size ());
		Method                    setter     = iDesc.getWriteMethod ();

		int idx = 0;
		for (Iterator v = values.iterator (); v.hasNext (); ++idx) {
		    Array.set (valueArray, idx, v.next ());
		}

		MethodUtils.invokeCatching (bean, setter, new Object [] { valueArray });
		domConverterUtils.cont ();
	    }
	}
    }

    //----------------------------------------------------------------------
    private        Object            bean;
    private	   FastBeanInfo      beanInfo;
    private        DOMConverterUtils domConverterUtils;
    private static boolean           debug            = "true".equals (System.getProperty (DOMToBean.class.getName () + ".debug"));
}
