package com.livis.flabes.util.beans;

import java.util.*;

/**
 * Bean property to XML tag mapping.
 **/
public class PropertyMap {

    //----------------------------------------------------------------------
    // constructors
    //----------------------------------------------------------------------
    /**
     * Create bean property to XML tag mapping.
     *
     * @params pairs alternating property name/XML tag sequence.
     **/
    public PropertyMap (String[] pairs) {
	if (null != pairs) {
	    if (0 != (pairs.length % 2)) {
		throw new IllegalArgumentException ("expecting alternating property name/XML tag sequence");
	    }
	    for (int i = 0; i < pairs.length; i += 2) {
		map.put (pairs [i], pairs [i + 1]);
	    }
	}
    }

    /**
     * Create empty bean property to XML tag mapping.
     *
     * {@link #getTag(String) getTag} will always return its input
     * parameter if the mapping is cunstructed this way.
     **/
    public PropertyMap () { this (null); }

    //----------------------------------------------------------------------
    /**
     * Return the XML tag for a given property name.
     *
     * @param  propertyName name of the property.
     * @return XML tag corresponding to <code>propertyName</code>.
     **/
    public String getTag (String propertyName) {
	String result = (String)map.get (propertyName);
	return (null == result) ? propertyName : result;
    }

    //----------------------------------------------------------------------
    // I/O
    //----------------------------------------------------------------------
    public String toString () { return map.toString (); }

    //----------------------------------------------------------------------
    private Map map = new HashMap ();
}
