package com.livis.flabes.util.ejb;

import com.livis.flabes.util.*;
import com.livis.flabes.util.beans.*;
import java.beans.*;
import java.lang.reflect.*;
import java.util.*;

/** Enterprise JavaBean utilities. */
public class EJBUtils {

    private EJBUtils () { }

    /**
     * Returns a list of names of persistent field of a CMP EJB.
     *
     * All public fields of an CMP EJB which also have a public getter
     * and a public setter are assumed to be persistent.
     *
     * @param   ejbClass class of the EJB with CMP to inspect.
     * @returns          list of persistent field names.
     **/
    public static List getCMPfieldNames (Class ejbClass) {
	FastBeanInfo bi            	= new FastBeanInfo (ejbClass, true);
        List         allFieldNames 	= Arrays.asList ((String[])ArraysUtils.extract (ejbClass.getFields (),        "getName"));
        List         allRWPropertyNames = Arrays.asList ((String[])ArraysUtils.extract (bi.getPropertyDescriptors (), "getName"));

	// intersection
        List         result             = new LinkedList (allFieldNames);
	result.retainAll (allRWPropertyNames);

        if (debug) {
	    System.err.println ("EJB CLASS:      " + ejbClass.getName ());
	    System.err.println ("R/W PROPERTIES: " + allRWPropertyNames);
	    System.err.println ("PUBLIC FIELDS:  " + allFieldNames);
	    System.err.println ("CMP FIELDS:     " + result);
	}

	return result;
    }

    //----------------------------------------------------------------------
    private static boolean debug = "true".equals (System.getProperty (EJBUtils.class.getName () + ".debug"));
}
